package controlador;


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.SortedSet;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import datos.EstructuraDAO;

import modelo.Area;
import modelo.Esquema;
import modelo.Estructura;

import utils.ManagerAreas;
import utils.ManagerEstructuras;
import utils.tablemodels.ModeloAreasEstructuras;
import utils.tablemodels.ModeloLocalizarAreas;
import utils.tablemodels.ModeloLocalizarEstructuras;
import utils.tablemodels.ModeloPermisos;
import vista.EditorAreas;

public class EditorAreasController {

	private static EditorAreasController INSTANCE = null;
	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorAreasController();
		}
	}
	public static EditorAreasController getInstance() {
		createInstance();
		return INSTANCE;
	}
	private EditorAreas view;
	private Area area;

	private int tipoOperacion;
	private ManagerAreas managerAreas = ManagerAreas.getInstance();
	private ArrayList<Estructura> tablaIzquierdaImportadas;
	private ArrayList <Estructura> tablaDerechaImportadas;
	private ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();

	public static int NUEVA = 0;

	public static int MODIFICAR = 1;

	// Private constructor suppresses 
	private EditorAreasController() {}

	private void actualizarCampos() {

		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");

		// Nombre
		view.getTextFieldNombre().setText(area.getNombre());

		// Descripcin
		view.getTextAreaDescripcion().setText(area.getDescripcion());

		// Fechas
		view.getTextFieldCreacion().setText(sdf.format(area.getFechaCreacion().getTime()));
		view.getTextFieldFechaModificacion().setText(sdf.format(area.getFechaModificacion().getTime()));

		// Autor

	}

	private void actualizarTablaImportadas() {

		int fila = view.getTableEstructurasArea().getSelectedRow();

		// Modelo

		ModeloAreasEstructuras modelo = new ModeloAreasEstructuras(tablaDerechaImportadas);

		// ActualizarTabla

		view.getTableEstructurasArea().setModel(modelo);

		fila = Math.min(fila, view.getTableEstructurasArea().getRowCount()-1);
		view.getTableEstructurasArea().getSelectionModel().setSelectionInterval(fila, fila);

	}

	private void actualizarTablaNoImportadas() {

		int fila = view.getTableEstructurasNoArea().getSelectedRow();

		// Modelo

		ModeloAreasEstructuras modelo = new ModeloAreasEstructuras(tablaIzquierdaImportadas);

		// ActualizarTabla

		view.getTableEstructurasNoArea().setModel(modelo);

		fila = Math.min(fila, view.getTableEstructurasNoArea().getRowCount()-1);
		view.getTableEstructurasNoArea().getSelectionModel().setSelectionInterval(fila, fila);


	}

	private void actualizarTablaPermisos() {

		ModeloPermisos modelo = new ModeloPermisos(area);

		view.getTablePermisos().setModel(modelo);

		TableColumn columnPermiso = view.getTablePermisos().getColumnModel().getColumn(1);

		// Render necesario para el combo de la tabla
		String [] permisos = {"Sin Acceso", "Lectura", "Escritura"};
		JComboBox comboPermisos = new JComboBox(permisos);
		columnPermiso.setCellEditor(new DefaultCellEditor(comboPermisos));
		DefaultTableCellRenderer renderer =
				new DefaultTableCellRenderer();
		columnPermiso.setCellRenderer(renderer);

	}

	public void actualizarTablasImportadasYPremisos() {
		actualizarTablaImportadas();

		actualizarTablaNoImportadas();

		actualizarTablaPermisos();

	}

	private boolean cambiarNombre() {

		String nuevoNombre = view.getTextFieldNombre().getText();

		if((tipoOperacion==NUEVA)||
				((tipoOperacion==MODIFICAR)&&(!area.getNombre().equals(nuevoNombre)))){


			if(managerAreas.existeNombreArea(nuevoNombre)){
				if((tipoOperacion==MODIFICAR)||((tipoOperacion==NUEVA)&&(!area.getNombre().equals(nuevoNombre)))){
					JOptionPane.showMessageDialog(view,
							"Ya existe un rea con el mismo nombre",
							"Error",
							JOptionPane.ERROR_MESSAGE);
					return false;
				}
			}
			else if(nuevoNombre.trim().equals("")){
				JOptionPane.showMessageDialog(view,
						"El campo de nombre de rea no puede estar vaco.",
						"Error",
						JOptionPane.ERROR_MESSAGE);
				return false;
			}
			else{

				managerAreas.eliminarAreaPorNombre(area.getNombre());

				area.setNombre(nuevoNombre);

				managerAreas.insertarArea(area);

				return true;

			}

		}
		return true;

	}

	public void cerrarVentana(){

		if(cambiarNombre()){
			view.getPreviousWindow().setEnabled(true);
			view.dispose();

			// actualizar tabla ventana anterior

			LocalizadorAreasEsquemasController clae = view.getPreviousWindow().getController().getInstance();
			clae.actualizarTablas();

			// Persistencia
			managerAreas.guardarAreas();
		}
	}

	public void desimportar() {

		int filaSeleccionada = view.getTableEstructurasArea().getSelectedRow();

		if(filaSeleccionada > -1){

			// Buscar el elemento de la tabla
			String nombreEstructura = (String) view.getTableEstructurasArea().getValueAt(filaSeleccionada, 1);

			ManagerEstructuras me = ManagerEstructuras.getInstance();

			Estructura estructuraSeleccionada = me.getEstructurasNombre().get(nombreEstructura);

			// Comprobar si el rea forma parte de algne esquema
			String esquemas = "";

			for(Esquema esq: area.getEsquemas()){
				for(String id: esq.getIdEstructuras()){
					if(estructuraSeleccionada.getId().equals(id)){
						esquemas = esquemas + "\t" + esq.getNombre() + "\n";
					}
				}
			}

			int option = JOptionPane.YES_OPTION;
			if(!esquemas.equals("")){ // Algn esquema usa la estructura
				option = JOptionPane.showConfirmDialog(
						view,
						"La estructura seleccionada est siendo usada\n" +
								"en los siguientes esquemas:\n\n" + esquemas +
								"\nEn caso de continuar, la estructura ser eliminada\n" +
								"de estos esquemas. Desea continuar?",
								"Estructura usada",
								JOptionPane.YES_NO_OPTION);
			}

			if(option == JOptionPane.YES_OPTION){
				// Borrar del conjunto de estructuras importadas
				area.getIdEstructurasImportadas().remove(estructuraSeleccionada.getId());

				// Elementos de las tablas
				tablaDerechaImportadas.remove(estructuraSeleccionada);
				tablaIzquierdaImportadas.add(estructuraSeleccionada);

				// Borrado de las estructuas de los esquemas que la usan

				for(Esquema esq: area.getEsquemas()){
					for(int i = esq.getIdEstructuras().size()-1; i >= 0; i--){	
						if(estructuraSeleccionada.getId().equals(esq.getIdEstructuras().get(i))){
							esq.getIdEstructuras().remove(estructuraSeleccionada.getId());
						}
					}
				}

				// Actualizar Tablas
				actualizarTablasImportadasYPremisos();
			}
		}



	}

	public Area getArea() {
		return area;
	}

	public void habilitarDisparos() {

		// Estructuras propias
		int filaIzquierdaEstructuras = view.getTableEstructurasPropias().getSelectedRow();
		int filaDerechaEstructuras = view.getTableEstructurasOtrasAreas().getSelectedRow();
		int filaDerechaAreas = view.getTableAreas().getSelectedRow();

		if(filaIzquierdaEstructuras > -1 && filaDerechaAreas > -1){
			view.getBtnDerecha().setEnabled(true);
		}
		else{
			view.getBtnDerecha().setEnabled(false);
		}

		if(filaDerechaEstructuras > -1){
			view.getBtnIzquierda().setEnabled(true);
		}
		else{
			view.getBtnIzquierda().setEnabled(false);
		}

		// Estructuras importadas
		int filaTablaIzquierda = view.getTableEstructurasNoArea().getSelectedRow();
		int filaTablaDerecha = view.getTableEstructurasArea().getSelectedRow();

		if(filaTablaIzquierda < 0) 
			view.getDge1button().setEnabled(false);
		else 
			view.getDge1button().setEnabled(true);

		if(filaTablaDerecha < 0) 
			view.getDge2button().setEnabled(false);
		else
			view.getDge2button().setEnabled(true);
	}

	public void importar() {

		int filaSeleccionada = view.getTableEstructurasNoArea().getSelectedRow();

		if(filaSeleccionada > -1){

			// Buscar el elemento de la tabla
			String nombreEstructura = (String) view.getTableEstructurasNoArea().getValueAt(filaSeleccionada, 1);

			ManagerEstructuras me = ManagerEstructuras.getInstance();

			Estructura estructuraSeleccionada = me.getEstructurasNombre().get(nombreEstructura);

			// Aadir al conjunto de estructuras importadas
			area.getIdEstructurasImportadas().add(estructuraSeleccionada.getId());

			// Borrar de tabla izquierda
			tablaIzquierdaImportadas.remove(estructuraSeleccionada);
			tablaDerechaImportadas.add(estructuraSeleccionada);
		}

		// Actualizar Tablas
		actualizarTablasImportadasYPremisos();

	}

	public void inicializar(EditorAreas view, Area area, int tipoOperacion){
		this.view = view;
		this.area = area;
		this.tipoOperacion = tipoOperacion;



		// Carga de tablas estructuras importadas
		actualizarContenidoTablasImportadas();
		// Carga tablas estructuras propias
		//		tablaIzquierdaPropias = new ArrayList<Estructura>();
		//		tablaIzquierdaPropias.addAll(area.getEstructuras());
		//		Collections.sort(tablaIzquierdaPropias);

		actualizarCampos();

		actualizarTablasImportadasYPremisos();
		actualizarTablasEstructurasPropias();
		actualizarRealimentacion();

		habilitarDisparos();
	}
	public void actualizarContenidoTablasImportadas() {
		tablaIzquierdaImportadas = new ArrayList<Estructura>();
		tablaDerechaImportadas = new ArrayList<Estructura>();

		tablaDerechaImportadas = area.obtenerEstructurasImportadas();

		SortedSet<Area> areas = managerAreas.obtenerAreas();

		for(Area a: areas){
			// No es el mismo rea que el que se est modificando
			if(a.getId() != area.getId()){

				for(Estructura e: a.getEstructuras()){

					// Si no est importada...
					if(!tablaDerechaImportadas.contains(e)){
						tablaIzquierdaImportadas.add(e);
					}

				}

			}

		}
	}

	public void setArea(Area area) {
		this.area = area;
	}

	public void actualizarTablasEstructurasPropias() {
		actualizarTablaIzquierda();
		actualizarTablaDerechaAreas();
		actualizarTablaDerechaEstructuras();
	}

	public void actualizarTablaDerechaEstructuras() {

		int fila = view.getTableEstructurasOtrasAreas().getSelectedRow();
		ArrayList<Estructura> estructuras = new ArrayList<Estructura>();

		// Deductivo, porque as es ms fcil
		int filaArea = view.getTableAreas().getSelectedRow();
		if(filaArea > -1){
			String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaArea, 0);
			Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);
			estructuras.addAll(areaSeleccionada.getEstructuras());
		}

		ModeloLocalizarEstructuras modelo = new ModeloLocalizarEstructuras(estructuras);
		view.getTableEstructurasOtrasAreas().setModel(modelo);

		fila = Math.min(fila, view.getTableEstructurasOtrasAreas().getRowCount()-1);

		view.getTableEstructurasOtrasAreas().getSelectionModel().setSelectionInterval(fila, fila);


	}

	private void actualizarTablaDerechaAreas() {

		int fila = view.getTableAreas().getSelectedRow();

		SortedSet<String> areas = managerAreas.listarNombresAreas();
		areas.remove(area.getNombre());

		ModeloLocalizarAreas modelo = new ModeloLocalizarAreas(areas);

		view.getTableAreas().setModel(modelo);

		fila = Math.min(fila, view.getTableAreas().getRowCount()-1);
		view.getTableAreas().getSelectionModel().setSelectionInterval(fila, fila);

	}

	private void actualizarTablaIzquierda() {

		int fila = view.getTableEstructurasPropias().getSelectedRow();

		ModeloLocalizarEstructuras modelo = new ModeloLocalizarEstructuras(area.getEstructuras());

		view.getTableEstructurasPropias().setModel(modelo);

		fila = Math.min(fila, view.getTableEstructurasPropias().getRowCount()-1);
		view.getTableEstructurasPropias().getSelectionModel().setSelectionInterval(fila, fila);
	}


	public void desplazarDerecha(){

		int filaSeleccionada = view.getTableEstructurasPropias().getSelectedRow();
		String estructuraSeleccionada = (String) view.getTableEstructurasPropias().getValueAt(filaSeleccionada, 0);
		Estructura estructura = managerEstructuras.getEstructurasNombre().get(estructuraSeleccionada);

		// Modificar modelo
		int filaArea = view.getTableAreas().getSelectedRow();
		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaArea, 0);
		Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		// Comprobar si la estructura pertenece a algn esquema del rea

		ArrayList<Esquema> esquemas = new ArrayList<Esquema>();
		for(Esquema esq: area.getEsquemas()){
			for(String idEstructura: esq.getIdEstructuras()){
				if(estructura.getId().equals(idEstructura)){
					esquemas.add(esq);
				}
			}
		}

		if(esquemas.size() > 0){

			Object[] options = {"Mantener referencia",
			"Eliminar de esquemas"};

			// Mostrar mensaje
			int opcion = JOptionPane.showOptionDialog(
					view,
					"La estructura seleccionada est includa en " + esquemas.size() + " esquemas del rea " + area.getNombre() +". " +
							"\nDesea mantener una referencia en modo lectura en este rea o eliminarla la estructura de todos los esquemas?",
							"Estructura en uso",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE,
							null,
							options,
							null);

			if(opcion == JOptionPane.YES_OPTION){
				// Mantener referencia en modo lectura
				area.getIdEstructurasImportadas().add(estructura.getId());
			}
			else{
				// Eliminar de esquemas
				for(Esquema esq: esquemas){
					esq.getIdEstructuras().remove(estructura.getId());
				}
			}

		}

		// Estructura
		estructura.setIdArea(areaSeleccionada.getId());
		EstructuraDAO dao = EstructuraDAO.getInstance();
		dao.guardarEstructura(estructura);

		areaSeleccionada.getEstructuras().add(estructura);
		areaSeleccionada.getIdEstructuras().add(estructura.getId());
		area.getEstructuras().remove(estructura);
		area.getIdEstructuras().remove(estructura.getId());

		// Comprobar si esta siendo importada en el rea de destino, y de ser as, eliminarla
		System.out.println("Estructura: " + estructura.getId());
		for(int i = areaSeleccionada.getIdEstructurasImportadas().size()-1; i >= 0; i--){
			if(areaSeleccionada.getIdEstructurasImportadas().get(i).equals(estructura.getId())){
				areaSeleccionada.getIdEstructurasImportadas().remove(i);
				System.out.println("Borro " + i);
			}
		}


		this.actualizarTablaIzquierda();
		this.actualizarTablaDerechaEstructuras();

	}

	public void desplazarIzquierda(){
		// Area 
		int filaArea = view.getTableAreas().getSelectedRow();
		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaArea, 0);
		Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		// Estructura
		int filaEstructura = view.getTableEstructurasOtrasAreas().getSelectedRow();
		String estructuraSeleccionada = (String) view.getTableEstructurasOtrasAreas().getValueAt(filaEstructura, 0);
		Estructura estructura = managerEstructuras.getEstructurasNombre().get(estructuraSeleccionada);

		// UI
		//		tablaIzquierdaPropias.add(estructura);

		// Comprobar si la estructura pertenece a algn esquema del rea

		ArrayList<Esquema> esquemas = new ArrayList<Esquema>();
		for(Esquema esq: areaSeleccionada.getEsquemas()){
			for(String idEstructura: esq.getIdEstructuras()){
				if(estructura.getId().equals(idEstructura)){
					esquemas.add(esq);
				}
			}
		}

		if(esquemas.size() > 0){

			Object[] options = {"Mantener referencia",
			"Eliminar de esquemas"};

			// Mostrar mensaje
			int opcion = JOptionPane.showOptionDialog(
					view,
					"La estructura seleccionada est includa en " + esquemas.size() + " esquemas del rea " + areaSeleccionada.getNombre() +". " +
							"\nDesea mantener una referencia en modo lectura en este rea o eliminarla la estructura de todos los esquemas?",
							"Estructura en uso",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE,
							null,
							options,
							null);

			if(opcion == JOptionPane.YES_OPTION){
				// Mantener referencia en modo lectura
				areaSeleccionada.getIdEstructurasImportadas().add(estructura.getId());
			}
			else{
				// Eliminar de esquemas
				for(Esquema esq: esquemas){
					esq.getIdEstructuras().remove(estructura.getId());
				}
			}

		}



		// Modelo
		area.getEstructuras().add(estructura);
		area.getIdEstructuras().add(estructura.getId());
		areaSeleccionada.getEstructuras().remove(estructura);
		areaSeleccionada.getIdEstructuras().remove(estructura.getId());
		
		// Estructura
		estructura.setIdArea(area.getId());
		EstructuraDAO dao = EstructuraDAO.getInstance();
		dao.guardarEstructura(estructura);
		
		// Comprobar si esta siendo importada en el rea de destino, y de ser as, eliminarla

		for(int i = area.getIdEstructurasImportadas().size()-1; i >= 0; i--){
			if(area.getIdEstructurasImportadas().get(i).equals(estructura.getId())){
				area.getIdEstructurasImportadas().remove(i);

			}
		}

		this.actualizarTablaIzquierda();
		this.actualizarTablaDerechaEstructuras();
	}
	
	public void actualizarRealimentacion(){
		
		view.getLblRealimentacion().setText("Estructuras en " + area.getNombre());
		
		int filaArea = view.getTableAreas().getSelectedRow();
		
		if(filaArea < 0){
			view.getLblRealimentacion2().setText("Seleccione un rea para ver sus estructuras.");
		}
		else{
			String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaArea, 0);
			Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);
			view.getLblRealimentacion2().setText("Estructuras en " + areaSeleccionada.getNombre());
		}
	}

}
