package controlador;

import java.text.SimpleDateFormat;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import modelo.Area;
import modelo.Esquema;
import modelo.Estructura;
import modelo.Presentacion;
import modelo.Visualizacion;
import utils.HTMLUtils;
import utils.MSWordUtils;
import utils.MSWordUtils;
import utils.ManagerAreas;
import utils.ManagerEstructuras;
import utils.ManagerPresentaciones;
import utils.tablemodels.ModeloAreasEstructuras;
import vista.EditorEsquemas;

public class EditorEsquemasController {
	private static EditorEsquemasController INSTANCE = null;
	private EditorEsquemas view;
	private Esquema esquema;
	private int tipoOperacion;
	private Area area;
	private ManagerEstructuras managerEstructuras;
	private ArrayList<Estructura> tablaIzquierda;
	private ArrayList<Estructura> tablaDerecha;

	public static int NUEVA = 0;
	public static int MODIFICAR = 1;

	// Private constructor suppresses 
	private EditorEsquemasController() {}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorEsquemasController();
		}
	}

	public static EditorEsquemasController getInstance() {
		createInstance();
		return INSTANCE;
	}

	public void inicializar(EditorEsquemas view, Esquema esquema, int tipoOperacion){
		this.view = view;
		this.esquema = esquema;
		this.tipoOperacion = tipoOperacion;
		managerEstructuras = ManagerEstructuras.getInstance();

		// area a la cual pertenece el esquema
		ManagerAreas managerAreas = ManagerAreas.getInstance();
		area = managerAreas.obtenerAreaPorID(esquema.getIdArea());

		// Contenido de las tablas

		tablaIzquierda = new ArrayList<Estructura>();
		tablaDerecha = new ArrayList<Estructura>();

		// Tabla izquierda
		// Estructuras propias
		for(Estructura e: area.getEstructuras()){
			tablaIzquierda.add(e);
		}
		// Estructura importadas


		for(String id: area.getIdEstructurasImportadas()){
			Estructura e = managerEstructuras.getEstructurasID().get(id);
			if(e!=null)
				tablaIzquierda.add(e);
		}

		// Tabla derecha
		for(String id: esquema.getIdEstructuras()){
			Estructura e = managerEstructuras.getEstructurasID().get(id);
			if(e!=null){
				tablaDerecha.add(e);
				tablaIzquierda.remove(e); // Las que estan en la tabla derecha no estn en la izquierda
			}
		}


		actualizarCampos();
		actualizarTablas();
		habilitarDisparos();
	}

	private void actualizarTablas() {
		actualizarTablaIzquierda();
		actualizarTablaDerecha();

	}

	private void actualizarTablaDerecha() {
		
		int fila = view.getTableEstructurasVisibles().getSelectedRow();

		ModeloAreasEstructuras modelo = new ModeloAreasEstructuras(tablaDerecha);

		view.getTableEstructurasVisibles().setModel(modelo);
		
		fila = Math.min(fila, view.getTableEstructurasVisibles().getRowCount()-1);
		view.getTableEstructurasVisibles().getSelectionModel().setSelectionInterval(fila, fila);

	}

	private void actualizarTablaIzquierda() {
		
		int fila = view.getTableEstructurasNoVisibles().getSelectedRow();

		ModeloAreasEstructuras modelo = new ModeloAreasEstructuras(tablaIzquierda);

		view.getTableEstructurasNoVisibles().setModel(modelo);
		
		fila = Math.min(fila, view.getTableEstructurasNoVisibles().getRowCount()-1);
		view.getTableEstructurasNoVisibles().getSelectionModel().setSelectionInterval(fila, fila);

	}

	public void cerrarVentana(){
		if(cambiarNombre()){
			view.getPreviousWindow().setEnabled(true);
			view.dispose();

			// actualizar tabla ventana anterior

			LocalizadorAreasEsquemasController clae = view.getPreviousWindow().getController().getInstance();
			clae.actualizarTablas();
		}
	}

	private void actualizarCampos() {

		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");

		// Nombre
		view.getTextFieldNombre().setText(esquema.getNombre());

		// Descripcin
		view.getTextAreaDescripcion().setText(esquema.getDescripcion());

		// Fechas
		view.getTextFieldCreacion().setText(sdf.format(esquema.getFechaCreacion().getTime()));
		view.getTextFieldFechaModificacion().setText(sdf.format(esquema.getFechaModificacion().getTime()));

		// Autor

		// Presentacin
		ManagerPresentaciones mp = ManagerPresentaciones.getInstance();
		Presentacion presentacionEsquema = mp.buscarPorId(esquema.getIdPresentacion());

		// si existe, y sigue siendo publica, se pone esa
		if((presentacionEsquema != null )&&(mp.getPresentacionesPublicas().contains(presentacionEsquema))){
			view.getComboBoxPresentacion().setSelectedItem(presentacionEsquema);
		}
		else{
			// si no, se pone la default y se actualiza la del esquema
			view.getComboBoxPresentacion().setSelectedItem(mp.getPresentacionDefault());
			esquema.setIdPresentacion(mp.getPresentacionDefault().getId());
		}

	}

	private boolean cambiarNombre() {

		String nuevoNombre = view.getTextFieldNombre().getText();

		if((tipoOperacion==NUEVA)||
				((tipoOperacion==MODIFICAR)&&(!esquema.getNombre().equals(nuevoNombre)))){


			if(area.listarNombresEsquemas().contains(nuevoNombre)){
				if((tipoOperacion==MODIFICAR)||((tipoOperacion==NUEVA)&&(!esquema.getNombre().equals(nuevoNombre)))){
					JOptionPane.showMessageDialog(view,
							"Ya existe un esquema con el mismo nombre para el rea " + area.getNombre() + ".",
							"Error",
							JOptionPane.ERROR_MESSAGE);
					return false;
				}
			}
			else if(nuevoNombre.trim().equals("")){
				JOptionPane.showMessageDialog(view,
						"El campo de nombre de esquema no puede estar vaco.",
						"Error",
						JOptionPane.ERROR_MESSAGE);
				return false;
			}
			else{
				esquema.setNombre(nuevoNombre);

				return true;

			}

		}
		return true;

	}

	public void excluirEstructura() {
		// Seleccionar estructura
		int filaSeleccionada = view.getTableEstructurasVisibles().getSelectedRow();
		String nombreEstructura = (String) view.getTableEstructurasVisibles().getValueAt(filaSeleccionada, 1);

		Estructura estructura = managerEstructuras.getEstructurasNombre().get(nombreEstructura);

		// Eliminar del esquema
		esquema.getIdEstructuras().remove(estructura.getId());
		tablaDerecha.remove(estructura);
		tablaIzquierda.add(estructura);

		//ActualizarTablas
		actualizarTablas();

	}

	public void incluirEstructura() {

		// Seleccionar estructura
		int filaSeleccionada = view.getTableEstructurasNoVisibles().getSelectedRow();
		String nombreEstructura = (String) view.getTableEstructurasNoVisibles().getValueAt(filaSeleccionada, 1);

		Estructura estructura = managerEstructuras.getEstructurasNombre().get(nombreEstructura);

		// Aadir al esquema
		esquema.getIdEstructuras().add(estructura.getId());
		tablaIzquierda.remove(estructura);
		tablaDerecha.add(estructura);

		//ActualizarTablas
		actualizarTablas();
	}

	public Esquema getEsquema() {
		return esquema;
	}

	public void setEsquema(Esquema esquema) {
		this.esquema = esquema;
	}

	public void habilitarDisparos() {

		int filaTablaIzquierda = view.getTableEstructurasNoVisibles().getSelectedRow();
		int filaTablaDerecha = view.getTableEstructurasVisibles().getSelectedRow();

		if(filaTablaIzquierda < 0) 
			view.getDge1button().setEnabled(false);
		else 
			view.getDge1button().setEnabled(true);

		if(filaTablaDerecha < 0) 
			view.getDge2button().setEnabled(false);
		else
			view.getDge2button().setEnabled(true);
	}

	public void generarHTML() {
		HTMLUtils.generarEsquemaHTML(esquema);

	}

	public void cambiarPresentacion() {

		Presentacion presentacionSeleccionada = (Presentacion) view.getComboBoxPresentacion().getSelectedItem();
		esquema.setIdPresentacion(presentacionSeleccionada.getId());

	}

	public void generarWord() {
		MSWordUtils word = new MSWordUtils(esquema);
		word.generarMSWordEsquema();

	}

	public EditorEsquemas getView() {
		return view;
	}

	public void setView(EditorEsquemas view) {
		this.view = view;
	}

	
	public void validar() {
// Busqueda de presentacion
		ManagerPresentaciones mp = ManagerPresentaciones.getInstance();
		Presentacion presentacion = mp.buscarPorId(esquema.getIdPresentacion());
		
		if(presentacion == null || presentacion.getTipo() == Presentacion.PRIVADA){
			presentacion = mp.getPresentacionDefault();
		}
		
		String informe = "";
		
		// Validacin de estructuras propias e importadas
		for(String id: esquema.getIdEstructuras()){
			Estructura e = managerEstructuras.getEstructurasID().get(id);
			if(e!=null){
				informe += e.validar(presentacion);
			}
		}
		
		if(informe.equals("")){
			JOptionPane.showMessageDialog(view,
					"El equema validado es correcto. No se han detectado errores.",
					"Error de validacin",
					JOptionPane.INFORMATION_MESSAGE);
		}
		else{
			StringBuffer subinforme = new StringBuffer();
			subinforme.append("El visualizacion validado no es correcto. Se han encontrado los siguientes errores: \n\n");
			if(informe.length() > 500){
				subinforme.append(informe.substring(0, 499));
				subinforme.append("...\n...");


			}
			else{
				subinforme.append(informe);
			}
			JOptionPane.showMessageDialog(view,
					subinforme.toString(),
					"Error de validacin",
					JOptionPane.ERROR_MESSAGE);

		}

	}

}
