package controlador;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.StringTokenizer;

import javax.swing.ListSelectionModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import controlador.undo.AgregarEstructuraDerivadaCommand;
import controlador.undo.EliminarComponenteCommand;
import controlador.undo.ModificarEstructuraDerivadaCommand;
import controlador.undo.ModificarNombreComponenteCommand;

import modelo.Area;
import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.SelectorLinea;
import utils.ColaComandos;
import utils.ManagerAreas;
import utils.ManagerEstructuras;
import utils.tablemodels.ModeloEditorEstructuraDerivada;
import utils.tablemodels.ModeloEditorEstructuras;
import vista.EditorEstructuraDerivada;
import vista.EditorSelector;

// TODO: Auto-generated Javadoc
/**
 * The Class EditorEstructuraDerivadaController.
 * Controlador del editor de estructura derivada
 */
public class EditorEstructuraDerivadaController {

	/** Interfaz de usuario. */
	private EditorEstructuraDerivada view;

	/** Estructura abierta en la ventana principal del editor. */
	private Estructura estructuraAbierta = null;

	/** Estructura derivada que se est modificando. */
	private ComponenteCompuesta estructuraDerivada = null;

	/** Estructura derivada que se pasa como parmetro en el caso de que se vaya a realizar una
	 * modificacin y no la creacin de una nueva. */
	private ComponenteCompuesta estructuraDerivadaOriginal = null;

	/** INSTANCE. */
	private static EditorEstructuraDerivadaController INSTANCE = null;

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	/**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorEstructuraDerivadaController();
		}
	}

	/**
	 * Gets the single instance of EditorEstructuraDerivadaController.
	 *
	 * @return single instance of EditorEstructuraDerivadaController
	 */
	public static EditorEstructuraDerivadaController getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}

	/** Es una nueva estructura derivada o se est modificando una estructura ya existente. */
	private Boolean esNuevaEstructura = true;

	/** reas. */
	//private ArrayList<Area> areas;

	/** Nmero de modificaciones desde que se ha abierto la ventana del editor. */
	private int numeroModificaciones;

	/**
	 * Inicializa un objeto de tipo editor estructura derivada controller.
	 */
	private EditorEstructuraDerivadaController() {
	}

	/**
	 * Abrir editor selector.
	 */
	public void abrirEditorSelector() {
		EditorSelector es = new EditorSelector(view);
		es.setVisible(true);
		es.setAlwaysOnTop(true);
		es.setEnabled(true);
		// Desactivacin del JFrame actual
		view.setEnabled(false);

	}


	/**
	 * Actualizacin del rbol de la parte izquierda de la interfaz de usuario (LAEC).
	 */
	public void actualizarArbolLAEC(){
		//		
		// Recorrer carpetas en busca de estructuras
		DefaultMutableTreeNode areas = new DefaultMutableTreeNode("Areas");

		// Aadir rea provisional y visualizacion provisional
		ManagerAreas managerAreas = ManagerAreas.getInstance();

		SortedSet<Area> areasObjetos = managerAreas.obtenerAreas();

		for(Area a: areasObjetos){

			DefaultMutableTreeNode area = new DefaultMutableTreeNode(a);


			for(Estructura e: a.getEstructuras()){
				// Todas menos la estructura abierta en el editor
				if(!e.getId().equals(estructuraAbierta.getId())){
					area.add(e.getRaiz().obtenerSubArbol());
				}
			}

			areas.add(area);
		}

		// Actualizar rbol
		DefaultTreeModel modelo = new DefaultTreeModel(areas);
		view.getLaecTree().setModel(modelo);

		// Expandir todos los nodos

		for (int i = 0; i < view.getLaecTree().getRowCount(); i++) {
			view.getLaecTree().expandRow(i);
		}

	}

	/**
	 * Actualizar habilitacion disparos.
	 */
	public void actualizarHabilitacionDisparos() {


		// El botn de eliminar componente est siempre activado, a menos que no haya fila seleccionada
		// o esta pertenezca a la componente base
		int filaSeleccionada = view.getEedTable().getSelectedRow();


		if(filaSeleccionada == -1){
			view.getDeed1button().setEnabled(false);
		}
		else{
			Linea l = estructuraDerivada.getLineas().get(filaSeleccionada);

			// Si se trata de un selector, no es posible quitar lineas
			if(l.getClass() == SelectorLinea.class){
				view.getDeed1button().setEnabled(false);
				view.getDeed3button().setEnabled(true);
			}
			else{ // Si se trata de una componente...
				ComponenteLinea cl = (ComponenteLinea) l;
				Componente componenteSeleccionada = cl.getComponente();

				// Si no tiene parent, es que es la componetne base
				if(componenteSeleccionada.getParent() == null){
					view.getDeed1button().setEnabled(false);
					view.getDeed3button().setEnabled(false);
				}
				else{
					view.getDeed1button().setEnabled(true);
					view.getDeed3button().setEnabled(true);
				}
			}
		}

		// Insertar ED disponible si se ha realizado al menos una operacin
		if(numeroModificaciones > 0)
			view.getDeed4button().setEnabled(true);

	}

	/**
	 * Actualizar tabla.
	 */
	public void actualizarTabla() {
		// Guardar fila seleccionada
		int selectedRow = view.getEedTable().getSelectedRow();

		ModeloEditorEstructuraDerivada modelo = new ModeloEditorEstructuraDerivada(estructuraDerivada);
		view.getEedTable().setModel(modelo);
		// Restaurar fila seleccionada
		ListSelectionModel selectionModel = 
				view.getEedTable().getSelectionModel();

		// Control para no salirse de la tabla (por ejemplo, al borrar una componente compuesta estando en el ltimo lugar de la estructura, y seleccionando la lnea final de smbolos)
		if(view.getEedTable().getRowCount() < selectedRow) 
			selectedRow = view.getEedTable().getRowCount()-1;

		//System.out.println("Selected row: " + selectedRow);
		// Restarurar el modelo de la tabla
		selectionModel.setSelectionInterval(selectedRow, selectedRow);
		view.getEedTable().setSelectionModel(selectionModel);

	}

	/**
	 * Agregar componente a la estructura derivada.
	 *
	 * @param selPath camino de la componente o estructura dentro del workspace.
	 */
	public void agregarComponente(TreePath selPath){

		// Si el camino seleccionado tiene menos de tres items, es incorrecto
		if(selPath.getPathCount() < 3) return;

		// Aumentar contador de modificaciones
		numeroModificaciones++;

		// Bsqueda de al componente seleccionada
		Componente componenteSeleccionada = buscarComponente(selPath);

		System.out.println("--Componete Seleccionada: " + componenteSeleccionada.getNombre());

		// Bsqueda de la raiz de la componente seleccionada

		Componente parentComponenteSeleccionada = componenteSeleccionada;
		Componente parentComponenteSeleccionadaDerivado;
		while(parentComponenteSeleccionada.getParent() != null){
			parentComponenteSeleccionada = parentComponenteSeleccionada.getParent();
		}

		// Comprobar si la raz est ya en la estructura derivada
		Componente ultimaCompoEnCamino = null;
		Boolean flagContenida = false;
		for(int i = 0; i < estructuraDerivada.getComponentes().size(); i++){
			if(estructuraDerivada.getComponentes().get(i).getIdComponenteOrigen().equals(parentComponenteSeleccionada.getId())){
				flagContenida = true;
				ultimaCompoEnCamino = estructuraDerivada.getComponentes().get(i);
			}
		}


		if(flagContenida == true){
			ultimaCompoEnCamino = buscarUltimaComponenteEnCamino(selPath, ultimaCompoEnCamino);

			/* Si la componente seleccionada es la ltima componente del camino, 
				significa que se est intentando volver a agregar otra vez la misma
				subcomponente. Esta accin no es posible
			 */

			if(!ultimaCompoEnCamino.getIdComponenteOrigen().equals(componenteSeleccionada.getId())){

				// Derivar el resto del camino
				Componente derivacion = derivarCaminoDesdeHasta(componenteSeleccionada, ultimaCompoEnCamino);

				System.out.println("Ultima componente del camino: " + derivacion.getNombre());

				derivacion.setParent((ComponenteCompuesta) ultimaCompoEnCamino);
				derivacion.setEstructura(estructuraAbierta);

				// Insertar el camino creado en el camino ya existente
				int numElementosUltimaCompoEnCamino = ((ComponenteCompuesta)ultimaCompoEnCamino).getComponentes().size();

				((ComponenteCompuesta)ultimaCompoEnCamino).agregarComponente(derivacion, numElementosUltimaCompoEnCamino);
			}


		}
		else{ // Es la primera componente elemental de esa componente derivada
			parentComponenteSeleccionadaDerivado = derivarCaminoHastaComponente(componenteSeleccionada);
			parentComponenteSeleccionadaDerivado.setParent(estructuraDerivada);
			parentComponenteSeleccionadaDerivado.setEstructura(estructuraAbierta);

			// Nmero de componentes actualmente
			int numElementosEstructuraDerivada = estructuraDerivada.getComponentes().size();

			// Insercin el la ltima posicin
			estructuraDerivada.agregarComponente(parentComponenteSeleccionadaDerivado, numElementosEstructuraDerivada);
		}


		// Actualizacin de la interfaz
		actualizarHabilitacionDisparos();
		actualizarTabla();


	}

	/**
	 * Buscar componente a partir de su ruta.
	 *
	 * @param selPath ruta dentro del workspace
	 * @return the la componente correspondiente a la ruta seleccionada
	 */
	public Componente buscarComponente(TreePath selPath) {
		DefaultMutableTreeNode node =  (DefaultMutableTreeNode) selPath.getLastPathComponent();
		return (Componente) node.getUserObject();
	}

	/**
	 * Buscar ultima componente en camino.
	 *
	 * @param selPath la ruta de la componente
	 * @param ultimaCompoEnCamino la componente que se est buscando
	 * @return la componente original.
	 */
	private Componente buscarUltimaComponenteEnCamino(TreePath selPath, Componente ultimaCompoEnCamino) {
		Boolean flag = false;
		// Empieza a buscar a partir del elemento 4 del camino : [Areas, AreaX, EstructuraX, Subcomponentes...]
		for(int i = 3; i < selPath.getPathCount()&&!flag; i++){ // recorre las componentes del camino
			flag = true;
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) selPath.getPathComponent(i);
			Componente pathComponente = (Componente) node.getUserObject();
			if(ultimaCompoEnCamino.getClass() == ComponenteCompuesta.class){
				ComponenteCompuesta aux = (ComponenteCompuesta) ultimaCompoEnCamino;
				for(int j = 0; j < aux.getComponentes().size()&&flag; j++){ // recorre las subcomponentes de la ltima componente en el camino

					if(aux.getComponentes().get(j).getIdComponenteOrigen().equals(pathComponente.getId())){
						// Se ha encontrado la componente en el camino y actualiza la ltima componente en el camino
						ultimaCompoEnCamino = aux.getComponentes().get(j);
						flag = false;
					}
				}
				// Si en este punto flag == true, es que no se ha encontrado la componente
			}		
		}
		return ultimaCompoEnCamino;
	}

	/**
	 * Cerrar ventana.
	 */
	public void cerrarVentana() {
		/* Si se ha estado editando una estructura derivada existente previamente, sta
		 * se actualiza posteriormente en la ventana anterior.
		 */
		EditorEstructurasController eec = EditorEstructurasController.getInstance();
		if(esNuevaEstructura == false){
			if(numeroModificaciones > 0){
				eec.getView().getDecp11button().setEnabled(true);
				eec.getView().getDecp12button().setEnabled(false);
				eec.actualizarHabilitacionDisparos();
				eec.actualizarTabla();
			}
		}
		else{ // es nueva estructura
			eec.actualizarTabla();
			eec.actualizarHabilitacionDisparos();
			eec.desplazarCeldaSeleccionada(1,0);
		}

		view.getPreviousWindow().setEnabled(true);
		view.dispose();

	}

	/**
	 * Realiza una derivacin desde la componente "desde" hasta la componente "hasta"
	 *
	 * @param desde componente inicio de la derivacin
	 * @param hasta componente fin de la derivacin
	 * @return componente derivada
	 */
	private Componente derivarCaminoDesdeHasta(Componente desde, Componente hasta){
		/*
		 * IMPORTANTE, LA PRIMERA COMPONENTE NO ES DERIVADA Y LA SEGUNDA SI
		 */
		Componente raizDerivada;
		raizDerivada = (Componente) desde.derivar();

		while(!desde.getParent().getId().equals(hasta.getIdComponenteOrigen())){
			desde = desde.getParent();

			// Derivacin del parent
			ComponenteCompuesta componenteAuxiliar = (ComponenteCompuesta) desde.derivar();

			// Vaciar todas las subcomponentes y conservar nicamente la subcomponente seleccionada
			componenteAuxiliar.getComponentes().clear();
			componenteAuxiliar.getComponentes().add(raizDerivada);
			raizDerivada = componenteAuxiliar;
		}

		return raizDerivada;
	}

	/**
	 * Derivar el caminio desde la componente compleja base hasta la componente pasada por parmetro
	 *
	 * @param componenteSeleccionada componente hasta la cual se realiza la derivacin
	 * @return componente derivada
	 */
	private Componente derivarCaminoHastaComponente(
			Componente componenteSeleccionada) {

		Componente raizDerivada;
		raizDerivada = (Componente) componenteSeleccionada.derivar();

		while(componenteSeleccionada.getParent() != null){
			componenteSeleccionada = componenteSeleccionada.getParent();

			// Derivacin del parent
			ComponenteCompuesta componenteAuxiliar = (ComponenteCompuesta) componenteSeleccionada.derivar();

			// Vaciar todas las subcomponentes y conservar nicamente la subcomponente seleccionada
			componenteAuxiliar.getComponentes().clear();
			componenteAuxiliar.getComponentes().add(raizDerivada);
			raizDerivada = componenteAuxiliar;
		}

		return raizDerivada;
	}

	/**
	 * Eliminar componente de la estructura derivada
	 */
	public void eliminarComponente() {
		int filaSeleccionada = view.getEedTable().getSelectedRow();

		numeroModificaciones++;

		if(filaSeleccionada < 0) return; // no hay fila seleccionada

		ComponenteLinea cl = (ComponenteLinea) estructuraDerivada.getLineas().get(filaSeleccionada);

		// No es posible eliminar la componente base de la estructura
		if(cl.getComponente().getParent()==null)
			return;

		int posicion = cl.getComponente().getParent().getPosicionComponente(cl.getComponente().getId());

		// Creacin del command

		cl.getComponente().getParent().eliminarComponente(posicion);

		actualizarHabilitacionDisparos();
		actualizarTabla();

	}

	/**
	 * Recupera estructura derivada.
	 *
	 * @return el estructura derivada
	 */
	public ComponenteCompuesta getEstructuraDerivada() {
		return estructuraDerivada;
	}

	/**
	 * Recupera view.
	 *
	 * @return el view
	 */
	public EditorEstructuraDerivada getView() {
		return view;
	}

	/**
	 * Inicializar.
	 *
	 * @param view the view
	 */
	public void inicializar(EditorEstructuraDerivada view){
		setView(view);

		EditorEstructurasController eecontroller = EditorEstructurasController.getInstance();

		// Estructura abierta en la ventana principal del editor
		estructuraAbierta = eecontroller.getEsquema().getEstructura();

		// Modificar el label correspondiente a la realimentacion del a estructura

		view.getEstructuraLabel().setText("Estructura: " + estructuraAbierta.getNombre());

		// Numero de modificaciones
		numeroModificaciones = 0;

		// Inicializar el rbol de LAEC

		actualizarArbolLAEC();

		/* En el caso de que en la otra ventana haya abierta una estructura derivada, se abre sta.
		 * En caso contrario, se crea una nueva estructura derivada 
		 */
		int lineaSeleccionada = eecontroller.getView().getEcpTable().getSelectedRow();
		Componente componenteSeleccionada = ((ComponenteLinea) estructuraAbierta.getAalc().get(lineaSeleccionada)).getComponente();
		if(componenteSeleccionada.getDerivada() == true){
			// Bsqueda de la componente compuesta base de la estructura derivada
			while(componenteSeleccionada.getParent().getDerivada()==true)
				componenteSeleccionada = componenteSeleccionada.getParent();
			// Asignacin a la estructura derivada actual

			estructuraDerivadaOriginal = (ComponenteCompuesta) componenteSeleccionada;

			esNuevaEstructura = false;

			// Clonacin por si hay que restaurar

			try {
				estructuraDerivada = (ComponenteCompuesta) estructuraDerivadaOriginal.clone();
			} catch (CloneNotSupportedException e) {}
			estructuraDerivada.actualizarSimbolos();
		}
		else{
			// Creacin de la estructura derivada
			nuevaEstructuraDerivada();
		}

		// ActualizarTabla

		actualizarTabla();

	}

	/**
	 * Insertar estructura derivada dentro de la estructura abierta en la ventana principal del editor.
	 */
	public void insertarEstructuraDerivada() {
		EditorEstructurasController eecontroller = EditorEstructurasController.getInstance();
		/* En el caso de que se edite una estructura derivada ya existente, no hace falta insertarla,
		 * ya que los cambios se vern reflejados en la primera ventana
		 */
		if(esNuevaEstructura)
			insertarNuevaEstructuraDerivada();
		else{
			if(numeroModificaciones > 0)
				modificarEstructuraDerivada();
			else return;
		}

		eecontroller.actualizarTabla();
	}


	/**
	 * Insertar nueva estructura derivada.
	 */
	public void insertarNuevaEstructuraDerivada(){

		// Obtener controlador ventana anterior
		EditorEstructurasController eecontroller = EditorEstructurasController.getInstance();

		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = eecontroller.getView().getEcpTable().getSelectedRow();

		if(fila < 0) return ;

		ComponenteLinea cl = (ComponenteLinea) eecontroller.getEsquema().getEstructura().getAalc().get(fila);

		// Creacin del command
		AgregarEstructuraDerivadaCommand command = new AgregarEstructuraDerivadaCommand();
		command.setEstructuraDerivada(estructuraDerivada);

		// Si la linea seleccionada es una componente elemental, insercin a continuacin de la misma

		if(cl.getComponente().getClass() == ComponenteElemental.class){
			int posicion = cl.getComponente().getParent().getPosicionComponente(cl.getComponente().getId());
			command.setPosicion(posicion+1);
			command.setComponente(cl.getComponente().getParent());
			//cl.getComponente().getParent().agregarComponente(posicion+1);
		}
		else{ // si la lnea seleccionada pertenece a una componente compleja
			ComponenteCompuesta cc = (ComponenteCompuesta) cl.getComponente();

			// si la linea seleccionada es la cabecera o la lnea inicial de smbolos, insertar dentro de la cc compleja
			if((cc.getCabecera().getId().equals(cl.getId())) || (cc.getSimbolosInicio().getId().equals(cl.getId()))){
				command.setPosicion(0);
				command.setComponente(cc);
			}

			// si la linea seleccionada es la de los smbolos finales, insertar a continuacin de la cc compleja, excepto si es la cc base
			else {
				if(cc.getParent()!= null){ // no es cc base
					int posicion = cl.getComponente().getParent().getPosicionComponente(cl.getComponente().getId());
					//cl.getComponente().getParent().agregarComponente(posicion+1);
					command.setComponente(cl.getComponente().getParent());
					command.setPosicion(posicion+1);
				}
			}
		}
		// ejecucin del command
		command.ejecutar();

		// Pilas undo/redo
		ColaComandos comandosEjecutados = ColaComandos.getInstance();
		comandosEjecutados.agregarComando(command);

	}


	/**
	 * Modificar estructura derivada.
	 */
	private void modificarEstructuraDerivada() {

		// Buscar la posicion de la estructuraDerivada en su parent
		ComponenteCompuesta parent = estructuraDerivadaOriginal.getParent();
		int posicion = parent.getPosicionComponente(estructuraDerivadaOriginal.getId());

		// Creacin del command

		ModificarEstructuraDerivadaCommand command = new ModificarEstructuraDerivadaCommand();
		command.setPosicion(posicion);
		//command.setEstructuraDerivadaOriginal(estructuraDerivadaOriginal);
		command.setEstructuraDerivadaModificada(estructuraDerivada);
		command.setComponente(parent);

		command.ejecutar();

		// Insercin en la pila undo

		ColaComandos comandosEjecutados = ColaComandos.getInstance();
		comandosEjecutados.agregarComando(command);

	}

	/**
	 * Nueva estructura derivada.
	 */
	private void nuevaEstructuraDerivada() {
		estructuraDerivada = new ComponenteCompuesta("EstructuraDerivada", null, null);
		estructuraDerivada.setAnidamiento(0);
		estructuraDerivada.setDerivada(true);
		estructuraDerivada.setIdComponenteOrigen("");
		estructuraDerivada.setIdEstructuraOrigen("");
		esNuevaEstructura = true;
	}


	/**
	 * Establece estructura derivada.
	 *
	 * @param estructuraDerivada el nuevo estructura derivada
	 */
	public void setEstructuraDerivada(ComponenteCompuesta estructuraDerivada) {
		this.estructuraDerivada = estructuraDerivada;
	}

	/**
	 * Establece view.
	 *
	 * @param view el nuevo view
	 */
	public void setView(EditorEstructuraDerivada view) {
		this.view = view;
	}




}
