package controlador;


import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import modelo.Area;
import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Esquema;
import modelo.Visualizacion;
import modelo.Estructura;
import modelo.Linea;
import modelo.Presentacion;
import modelo.Propiedad;
import modelo.SelectorLinea;
import modelo.Usuario;
import utils.ColaComandos;
import utils.HTMLUtils;
import utils.HTMLUtils;
import utils.MSWordUtils;
import utils.MSWordUtils;
import utils.ManagerAreas;
import utils.ManagerEstructuras;
import utils.ManagerPresentaciones;
import utils.ManagerUsuarios;
import utils.renders.EditorEstructurasCellRender;
import utils.tablemodels.ModeloEditorEstructuras;
import utils.tablemodels.RXTable;
import vista.EditorDescripcionEstructura;
import vista.EditorEstructuraDerivada;
import vista.EditorEstructuras;
import vista.EditorTipoPresentacion;
import vista.EditorGrupos;
import vista.GestionUsuarios;
import vista.LocalizadorAreasEsquemas;
import vista.NuevoUsuario;
import controlador.undo.AgregarComponenteCommand;
import controlador.undo.DesplazarHaciaAbajoCommand;
import controlador.undo.DesplazarHaciaArribaCommand;
import controlador.undo.DesplazarHaciaDerechaCommand;
import controlador.undo.DesplazarHaciaIzquierdaCommand;
import controlador.undo.EliminarComponenteCommand;
import controlador.undo.ModificarClaseCommand;
import controlador.undo.ModificarIteracionCommand;
import controlador.undo.ModificarNombreComponenteCommand;
import controlador.undo.ModificarNombrePropiedadCommand;
import controlador.undo.ModificarOpcionalidadCommand;
import controlador.undo.ModificarTipoCommand;



// TODO: Auto-generated Javadoc
/**
 * EditorEstructurasController es el controlador relativo al entorno editorial EE1: EditorEstructuras.
 * Sigue el patrn Singleton, de forma que en todo momento exista nicamente una instancia de si mismo.
 */
public class EditorEstructurasController {

	/** La instancia del controlador. (Patrn singleton) */
	private static EditorEstructurasController INSTANCE = null;

	// Patrn singleton
	/**
	 * Creacin de la instancia.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorEstructurasController();
		}
	}

	/**
	 * Recupera la instancia nica de la clase.
	 *
	 * @return nica instancia de EditorEstructurasController
	 */
	public static EditorEstructurasController getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}

	/** La vista sobre la cual actua el controlador. */
	private EditorEstructuras view = null;

	/** El visualizacion que se muestra en un momento dado en el editor. Definido por una presentacin y una estructura. */
	private Visualizacion visualizacion;

	/** Fila seleccionada prviamente a la actual en la tabla ECP. Se utiliza para no realizar operaciones innecearias de habilitacion de botones */
	private int ultimaFilaSeleccionada = -1;

	/** Pila en la cual se van apilando las acciones editoriales ejecutadas . Se utiliza un arrayDeque por si en un futuro se quiere limitar el tamao mximo de operaciones para deshacer *. */
	private ColaComandos comandosEjecutados = ColaComandos.getInstance();

	/** Numero de modificaciones hechas en el editor desde la ltima apertura. */
	private int numeroModificaciones = 0;
	/**
	 * Constructor vaco y privado exigido por el patrn Singleton.
	 */

	private EditorEstructurasController(){
	}

	/**
	 * Se ofrece al usuario el guardado de la estructura actual mediante un cuadro de dilogo.
	 *
	 * @return the opcin elegida por el usuario en el dilogo de alerta
	 */
	public int abrirDialogoAlertaGuardado() {

		int valorRetorno = JOptionPane.CANCEL_OPTION;

		if(numeroModificaciones != 0){ // NO se abre el dilogo si no se ha modificado la estructura
			if(visualizacion.getEstructura() != null){
				String mensaje = "Desea guardar previamente " + visualizacion.getEstructura().getNombre() + "?";
				String titulo = "Guardar cambios";
				valorRetorno = JOptionPane.showOptionDialog(view.getFrmEditorDeEstructuras(), mensaje, titulo, JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, null, titulo);
				// Se guarda la estructura
				if(valorRetorno == JOptionPane.OK_OPTION){
					guardarEstructura();
				}

			}		
		}
		return valorRetorno;
	}

	public void abrirEditorAreas() {
		LocalizadorAreasEsquemas lae = new LocalizadorAreasEsquemas(view.getFrmEditorDeEstructuras());
		lae.setVisible(true);
		lae.setAlwaysOnTop(true);
		lae.setEnabled(true);

		// Desactivacin del JFrame actual
		view.getFrmEditorDeEstructuras().setEnabled(false);

	}

	// Aadir estructura derivada
	/**
	 * Abrir el segundo formulario del entorno editorial: editor estructura derivada (EE1.F2).
	 */
	public void abrirEditorEstructuraDerivada(){

		EditorEstructuraDerivada eed = new EditorEstructuraDerivada(view.getFrmEditorDeEstructuras());
		eed.setVisible(true);
		eed.setAlwaysOnTop(true);
		eed.setEnabled(true);

		// Desactivacin del JFrame actual
		view.getFrmEditorDeEstructuras().setEnabled(false);


	}

	/**
	 * Abre la ventana de edicin de presentacioens
	 */
	public void abrirEditorPresentaciones(){
		EditorTipoPresentacion et = new EditorTipoPresentacion(view.getFrmEditorDeEstructuras());
		et.setVisible(true);
		et.setAlwaysOnTop(true);
		et.setEnabled(true);

		// Desactivacin del JFrame actual
		view.getFrmEditorDeEstructuras().setEnabled(false);
	}

	/**
	 * Abrir estructura contenida en el fichero xml pasado como argumento
	 *
	 * @param fileEstructura fichero en el cual se encuentra la estructura que se quiere cargar
	 */
	public void abrirEstructura(String nombre){

		// Manager
		ManagerEstructuras manager = ManagerEstructuras.getInstance();
		Estructura estructura = manager.abrirEstructura(nombre);

		// Aadir al visualizacion
		visualizacion.setEstructura(estructura);
		// Actualizar tabla

		actualizarTabla();

		// Reinicializar atributos
		deshabilitarFilaSeleccionada();
		ultimaFilaSeleccionada = -1;
		comandosEjecutados.clear();
		actualizarHabilitacionDisparos();
		numeroModificaciones = 0;

		// Realimentacin
		actualizarRealimentacion();
		
		// Hacer visible editor
		view.getEcpScrollPane().setVisible(true);

	}

	/**
	 * Abrir estructura desde arbol LAE de la parte izquierda de la pantalla tras un doble click.
	 *
	 * @param selPath camino desde el elemento seleccionado hasta la raiz, para reconstruir el path del fichero
	 */
	public void abrirEstructuraDesdeArbol(TreePath selPath) {

		// Recuperar el nombre de la estructura
		String nombre = selPath.getLastPathComponent().toString();

		if(numeroModificaciones == 0)
			abrirEstructura(nombre);
		else{
			if(abrirDialogoAlertaGuardado() != JOptionPane.CANCEL_OPTION)
				abrirEstructura(nombre);
		}

	}

	/**
	 * Abrir la ventana de gestin de grupos
	 */
	public void abrirGestionGrupos(){
		EditorGrupos gu = new EditorGrupos(view.getFrmEditorDeEstructuras());

		gu.setVisible(true);
		gu.setAlwaysOnTop(true);
		gu.setEnabled(true);
		view.getFrmEditorDeEstructuras().setEnabled(false);

	}


	/**
	 * Abre la ventana de gestin de usuarios
	 */
	public void abrirGestionUsuarios(){
		GestionUsuarios gu = new GestionUsuarios(view.getFrmEditorDeEstructuras());

		gu.setVisible(true);
		gu.setAlwaysOnTop(true);
		gu.setEnabled(true);
		view.getFrmEditorDeEstructuras().setEnabled(false);

	}


	/**
	 * Actualizar el localizador de reas y estructuras
	 */
	public void actualizarArbolLAE(){
		// Recorrer carpetas en busca de estructuras
		DefaultMutableTreeNode areas = new DefaultMutableTreeNode("Areas");

		// Aadir rea provisional y visualizacion provisional
		ManagerAreas managerAreas = ManagerAreas.getInstance();

		SortedSet<Area> areasObjetos = managerAreas.obtenerAreas();

		for(Area a: areasObjetos){

			DefaultMutableTreeNode area = new DefaultMutableTreeNode(a);

			for(Estructura e: a.getEstructuras()){
				DefaultMutableTreeNode estructura = new DefaultMutableTreeNode(e);
				area.add(estructura);
			}

			areas.add(area);
		}

		// Actualizar rbol
		DefaultTreeModel modelo = new DefaultTreeModel(areas);
		view.getLaeTree().setModel(modelo);

		// Expandir todos los nodos

		for (int i = 0; i < view.getLaeTree().getRowCount(); i++) {
			view.getLaeTree().expandRow(i);
		}


	}


	/**
	 * Activacin y desactivacin de los botones dependiendo del tipo de componente actualmente seleccionado
	 */
	public void actualizarHabilitacionDisparos() {
		// Fila seleccionada
		int filaSeleccionada = view.getEcpTable().getSelectedRow();
		//System.out.println("Fila seleccionada: " + filaSeleccionada);


		/*
		 * Los siguiente botones estn deshabilitados nicamente al principio, cuando no hay
		 * una estructura cargada.
		 */
		view.getDlae03button().setEnabled(true);
		view.getDlae04button().setEnabled(true);
		view.getDlae05button().setEnabled(true);
		view.getDlae06button().setEnabled(true);
		view.getDecp14button().setEnabled(true);
		view.getDlae08button().setEnabled(true);
		// Seleccin de presentacin unicamente disabled al principio, cuando no hay estructura seleccioanda
		view.getLpComboBox().setEnabled(true);

		if(filaSeleccionada == -1){
			desactivarOperacionesEditoriales();
			if(!comandosEjecutados.esPosibleDeshacer()) view.getDecp11button().setEnabled(false);
			if(!comandosEjecutados.esPosibleRehacer()) view.getDecp12button().setEnabled(false);
		}
		// Se comprueba si es la misma que la anterior para evitar hacer operaciones innecesarias
		else if(filaSeleccionada != ultimaFilaSeleccionada){


			// Recupera la lnea
			Linea l = visualizacion.getEstructura().getAalc().get(filaSeleccionada);
			ComponenteLinea cl;

			if(l.getClass() == SelectorLinea.class){
				desactivarOperacionesEditoriales();
				return;
			}
			else{
				cl = (ComponenteLinea) l;
			}

			// Independientes del tipo de componente, pero con casos especiales
			view.getDecp05button().setEnabled(true);
			view.getDecp06button().setEnabled(true); // aadirComponente
			view.getDecp07button().setEnabled(true); // quitarComponente
			view.getDecp03button().setEnabled(true); // cambiarClase
			view.getDecp10button().setEnabled(true); // agregar estructura derivada

			// DECP08.1: Desplazar hacia arriba
			if(cl.getComponente().getParent() == null){
				// No es posible desplazar hacia arriba la componente base
				view.getDecp081button().setEnabled(false);
			}
			else{
				// Es posible desplazar hacia arriba la componente, a menos que sea la primera
				if(cl.getComponente().getParent().getComponentes().get(0).hashCode() == cl.getComponente().hashCode())
					view.getDecp081button().setEnabled(false);
				else
					view.getDecp081button().setEnabled(true);
			}

			// DECP08.2: Desplazar hacia abajo
			if(cl.getComponente().getParent() == null){
				// No es posible desplazar hacia abajo la componente base
				view.getDecp082button().setEnabled(false);
			}
			else{
				// Es posible desplazar hacia abajo la componente, a menos que sea la ltima
				int ultimo = cl.getComponente().getParent().getComponentes().size() -1;
				if(cl.getComponente().getParent().getComponentes().get(ultimo).hashCode() == cl.getComponente().hashCode())
					view.getDecp082button().setEnabled(false);
				else
					view.getDecp082button().setEnabled(true);
			}

			// DECP09.1: Desplazar hacia la derecha

			if(cl.getComponente().getParent() == null){
				// No es posible mover a la derecha la componente base
				view.getDecp091button().setEnabled(false);
			}
			else  {

				// en primer lugar se deshabilita el botn, y posteriormente se comprueba si es posible habilitarlo
				view.getDecp091button().setEnabled(false);
				// En el caso de que sea una componente compuesta, nicamente es posible si tiene una componente compuesta al lado
				Componente componente = cl.getComponente();
				ComponenteCompuesta parent = cl.getComponente().getParent();
				int posicion = parent.getPosicionComponente(componente.getId());

				// arriba
				if(posicion-1 >= 0){
					view.getDecp091button().setEnabled(true);
				}
				// abajo
				if(posicion+1 < parent.getComponentes().size()){
					if(parent.getComponentes().get(posicion+1).getClass() == ComponenteCompuesta.class)
						view.getDecp091button().setEnabled(true);
				}


			}

			// DECP09.2: Desplazar hacia la izquierda

			if(cl.getComponente().getParent() == null)
				view.getDecp092button().setEnabled(false); // no es posible desplazar a la izquierda la componeente compuesta base
			else if (cl.getComponente().getParent().getParent() == null)
				view.getDecp092button().setEnabled(false); // no es posible desplazar a la izquierda cualquier subcomponente de la componente compuesta base
			else
				view.getDecp092button().setEnabled(true);  // en cualquier otro caso es posible
			// Dependientes del tipo de componente
			if(cl.getComponente().getClass() == ComponenteElemental.class){
				view.getDecp02button().setEnabled(false); // modificarTipo
				view.getDecp04button().setEnabled(false); // modificarIterador
			}
			else{
				view.getDecp02button().setEnabled(true); // modificarTipo
				view.getDecp04button().setEnabled(true); // modificarIterador

				/* Casos especiales */

				ComponenteCompuesta cc = (ComponenteCompuesta) cl.getComponente();
				// 1. Si la lnea pertenece a la lnea de smbolos finales de la componente compuesta base, no se puede aadir una componente o una estructra derivada
				if(cl.getComponente().getParent() == null){
					if(cl.hashCode() == cc.getSimbolosFin().hashCode()){
						view.getDecp06button().setEnabled(false);
						view.getDecp10button().setEnabled(false);
					}
				}
				// 2. No es posible borrar la componente compuesta base
				if(cl.getComponente().getParent() == null)
					view.getDecp07button().setEnabled(false);

				// 3. No es posible realizar un cambio de clase de la componente compuesta base
				if(cl.getComponente().getParent() == null)
					view.getDecp03button().setEnabled(false);
			}

			// Componentes derivadas
			if(cl.getComponente().getDerivada() == true){
				// Insercin deshabilitada

				view.getDecp06button().setEnabled(false);
				// Cambio clase deshabilitado
				view.getDecp02button().setEnabled(false);
				// Cambio de tipo deshabilitado
				view.getDecp03button().setEnabled(false);
				// Mover a derecha o izquierda deshabilitado
				view.getDecp091button().setEnabled(false);
				view.getDecp092button().setEnabled(false);

			}
		}
		ultimaFilaSeleccionada = filaSeleccionada;

	}

	/**
	 * El listener de la UI avisa al controlador de que se ha producido un cambio en el contenido de una celda.
	 * Este cambio puede ser el valor de una propieda, o el valor de una componente. 
	 * El este ltimo caso hace falta un redibujado de los smbolos para que stos sean coherentes.
	 *
	 * @param value el nuevo valor de la celda
	 * @param componente es la fila a la cual pertenece la celda modificada
	 * @param col es la columna a la cual pertenece la celda modificada
	 */
	public void actualizarLinea(String value, int componente, int col) {

		System.out.println("----------" + componente);

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(componente);

		System.out.println("----------" + cl.getContenido());
		System.out.println("----------" + value);

		if(col==0){ // Se actualiza la componente
			// Parsing
			value = value.replace('<', ' ');
			value = value.replace('>', ' ');
			value = value.replace('+', ' ');
			value = value.replace('[', ' ');
			value = value.replace(']', ' ');
			value = value.replace('|', ' ');
			value = value.replace('=', ' ');
			value = value.replace('{', ' ');
			value = value.replace('}', ' ');
			value = value.trim();

			// Sustitucin del valor

			// Creacin del command
			ModificarNombreComponenteCommand command = new ModificarNombreComponenteCommand();
			command.setComponente(cl.getComponente());
			command.setNuevoValor(value);

			// Ejecucin

			command.ejecutar();

			// Pilas undo/redo
			comandosEjecutados.agregarComando(command);

			//			// Numero Modificaciones
			//			numeroModificaciones++;
			//
			//			// Undo/Redo
			//			view.getDecp11button().setEnabled(true);
			//			view.getDecp12button().setEnabled(false);
			//
			//			// Actualizar Tabla
			//			actualizarTabla();

		}
		else{ 
			// Localizacin de la propiedad que se va a actualizar
			Presentacion presentacionActual = visualizacion.getPresentacion();
			Propiedad propiedadModificada = null;

			int contador = 0;
			for(Propiedad p: presentacionActual.getPropiedades()){
				if(p.getVisible() == true){
					contador++;
					if(contador == col){
						propiedadModificada = p;
					}
				}
			}

			for(int i = 0; i < cl.getComponente().getPropiedades().size(); i++){
				if(cl.getComponente().getPropiedades().get(i).getNombre().equals(propiedadModificada.getNombre())){ // Bsqueda de la propiedad por alias
					ModificarNombrePropiedadCommand command2 = new ModificarNombrePropiedadCommand();
					command2.setPropiedad(cl.getComponente().getPropiedades().get(i));
					command2.setNuevoValor(value);

					// Ejecucin

					command2.ejecutar();

					// Pilas undo/redo
					comandosEjecutados.agregarComando(command2);
				}

			}	

		}

		// Numero Modificaciones
		numeroModificaciones++;

		// Undo/Redo
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar Tabla
		actualizarTabla();

	}

	/**
	 * Actualizar la lista de presentaciones que se muestran en el comboBox de la interfaz
	 */
	public void actualizarListaPresentaciones(){

		// Recuperar nombres de presentaciones
		Vector<String> nombres = new Vector<String>();

		ManagerPresentaciones manager = ManagerPresentaciones.getInstance();

		nombres.add(manager.getPresentacionDefault().getNombre());

		for(Presentacion p: manager.getPresentacionesPublicas()){
			if(!nombres.contains(p.getNombre())) // para no incluir la default dos veces
				nombres.add(p.getNombre());
		}

		for(Presentacion p: manager.getPresentacionesPrivadas()){
			nombres.add(p.getNombre());
		}
		// Crear modelo
		DefaultComboBoxModel cbm = new DefaultComboBoxModel(nombres);

		// Actualizar modelo
		view.getLpComboBox().setModel(cbm);
		if(view.getLpComboBox().isEnabled())
			view.getLpComboBox().setSelectedItem(visualizacion.getPresentacion().getNombre());
		else{
			// Al cambiar la presentacin por defecto, cambiar tambien la estructura
			// Asociada al visualizacion
			visualizacion.setPresentacion(manager.getPresentacionDefault());
		}

	}

	public void actualizarRealimentacion(){
		Estructura e = visualizacion.getEstructura();
		if(e!=null){
			// Usuario
			if(e.getIdUsuarioCreacion() != null){
				ManagerUsuarios manager = ManagerUsuarios.getInstance();
				if(manager.existeIDUsuario(e.getIdUsuarioCreacion())){
					Usuario u = manager.obtenerUsuarioPorID(e.getIdUsuarioCreacion());
					view.getAutortextField().setText(u.getNombreUsuario());
				}
				else{
					view.getAutortextField().setText("");
				}
			}
			else{
				view.getAutortextField().setText("");
			}

			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			// Creacin
			if(e.getCreacion()!=null){
				view.getCreaciontextField().setText(sdf.format(e.getCreacion().getTime()));
			}
			else{
				view.getCreaciontextField().setText("");
			}

			// Modificacin
			if(e.getModificacion()!=null){
				view.getModificaciontextField().setText(sdf.format(e.getModificacion().getTime()));
			}
			else{
				view.getModificaciontextField().setText("");
			}
		}
		else{
			view.getAutortextField().setText("");
			view.getCreaciontextField().setText("");
			view.getModificaciontextField().setText("");
		}
	}

	/**
	 * Actualizar la tabla ECP del editor de estructuras. Actualiza la tabla creando un modelo nuevo a partir de la presentacin y la estructura que contiene el visualizacion.
	 */
	public void actualizarTabla(){
		Presentacion p = visualizacion.getPresentacion();
		Estructura e = visualizacion.getEstructura();
		e.actualizarLineas();
		ModeloEditorEstructuras modelo = new ModeloEditorEstructuras(p, e);

		// Guardar fila seleccionada
		int selectedRow = view.getEcpTable().getSelectedRow();
		int selectedColumn = view.getEcpTable().getSelectedColumn();

		view.getEcpTable().setModel(modelo);

		// Restaurar fila seleccionada
		ListSelectionModel selectionModel = 
				view.getEcpTable().getSelectionModel();
		TableColumnModel columnModel = view.getEcpTable().getColumnModel();

		// Control para no salirse de la tabla (por ejemplo, al borrar una componente compuesta estando en el ltimo lugar de la estructura, y seleccionando la lnea final de smbolos)
		if(view.getEcpTable().getRowCount() < selectedRow) 
			selectedRow = view.getEcpTable().getRowCount()-1;

		// Restarurar el modelo de la tabla
		selectionModel.setSelectionInterval(selectedRow, selectedRow);
		columnModel.getSelectionModel().setSelectionInterval(selectedColumn, selectedColumn);
		view.getEcpTable().setSelectionModel(selectionModel);
		view.getEcpTable().setColumnModel(columnModel);

		// Render
		EditorEstructurasCellRender cr = new EditorEstructurasCellRender(visualizacion);
		view.getEcpTable().setDefaultRenderer(Object.class, cr);
	}


	/**
	 * Agregar una nueva componente elementa a la estructura.
	 * Si no hay linea seleccionada -> Accin deshabilitada
	 * Si la linea seleccionada pertenece a una componente elemental -> Inserta a continuacin
	 * Si la lnea seleccionada es la cabecera o los simbolos iniciales de una componente compuesta -> Inserta dentro de la estructura
	 * Si la lnea seleccionada es la de smbolos finales de una componente compuesta -> Inserta a continuacin de la componente compuesta (a menos que se trate de la componente compuesta base, en cuyo caso la accin est deshabilitada)
	 */
	public void agregarComponente(){
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return ;

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		// Creacin del command
		AgregarComponenteCommand command = new AgregarComponenteCommand();

		// Si la linea seleccionada es una componente elemental, insercin a continuacin de la misma

		if(cl.getComponente().getClass() == ComponenteElemental.class){
			int posicion = cl.getComponente().getParent().getPosicionComponente(cl.getComponente().getId());
			command.setPosicion(posicion+1);
			command.setComponente(cl.getComponente().getParent());
			//cl.getComponente().getParent().agregarComponente(posicion+1);
		}
		else{ // si la lnea seleccionada pertenece a una componente compleja
			ComponenteCompuesta cc = (ComponenteCompuesta) cl.getComponente();

			// si la linea seleccionada es la cabecera o la lnea inicial de smbolos, insertar dentro de la cc compleja
			if((cc.getCabecera().hashCode() == cl.hashCode()) || (cc.getSimbolosInicio().hashCode() == cl.hashCode())){
				command.setPosicion(0);
				command.setComponente(cc);
			}

			// si la linea seleccionada es la de los smbolos finales, insertar a continuacin de la cc compleja, excepto si es la cc base
			else {
				if(cc.getParent()!= null){ // no es cc base
					int posicion = cl.getComponente().getParent().getPosicionComponente(cl.getComponente().getId());
					//cl.getComponente().getParent().agregarComponente(posicion+1);
					command.setComponente(cl.getComponente().getParent());
					command.setPosicion(posicion+1);
				}
			}
		}
		// ejecucin del command
		command.ejecutar();

		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();
		desplazarCeldaSeleccionada(1,0);

	}



	/**
	 * Cambiar presentacion.
	 */
	public void cambiarPresentacion() {
		/* Modificar, codigo de muestra antes de hacer la gestin de presentaciones */
		JComboBox cb = view.getLpComboBox();
		String selectedItem = (String) cb.getSelectedItem();

		// Busqueda de la presentacion
		ManagerPresentaciones manager = ManagerPresentaciones.getInstance();
		Presentacion presentacion = manager.buscarPorNombre(selectedItem);

		// Actualizar visualizacion
		visualizacion.setPresentacion(presentacion);


		actualizarTabla();
	}

	/**
	 * Desactiva los botones pertenecientes a las operaciones editoriales de la estructura
	 */
	public void desactivarOperacionesEditoriales() {
		view.getDecp02button().setEnabled(false);
		view.getDecp03button().setEnabled(false);
		view.getDecp04button().setEnabled(false);
		view.getDecp05button().setEnabled(false);
		view.getDecp06button().setEnabled(false);
		view.getDecp07button().setEnabled(false);
		view.getDecp081button().setEnabled(false);
		view.getDecp082button().setEnabled(false);
		view.getDecp091button().setEnabled(false);
		view.getDecp092button().setEnabled(false);
		view.getDecp10button().setEnabled(false);

	}

	/**
	 * Deshabilitar fila seleccionada.
	 */
	public void deshabilitarFilaSeleccionada(){
		ListSelectionModel sm = view.getEcpTable().getSelectionModel();	
		sm.removeSelectionInterval(sm.getMinSelectionIndex(), sm.getMaxSelectionIndex());
		view.getEcpTable().setSelectionModel(sm);

		actualizarHabilitacionDisparos();
	}

	/**
	 * Operacin deshacer.
	 */
	public void deshacer(){
		comandosEjecutados.deshacer();

		actualizarTabla();

		// desactivacin del botn undo
		if(!comandosEjecutados.esPosibleDeshacer()) view.getDecp11button().setEnabled(false);

		// activacin de la pila redo
		view.getDecp12button().setEnabled(true);

		deshabilitarFilaSeleccionada();

		// Actualizar modificaciones
		numeroModificaciones--;
	}

	/**
	 * Desplazar el cursor de seleccion un nmero de posiciones determinado por el parmetro de entrada
	 *
	 * @param desplazamiento numero de posiciones que se desplaza el cursor
	 */
	public void desplazarCeldaSeleccionada(int desplazamientoFila, int desplazamientoColumna){
		// Guardar fila seleccionada
		int selectedRow = view.getEcpTable().getSelectedRow()+desplazamientoFila;
		int selectedColumn = view.getEcpTable().getSelectedColumn()+desplazamientoColumna;

		// Restaurar fila seleccionada
		ListSelectionModel selectionModel = 
				view.getEcpTable().getSelectionModel();
		TableColumnModel columnModel = view.getEcpTable().getColumnModel();

		// Control para no salirse de la tabla (por ejemplo, al borrar una componente compuesta estando en el ltimo lugar de la estructura, y seleccionando la lnea final de smbolos)
		if(view.getEcpTable().getRowCount() < selectedRow) 
			selectedRow = view.getEcpTable().getRowCount()-1;
		if(0 > selectedRow) 
			selectedRow = 0;

		if(view.getEcpTable().getColumnCount() < selectedColumn) 
			selectedColumn = view.getEcpTable().getColumnCount()-1;
		if(0 > selectedColumn) 
			selectedColumn = 0;

		// Restarurar el modelo de la tabla
		selectionModel.setSelectionInterval(selectedRow, selectedRow);
		columnModel.getSelectionModel().setSelectionInterval(selectedColumn, selectedColumn);
		view.getEcpTable().setSelectionModel(selectionModel);
		view.getEcpTable().setColumnModel(columnModel);	
		actualizarHabilitacionDisparos();
	}

	/**
	 * Desplazar hacia abajo una posicin la componente a la cual pertenece la lnea seleccionada, en el caso de que sea posible.
	 */
	public void desplazarHaciaAbajo() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		if(cl.getComponente().getParent() == null)
			return;

		// Creacin del command
		DesplazarHaciaAbajoCommand command = new DesplazarHaciaAbajoCommand();
		command.setComponente(cl.getComponente());
		command.ejecutar();
		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();
		desplazarCeldaSeleccionada(1,0);

	}

	/**
	 * Desplazar hacia arriba una posicin la componente a la cual pertenece la lnea seleccionada, en el caso de que sea posible.
	 */
	public void desplazarHaciaArriba() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		if(cl.getComponente().getParent() == null)
			return;

		// Creacin del command
		DesplazarHaciaArribaCommand command = new DesplazarHaciaArribaCommand();
		command.setComponente(cl.getComponente());
		command.ejecutar();
		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();
		desplazarCeldaSeleccionada(-1,0);

	}

	/**
	 * Desplazar hacia la izquierda (a un nivel de anidamiento superior) la componente a la cual pertenece la lnea seleccionada, en el caso de que sea posible.
	 */
	public void desplazarHaciaDerecha() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		// No es posible mover a la derecha la componente compleja base
		if(cl.getComponente().getParent() == null)
			return;

		// Creacin del command
		DesplazarHaciaDerechaCommand command = new DesplazarHaciaDerechaCommand();
		command.setComponente(cl.getComponente());
		command.ejecutar();
		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();


		switch(command.getTipoDesplazamiento()){
		case 1:
			desplazarCeldaSeleccionada(-1,0);
			break;
		case 2:
			desplazarCeldaSeleccionada(2,0);
			break;
		case 3:
			desplazarCeldaSeleccionada(2,0);
			break;
		}


	}

	/**
	 * Desplazar hacia la izquierda (a un nivel de anidamiento inferior) la componente a la cual pertenece la lnea seleccionada, en el caso de que sea posible.
	 */
	public void desplazarHaciaIzquierda() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		// No es posible mover a la izquierda la componente compleja base
		if(cl.getComponente().getParent() == null)
			return;

		// No es posible mover a la izquierda cualquier subcomponente cuyo parent sea la componente compleja base
		if(cl.getComponente().getParent().getParent() == null)
			return;

		// Creacin del command
		DesplazarHaciaIzquierdaCommand command = new DesplazarHaciaIzquierdaCommand();
		command.setComponente(cl.getComponente());
		command.ejecutar();

		// Actualizar modificaciones
		numeroModificaciones++;

		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		actualizarTabla();

	}

	/**
	 * Eliminar una componente de la estructura.
	 * Si es una componente compuesta, las subcomponentes que la forman pasan al nivel de anidamiento inmediatamente inferior.
	 * No es posible borrar la componente compuesta base.
	 */
	public void eliminarComponente() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		// No es posible eliminar la componente base de la estructura
		if(cl.getComponente().getParent()==null)
			return;

		int posicion = cl.getComponente().getParent().getPosicionComponente(cl.getComponente().getId());

		// Creacin del command

		EliminarComponenteCommand command = new EliminarComponenteCommand();
		command.setComponente(cl.getComponente().getParent());
		command.setPosicion(posicion);

		command.ejecutar();

		// Actualizar modificaciones
		numeroModificaciones++;

		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);


		actualizarTabla();
	}

	/**
	 * Genera un documento html a partir de la tabla del editor.
	 */
	public void generarHTML(){ 

		HTMLUtils.generarHTMLEstructura(visualizacion, view.getEcpTable());
	}

	/**
	 * Genera un documento Word a partir de la tabla del editor
	 */
	public void generarWord() {
		MSWordUtils word = new MSWordUtils(visualizacion, view.getEcpTable());
		word.generarMSWordEstructura();
	}

	/**
	 * Recupera visualizacion.
	 *
	 * @return el visualizacion
	 */
	public Visualizacion getEsquema() {
		return visualizacion;
	}

	/**
	 * Gets the view.
	 *
	 * @return the view
	 */
	public EditorEstructuras getView() {
		return view;
	}

	/**
	 * Guardar estructura
	 */
	public void guardarEstructura(){

		int valorRetorno = JOptionPane.OK_OPTION;
		ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();
		ManagerAreas managerAreas = ManagerAreas.getInstance();


		// 1 Comprobar que existen reas en el sistema
		SortedSet<String> areas = managerAreas.listarNombresAreas();
		if(areas.size() == 0){
			JOptionPane.showMessageDialog(view.getFrmEditorDeEstructuras(),
					"No existe ningn rea. Cree un rea antes de guardar la estructura.",
					"No existen reas",
					JOptionPane.WARNING_MESSAGE);
			return;
		}

		// 2 si es estructura modificada y no ha cambiado el nombre, solo se guarda

		if((managerEstructuras.getTipo() == ManagerEstructuras.MODIFICAR) 
				&& (visualizacion.getEstructura().getNombre().equals(managerEstructuras.getCopia().getNombre()))){

			managerEstructuras.guardar(visualizacion.getEstructura());
			numeroModificaciones = 0;
			return;
		}

		// 3 si el nombre de la estructura nueva, o de la vieja modificada coincide con otra
		// Se pregunta si se quiere sobreescribir

		if((managerEstructuras.getTipo() == ManagerEstructuras.NUEVA)||
				(((managerEstructuras.getTipo() == ManagerEstructuras.MODIFICAR) 
						&& (!visualizacion.getEstructura().getNombre().equals(managerEstructuras.getCopia().getNombre()))))){

			if(managerEstructuras.getEstructurasNombre().containsKey(visualizacion.getNombre())){
				String mensaje = "Se va a sobreescribir la estructura " + visualizacion.getEstructura().getNombre() + ". Desea continuar?";
				String titulo = "Sobreescribir estructura";
				valorRetorno = JOptionPane.showOptionDialog(view.getFrmEditorDeEstructuras(), mensaje, titulo, JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, null, titulo);

				// Si el usuario cancela, se cancela la operacin
				if(valorRetorno == JOptionPane.NO_OPTION)
					return;
				else{
					// En caso contrario, se elimina la estructura existente con ese nombre
					Estructura e = managerEstructuras.getEstructurasNombre().get(visualizacion.getEstructura().getNombre());
					managerEstructuras.getEstructurasNombre().remove(e.getNombre());
					managerEstructuras.getEstructurasID().remove(e.getId());
				}
			}

		}

		String seleccion = (String) JOptionPane.showInputDialog(
				view.getFrmEditorDeEstructuras(),
				"Seleccione opcion",
				"Selector de opciones",
				JOptionPane.QUESTION_MESSAGE,
				null,  // null para icono defecto
				areas.toArray(), 
				"opcion 1");

		if(seleccion != null){

			// REFERENCIAS
			Area area = managerAreas.obtenerAreaPorNombre(seleccion);
			Estructura estructura = visualizacion.getEstructura();
			estructura.setIdArea(area.getId());
			area.getEstructuras().add(estructura);
			for(Estructura e: area.getEstructuras()){
				System.out.println("++" + e.hashCode());
			}
			area.getIdEstructuras().add(estructura.getId());

			// Guardado
			managerEstructuras.guardar(estructura);
			managerAreas.guardarAreas();

			// Despus de guardar se actualiza el rbol

			actualizarArbolLAE();

			// Actualizar numero modificaciones
			actualizarTabla();
			numeroModificaciones = 0;
		}
		
	}

	// Patrn singleton
	/**
	 * Test run.
	 */
	public void inicializar() {

		visualizacion = new Visualizacion();

		/*
		 * Para la presentacin mostrada inicialmente, se selecciona la presentacin default del manager
		 */

		ManagerPresentaciones manager = ManagerPresentaciones.getInstance();
		Presentacion pdefecto = manager.getPresentacionDefault();
		visualizacion.setPresentacion(pdefecto);

		// Inicializaciones
		actualizarArbolLAE();
		actualizarListaPresentaciones();
	}


	/**
	 * Transformacin de una componente elemental en una componente compleja, y viceversa.
	 */
	public void modificarClase(){
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		// No es posible modificar la clase de la componente base de la estructura
		if(cl.getComponente().getParent()==null)
			return;

		// Creacin del command
		ModificarClaseCommand command = new ModificarClaseCommand();
		command.setComponente(cl.getComponente());
		command.ejecutar();

		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();
	}

	/**
	 * Abre la ventana de edicin de descripcin
	 */
	public void modificarDescripcion() { EditorDescripcionEstructura ede = new EditorDescripcionEstructura(view); ede.setVisible(true);
	ede.setAlwaysOnTop(true);
	ede.setEnabled(true);

	// Desactivacin del JFrame actual
	view.getFrmEditorDeEstructuras().setEnabled(false);

	}

	/**
	 * Modificar iteracion.
	 */
	public void modificarIteracion() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		if(cl.getComponente().getClass() == ComponenteCompuesta.class){ // operacin nicamente vlida para una componente compuesta
			// Creacin del command
			ModificarIteracionCommand command = new ModificarIteracionCommand();
			command.setComponente((ComponenteCompuesta) cl.getComponente());
			command.ejecutar();
			// Pilas undo/redo
			comandosEjecutados.agregarComando(command);
			view.getDecp11button().setEnabled(true);
			view.getDecp12button().setEnabled(false);

			// Actualizar modificaciones
			numeroModificaciones++;

			actualizarTabla();
		}

	}


	/**
	 * Modificar opcionalidad.
	 */
	public void modificarOpcionalidad() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		// Creacin del command
		ModificarOpcionalidadCommand command = new ModificarOpcionalidadCommand();
		command.setComponente(cl.getComponente());
		command.ejecutar();
		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();
	}

	/**
	 * modificar una agregacin por una especializacin, y viceversa.
	 */
	public void modificarTipo() {
		// fila de la tabla seleccionada, si -1, no hay fila seleccionada
		int fila = view.getEcpTable().getSelectedRow();

		if(fila < 0) return; // no hay fila seleccionada	

		ComponenteLinea cl = (ComponenteLinea) visualizacion.getEstructura().getAalc().get(fila);

		if(cl.getComponente().getClass() == ComponenteElemental.class) // no es posible
			return;

		// Creacin del command
		ModificarTipoCommand command = new ModificarTipoCommand();
		command.setComponente((ComponenteCompuesta) cl.getComponente());
		command.ejecutar();
		// Pilas undo/redo
		comandosEjecutados.agregarComando(command);
		view.getDecp11button().setEnabled(true);
		view.getDecp12button().setEnabled(false);

		// Actualizar modificaciones
		numeroModificaciones++;

		actualizarTabla();
	}

	/**
	 * Creacin de una estructura nueva que se abre en la ventana del editor.
	 */
	public void nuevaEstructura() {
		if(numeroModificaciones == 0 ||
				abrirDialogoAlertaGuardado() != JOptionPane.CANCEL_OPTION){

			ManagerEstructuras manager = ManagerEstructuras.getInstance();
			visualizacion.setEstructura(manager.nuevaEstructura());

			actualizarTabla();

			// Reinicializar atributos
			deshabilitarFilaSeleccionada();
			ultimaFilaSeleccionada = -1;
			comandosEjecutados.clear();
			actualizarHabilitacionDisparos();
			numeroModificaciones = 0;

			// Realimentacin
			actualizarRealimentacion();
			
			// Hacer visible editor
			view.getEcpScrollPane().setVisible(true);
		}	
	}



	/**
	 * Operacin rehacer.
	 */
	public void rehacer(){
		comandosEjecutados.rehacer();

		actualizarTabla();

		// desactivacin del botn undo
		if(!comandosEjecutados.esPosibleRehacer()) view.getDecp12button().setEnabled(false);

		// activacin de la pila redo
		view.getDecp11button().setEnabled(true);

		// Actualizar modificaciones
		numeroModificaciones++;
	}

	/**
	 * Establece visualizacion.
	 *
	 * @param visualizacion el nuevo visualizacion
	 */
	public void setEsquema(Visualizacion visualizacion) {
		this.visualizacion = visualizacion;
	}

	/**
	 * Sets the view.
	 *
	 * @param view the new view
	 */
	public void setView(EditorEstructuras view) {
		this.view = view;
	}

	public void validar() {

		String informe = visualizacion.getEstructura().validar(visualizacion.getPresentacion());
		System.out.println(informe);
		if(informe.equals("")){
			JOptionPane.showMessageDialog(view.getFrmEditorDeEstructuras(),
					"El equema validado es correcto. No se han detectado errores.",
					"Error de validacin",
					JOptionPane.INFORMATION_MESSAGE);
		}
		else{
			StringBuffer subinforme = new StringBuffer();
			subinforme.append("El visualizacion validado no es correcto. Se han encontrado los siguientes errores: \n\n");
			if(informe.length() > 500){
				subinforme.append(informe.substring(0, 499));
				subinforme.append("...\n...");


			}
			else{
				subinforme.append(informe);
			}
			JOptionPane.showMessageDialog(view.getFrmEditorDeEstructuras(),
					subinforme.toString(),
					"Error de validacin",
					JOptionPane.ERROR_MESSAGE);

		}

	}

	public void eliminarEstructura(){

		ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();
		ManagerAreas managerAreas = ManagerAreas.getInstance();

		// Preguntar si realmente quiere borrar el rea
		int opcion = JOptionPane.showConfirmDialog(
				view.getFrmEditorDeEstructuras(),
				"Desea eliminar la estructura " + visualizacion.getEstructura().getNombre() + "?\n" +
						"Nota: esta operacin no se podr deshacer.",
						"Eliminar estructura",
						JOptionPane.YES_NO_OPTION);

		if(opcion == JOptionPane.YES_OPTION){
			Estructura estructura = visualizacion.getEstructura();
			// Borrado de la estructura del rea y de sus esqemas
			Area area = managerAreas.obtenerAreaPorID(estructura.getIdArea());
			if(area != null){
				// Como en este punto, estructura es un clon, hay que eliminarla de la lista de estructuras de forma manual
				for(int i = area.getEstructuras().size()-1; i >= 0; i--){
					if(area.getEstructuras().get(i).getId().equals(estructura.getId())){
						area.getEstructuras().remove(i);
					}
				}
				
				
				area.getIdEstructuras().remove(estructura.getId());
				for(Esquema e: area.getEsquemas()){
					if(e.getIdEstructuras().contains(estructura.getId())){
						e.getIdEstructuras().remove(estructura.getId());
					}
				}
				managerAreas.guardarAreas();
			}

			// Borrado de la estructura
			managerEstructuras.eliminar(estructura);

			// Reinicializar interfaz

			visualizacion.setEstructura(null);
			actualizarArbolLAE();
			actualizarRealimentacion();
			view.getEcpScrollPane().setVisible(false);
		}
	}
}
