package controlador;

import java.util.SortedSet;

import javax.swing.ListSelectionModel;
import javax.swing.UIManager;

import modelo.Grupo;
import modelo.Usuario;

import utils.ManagerGrupos;
import utils.ManagerUsuarios;
import utils.tablemodels.ModeloGestionGrupos;
import utils.tablemodels.ModeloGestionUsuarios;
import vista.EditorGrupos;
import vista.GestionUsuariosGrupo;
import vista.NuevoGrupo;
import vista.NuevoUsuario;

public class EditorGruposController {
    private static EditorGruposController INSTANCE = null;
    private EditorGrupos view;
    private ManagerGrupos manager;
    private int ultimaFilaSeleccionada;
 
    // Private constructor suppresses 
    private EditorGruposController() {
    }
 
    // creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new EditorGruposController();
        }
    }
 
    public static EditorGruposController getInstance() {
        createInstance();
        return INSTANCE;
    }
    
    public void inicializar(EditorGrupos view){
    	this.view = view;
		manager = ManagerGrupos.getInstance();
		ultimaFilaSeleccionada = -1;

		actualizarTabla();
		
		// Deshabilitar campos si no hay filas en la tabla
		if(view.getTable().getRowCount() == 0)
			deshabilitarCampos();
    }
    
	public void actualizarTabla() {
		// Selection model
		ListSelectionModel sm = view.getTable().getSelectionModel();
		
		// Obtener listado Usuarios

		SortedSet<String> listado = manager.listarGrupos();

		// Crear modelo

		ModeloGestionGrupos modelo = new ModeloGestionGrupos(listado);

		// Aplicar modelo

		view.getTable().setModel(modelo);
		
		// Recuperar fila seleccionada

		
		if(ultimaFilaSeleccionada < 0)
			ultimaFilaSeleccionada = 0;
		if(ultimaFilaSeleccionada >= view.getTable().getRowCount())
			ultimaFilaSeleccionada = view.getTable().getRowCount()-1;
		
		sm.setSelectionInterval(ultimaFilaSeleccionada, ultimaFilaSeleccionada);
		

	}
	
	public void actualizarFilaSeleccionada(){
		ultimaFilaSeleccionada = view.getTable().getSelectedRow();
	}
    
	public void cerrarVentana(){
		manager.guardarGrupos();

		view.getPreviousWindow().setEnabled(true);
		view.dispose();
	}
	
	public void habilitarCampos() {
		view.getTextArea().setEnabled(true);
		view.getTextFieldNombre().setEnabled(true);
		view.getTextArea().setEditable(true);
		view.getTextArea().setBackground(UIManager.getColor("TextArea.background"));
		view.getEditarUsuariosButton().setEnabled(true);

	}
	
	public void deshabilitarCampos() {
		view.getTextArea().setEnabled(false);
		view.getTextFieldNombre().setEnabled(false);
		view.getTextArea().setEnabled(false);
		view.getTextArea().setBackground(UIManager.getColor("TextArea.disabledBackground"));
		view.getEditarUsuariosButton().setEnabled(false);

	}
	
	public void agregarGrupo(){
		NuevoGrupo dialog = new NuevoGrupo();
		dialog.setVisible(true);
		dialog.setAlwaysOnTop(true);
		dialog.setView(view);

		view.setEnabled(false);
	}
	
	public void eliminarGrupo(){
		// Obtener nombre de usuario de la tabla
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada >= 0){

			String grupoSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);

			// borrar
			manager.eliminarGrupoPorNombre(grupoSeleccionado);

			// actualizar

			actualizarTabla();
			
			// Deshabilitar campos si no hay filas en la tabla
			if(view.getTable().getRowCount() == 0)
				deshabilitarCampos();
			
			actualizarFilaSeleccionada();

		}
	}
	
	public void actualizarCampos() {
		// Obtener nombre de usuario de la tabla
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada >= 0){
			String grupoSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);

			// Obtener objeto grupo del manager
			Grupo grupo = manager.obtenerGrupoPorNombre(grupoSeleccionado);

			// Actualizar campos

			view.getTextArea().setText(grupo.getDescripcion());
			view.getTextFieldNombre().setText(grupo.getNombre());
		}
		else{
			view.getTextArea().setText("");
			view.getTextFieldNombre().setText("");
		}

	}
	
	public void cambiarNombre(){
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada > -1){
			String grupoSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);
			Grupo grupo = manager.obtenerGrupoPorNombre(grupoSeleccionado);

			// Obtener datos
			String nombre = view.getTextFieldNombre().getText();

			// Eliminar grupo
			manager.eliminarGrupoPorID(grupo.getID());
			
			// Cambiar el nombre
			grupo.setNombre(nombre);
			
			// Volver a insertar
			manager.insertarGrupo(grupo);
			
			// Actualizar la tabla
			actualizarTabla();
			actualizarCampos();
			
		}
	}

	
	public void cambiarDescripcion(){
		// Obtener usuario
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada > -1){
			String grupoSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);
			Grupo grupo = manager.obtenerGrupoPorNombre(grupoSeleccionado);

			// Obtener datos
			String descripcion = view.getTextArea().getText();

			// Actualizar
			grupo.setDescripcion(descripcion);
		}
	}

	public void abrirGestionUsuariosGrupo() {
		// Localizacin del grupo seleccionado
		ManagerGrupos mg = ManagerGrupos.getInstance();
		int filaSeleccionada = view.getTable().getSelectedRow();
		String grupoSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);
		Grupo grupo = mg.obtenerGrupoPorNombre(grupoSeleccionado);
		
		GestionUsuariosGrupo gug = new GestionUsuariosGrupo(view, grupo);
		gug.setVisible(true);
		gug.setAlwaysOnTop(true);
		gug.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
		
	}
}
