package controlador;

import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Frame;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.TableCellRenderer;

import modelo.Acabado;
import modelo.Area;
import modelo.Aspecto;
import modelo.Presentacion;
import modelo.Propiedad;

import utils.ManagerPresentaciones;
import utils.tablemodels.ModeloEditorPresentacionesAspecto;
import utils.tablemodels.ModeloEditorPresentacionesPropiedades;
import vista.EditorPresentaciones;
import vista.EditorTipoDatos;
import vista.EditorTipoPresentacion;


public class EditorPresentacionesController {
	private static class JTableButtonMouseListener extends MouseAdapter {
		private final JTable table;

		public JTableButtonMouseListener(JTable table) {
			this.table = table;
		}

		public void mouseClicked(MouseEvent e) {
			int column = table.getColumnModel().getColumnIndexAtX(e.getX());
			int row    = e.getY()/table.getRowHeight(); 

			if (row < table.getRowCount() && row >= 0 && column < table.getColumnCount() && column >= 0) {
				Object value = table.getValueAt(row, column);
				if (value instanceof JButton) {
					((JButton)value).doClick();
				}
			}
		}
	}
	private static class JTableButtonRenderer implements TableCellRenderer {		
		@Override public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			JButton button = (JButton)value;
			if (isSelected) {
				button.setForeground(table.getSelectionForeground());
				button.setBackground(table.getSelectionBackground());
			} else {
				button.setForeground(table.getForeground());
				button.setBackground(UIManager.getColor("Button.background"));
			}
			return button;	
		}
	}

	public static final int CREAR = 0;
	public static final int MODIFICAR = 1;
	private static EditorPresentacionesController INSTANCE = null;
	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorPresentacionesController();
		}
	}
	public static EditorPresentacionesController getInstance() {
		createInstance();
		return INSTANCE;
	}

	private EditorPresentaciones view;

	private Presentacion presentacion;
	
	private String nombreAntiguo;

	private int accion;

	private ArrayList<String> valoresAcabado;

	// Private constructor suppresses 
	private EditorPresentacionesController() {}

	public void abrirEditorTipoDatos(){
		
		// Propiedad seleccionada
		int filaSeleccionada = view.getTablePropiedades().getSelectedRow();
		Propiedad propiedadSeleccionada = presentacion.getPropiedades().get(filaSeleccionada);
		
		EditorTipoDatos et = new EditorTipoDatos(view, propiedadSeleccionada);
		et.setVisible(true);
		et.setAlwaysOnTop(true);
		et.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
	}

	public void abrirManual() {
		Desktop desktop;
		String direccion = System.getProperty("user.dir")+"\\manual\\presentaciones\\";
		String filename = "ee2_f2.html";
		File file = new File(direccion+filename);
		// minimizar ventanas
		view.setState(Frame.ICONIFIED);
		view.getPreviousWindow().setState(Frame.ICONIFIED);

		if (Desktop.isDesktopSupported()){// si ste Host soporta esta API 
			desktop = Desktop.getDesktop();//objtengo una instancia del Desktop(Escritorio)de mi host 
			try {
				desktop.open(file);//abro el archivo con el programa predeterminado
			}
			catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		else{ JOptionPane.showMessageDialog(null,"No es posible mostrar el manual de usuario en esta mquina.");
		}
	}

	public void acabadoActualizarCampos(){
		view.getTextFieldAcabado1().setText(valoresAcabado.get(0));
		view.getTextFieldAcabado2().setText(valoresAcabado.get(1));
	}

	public void actualizarRealimentacion() {
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		//view.getTextAutor();
		//view.getTextAutorModificacion();
		view.getTextCreacion().setText(sdf.format(presentacion.getCreacion().getTime()));
		view.getTextUltimaModificacion().setText(sdf.format(presentacion.getModificacion().getTime()));
		view.getTextNombre().setText(presentacion.getNombre());
	}

	public void aspectoActualizarCampos() {
		int filaSeleccionada = view.getTableAspecto().getSelectedRow();
		if(filaSeleccionada >= 0){
			Aspecto aspecto = presentacion.getAspectos().get(filaSeleccionada);
			Color c = new Color(aspecto.getColor());
			view.getTextFieldColor().setBackground(c);
			view.getComboBoxTamanyo().setSelectedItem(aspecto.getTamanyo()+"");
			view.getComboBoxFuente().setSelectedItem(aspecto.getFuente());
		}
		else{
			view.getTextFieldColor().setBackground(Color.WHITE);
			view.getComboBoxTamanyo().setSelectedItem("14");
			view.getComboBoxFuente().setSelectedItem("Arial");
		}
	}

	public void aspectoActualizarTabla(){

		// Creacin del modelo
		ModeloEditorPresentacionesAspecto modelo = new ModeloEditorPresentacionesAspecto(presentacion.getAspectos());

		// Actualizar tabla
		view.getTableAspecto().setModel(modelo);
	}

	public void aspectoCambiarColor() {

		// color chooser
		Color newColor = JColorChooser.showDialog(
				view,
				"Selecciona un color",
				null);

		// actualizar campo

		view.getTextFieldColor().setBackground(newColor);

		// guardar color en aspecto

		int filaSeleccionada = view.getTableAspecto().getSelectedRow();
		Aspecto aspecto = presentacion.getAspectos().get(filaSeleccionada);
		aspecto.setColor(newColor.getRGB());

	}

	public void aspectoHabilitarCampos(){

		int filaSeleccionada = view.getTableAspecto().getSelectedRow();

		if(filaSeleccionada >= 0){
			view.getTextFieldColor().setEnabled(true);
			view.getComboBoxTamanyo().setEnabled(true);
			view.getComboBoxFuente().setEnabled(true);
		}
		else{
			view.getTextFieldColor().setEnabled(false);
			view.getComboBoxTamanyo().setEnabled(false);
			view.getComboBoxFuente().setEnabled(false);
		}
	}


	public void cerrarVentana() {
		// Se comprueba que el nombre de la presentacin no est vaco
		String presentacionText = view.getTextNombre().getText();
		if(presentacionText.trim().equals("")){

			JOptionPane.showMessageDialog(view,
					"El campo de nombre de presentacin no puede estar vaco.",
					"Error",
					JOptionPane.ERROR_MESSAGE);
		}
		else{

			// Comprobar que no exista ninguna presentacion con el mismo nombre
			// en el caso de que se est creando una nueva. En el caso de modificar, 
			// comprobar que no se corresponde con otra que no sea ella misma.

			ManagerPresentaciones manager = ManagerPresentaciones.getInstance();
			boolean repetido = false;

			for(Presentacion p: manager.getPresentacionesPrivadas()){
				if(p.getNombre().equals(presentacionText)){
					if(accion == CREAR){
						repetido = true;
					}
					else{ // accion = MODIFICAR
						if(p!=presentacion)
							repetido = true;
						else
							repetido = false;
					}
				}
			}
			for(Presentacion p: manager.getPresentacionesPublicas()){
				if(p.getNombre().equals(presentacionText)){
					if(accion == CREAR){
						repetido = true;
					}
					else{ // accion = MODIFICAR
						if(p!=presentacion)
							repetido = true;
						else
							repetido = false;
					}
				}				
			}

			if(repetido){

				JOptionPane.showMessageDialog(view,
						"Nombre de presentacin invlido. Existe una presentacin con el mismo nombre.",
						"Error",
						JOptionPane.ERROR_MESSAGE);
			}
			else{
				
				// Comprobar los valores de los campos de acabado
				try {
					int aux1 = Integer.parseInt(valoresAcabado.get(0));
					int aux2 = Integer.parseInt(valoresAcabado.get(1));
					if(aux1 < 0 || aux2 < 0) throw new NumberFormatException();
					
					guardarCambios();
					view.getPreviousWindow().setEnabled(true);

					// Actualizar tablas de la ventana anterior.
					((EditorTipoPresentacion) view.getPreviousWindow()).getController().actualizarTablas();
					view.dispose();
				} catch (NumberFormatException e) {
					JOptionPane.showMessageDialog(view,
							"Valores de propiedades de ACABADO invlidos. \nLos valores han de ser enteros mayores o iguales que 0.",
							"Error",
							JOptionPane.ERROR_MESSAGE);
				}
			}
		}

	}

	public int getAccion() {
		return accion;
	}

	public Presentacion getPresentacion() {
		return presentacion;
	}

	public ArrayList<String> getValoresAcabado() {
		return valoresAcabado;
	}

	public EditorPresentaciones getView() {
		return view;
	}

	private void guardarCambios() {
		
		// Cambios de elementos de acabado
		presentacion.getAcabados().get(0).setValor(Integer.parseInt(valoresAcabado.get(0)));
		presentacion.getAcabados().get(1).setValor(Integer.parseInt(valoresAcabado.get(1)));	
		
		// Cambios en el manager de presentaciones
		
		ManagerPresentaciones manager = ManagerPresentaciones.getInstance();
		if(accion == CREAR){ // Se inserta una nueva presentacin en el manager
			switch(presentacion.getTipo()){
			case Presentacion.PRIVADA:
				manager.agregarPresentacionPrivada(presentacion);
				break;
			case Presentacion.PUBLICA:
				manager.agregarPresentacionPublica(presentacion);
				break;
			case Presentacion.DEFAULT:
				manager.agregarPresentacionDefault(presentacion);
				break;
			default:
				break;
			}
		}
		else{
			manager.modificarPresentacion(presentacion, this.nombreAntiguo);
		}


	}

	public void inicializar(Presentacion presentacion, int tipo) {
		if(presentacion == null){
			
			// Seleccin de nombre predefinido no repetido
			String nombre = "Presentacion";
			ManagerPresentaciones managerPresentaciones = ManagerPresentaciones.getInstance();
			int i = 1;

			while(managerPresentaciones.listarNombrePresentaciones().contains(nombre)){
				nombre = "Presentacion"+i;
				i++;
			}

			this.presentacion = new Presentacion(nombre, tipo);
			
			accion = CREAR; // parmetro necesario a la hora de realizar acciones a la salida del formulario
		}
		else{
			this.presentacion = presentacion;
			accion = MODIFICAR;
			this.nombreAntiguo = presentacion.getNombre();
		}
		actualizarRealimentacion();

		propiedadesActualizarTabla();
		propiedadesInicializarTabla();

		aspectoActualizarTabla();
		
		valoresAcabado = new ArrayList<String>();
		for(Acabado a: this.presentacion.getAcabados())
			valoresAcabado.add(a.getValor()+"");
		
		acabadoActualizarCampos();
	}

	public void propiedadesActualizarModelo(Object value, int row, int col) {

		// recuperar propiedad
		Propiedad p = presentacion.getPropiedades().get(row);
		// actualizar propiedad
		switch(col){
		case 0:
			p.setVisible((Boolean) value);
			break;
		case 1:
			p.setNombre((String) value);
			break;
		case 2:
			p.setAlias((String) value);
			break;
		case 3:
			break;
		default:
			break;
		}

		// refrescar tabla
		propiedadesActualizarTabla();
	}

	public void propiedadesActualizarTabla() {

		// Creacin del modelo
		ModeloEditorPresentacionesPropiedades modelo = new ModeloEditorPresentacionesPropiedades(presentacion.getPropiedades());

		// Actualizacin de la tabla
		view.getTablePropiedades().setModel(modelo);

		// Render necesario para el botn de la tabla
		TableCellRenderer buttonRenderer = new JTableButtonRenderer();
		view.getTablePropiedades().getColumn("Editar").setCellRenderer(buttonRenderer);
	}
	
	public void propiedadesInicializarTabla(){
		// Aadir listener para el boton
		view.getTablePropiedades().addMouseListener(new JTableButtonMouseListener(view.getTablePropiedades()));
	}

	public void setAccion(int accion) {
		this.accion = accion;
	}

	public void setPresentacion(Presentacion presentacion) {
		this.presentacion = presentacion;
	}

	public void setValoresAcabado(ArrayList<String> valoresAcabado) {
		this.valoresAcabado = valoresAcabado;
	}

	public void setView(EditorPresentaciones view) {
		this.view = view;
	}
	public void aspectoCambiarFuente() {
		int filaSeleccionada = view.getTableAspecto().getSelectedRow();
		Aspecto aspecto = presentacion.getAspectos().get(filaSeleccionada);
		aspecto.setFuente(view.getComboBoxFuente().getSelectedItem().toString());
		
		
	}
	public void aspectoCambiarTamanyo() {
		int filaSeleccionada = view.getTableAspecto().getSelectedRow();
		Aspecto aspecto = presentacion.getAspectos().get(filaSeleccionada);
		aspecto.setTamanyo(Integer.parseInt(view.getComboBoxTamanyo().getSelectedItem().toString()));
		
	}
	
	

}
