package controlador;

import java.util.ArrayList;

import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.Selector;
import modelo.SelectorLinea;
import utils.ManagerComponentes;
import utils.tablemodels.ModeloEditorSelectorComponentesInsertadas;
import utils.tablemodels.ModeloEditorSelectorComponentesNoInsertadas;
import utils.tablemodels.ModeloEditorSelectorComponentesSelector;
import vista.EditorEstructuraDerivada;
import vista.EditorSelector;

// TODO: Auto-generated Javadoc
/**
 * The Class EditorSelectorController.
 * Operaciones relacionada con el entorno editorial del selector
 */
public class EditorSelectorController {

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	/**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorSelectorController();
		}
	}

	/**
	 * Gets the single instance of EditorSelectorController.
	 *
	 * @return single instance of EditorSelectorController
	 */
	public static EditorSelectorController getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}

	/** Interfaz de usuario. */
	private EditorSelector view;

	/** Estructura abierta en la ventana principal del editor. */
	private Estructura estructuraAbierta;

	/** Subestructura derivada seleccionada en la interfaz del editor de estructura derivada. */
	private ComponenteCompuesta subestructuraDerivadaSeleccionada;

	/** Componentes que integran la tabla superior. */
	private ArrayList<Componente> componentesTablaSuperior;

	/** Componentes que integran la tabla inferior. */
	private ArrayList<Componente> componentesTablaInferior;

	/** Componentes que integran la tabla derecha. */
	private ArrayList<Componente> componentesTablaDerecha;
	
	/** Expresiones adjuntas a las componentes de la tabla derecha. */
	private ArrayList<String> expresionesTablaDerecha;

	/** Origen (de que tabla proceden) las componentes de la tabla derecha. */
	private ArrayList<Integer> origenComponente;

	/** INSTANCE. */
	private static EditorSelectorController INSTANCE = null;

	/** TABL a_ superior. */
	private static int TABLA_SUPERIOR = 0;

	/** TABL a_ inferior. */
	private static int TABLA_INFERIOR = 1;

	/** Numero de modificaciones realizadas desde que se abri la ventana. */
	private int numeroModificaciones;

	// Private constructor suppresses 
	/**
	 * Inicializa un objeto de tipo editor selector controller.
	 */
	private EditorSelectorController() {}


	/**
	 * Actualizar habilitacion disparos.
	 */
	public void actualizarHabilitacionDisparos(){

		if(numeroModificaciones==0) view.getDes02button().setEnabled(false);
		else view.getDes02button().setEnabled(true);

	}


	/**
	 * Actualizar tabla derecha.
	 */
	private void actualizarTablaDerecha() {
		int selected = view.getDerechaTable().getSelectedRow();
		view.getDerechaTable().setModel(new ModeloEditorSelectorComponentesSelector(componentesTablaDerecha, expresionesTablaDerecha));
		// Restaurar fila seleccionada

		if(selected >= 0){
			if(view.getDerechaTable().getRowCount()>0){
				if(selected < view.getDerechaTable().getRowCount())
					view.getDerechaTable().setRowSelectionInterval(selected, selected);
				else if (selected-1 >= 0)
					view.getDerechaTable().setRowSelectionInterval(selected-1, selected-1);
			}
		}
	}

	/**
	 * Actualizar tabla editor estructura derivada.
	 */
	private void actualizarTablaEditorEstructuraDerivada() {
		EditorEstructuraDerivadaController eedController = EditorEstructuraDerivadaController.getInstance();
		eedController.actualizarTabla();
	}

	/**
	 * Actualizar tabla inferior.
	 */
	private void actualizarTablaInferior() {
		int selected = view.getInferiorTable().getSelectedRow();
		view.getInferiorTable().setModel(new ModeloEditorSelectorComponentesNoInsertadas(componentesTablaInferior));

		if(selected >= 0){
			if(view.getInferiorTable().getRowCount()>0){
				if(selected < view.getInferiorTable().getRowCount())
					view.getInferiorTable().setRowSelectionInterval(selected, selected);
				else if (selected-1 >= 0)
					view.getInferiorTable().setRowSelectionInterval(selected-1, selected-1);
			}
		}
	}

	/**
	 * Actualizar tablas.
	 */
	public void actualizarTablas(){
		actualizarTablaSuperior();
		actualizarTablaInferior();
		actualizarTablaDerecha();
	}

	/**
	 * Actualizar tabla superior.
	 */
	private void actualizarTablaSuperior() {
		int selected = view.getSuperiorTable().getSelectedRow();
		System.out.println(selected+"");
		view.getSuperiorTable().setModel(new ModeloEditorSelectorComponentesInsertadas(componentesTablaSuperior));

		if(selected >= 0){
			if(view.getSuperiorTable().getRowCount()>0){
				if(selected < view.getSuperiorTable().getRowCount())
					view.getSuperiorTable().setRowSelectionInterval(selected, selected);
				else if (selected-1 >= 0)
					view.getSuperiorTable().setRowSelectionInterval(selected-1, selected-1);
			}
		}
	}

	/**
	 * Agregar a la tabla derecha una componetne que procede de la tabla inferior.
	 */
	public void agregarDeTablaInferior(){
		// Recuperar fila de tabla inferior seleccionada
		int filaSeleccionada = view.getInferiorTable().getSelectedRow();

		// Si no hay fila seleccionada, accion no disponible
		if(filaSeleccionada == -1) return;

		// Recuperar componente en la fila seleccionada
		Componente componenteSeleccionada = componentesTablaInferior.get(filaSeleccionada);

		// Eliminar componente de la tabla inferior
		componentesTablaInferior.remove(filaSeleccionada);

		// Agregar componente a la tabla derecha
		componentesTablaDerecha.add(componenteSeleccionada);
		
		// Creacin de la expresin que iguala a la componente agregada
		expresionesTablaDerecha.add("");

		// Aadir referencia
		origenComponente.add(TABLA_INFERIOR);

		// Modificacion
		numeroModificaciones++;
		actualizarHabilitacionDisparos();

		// ActualizarTablas
		actualizarTablaInferior();
		actualizarTablaDerecha();		
	}

	/**
	 * Agregar a la tabla derecha una componente que procede de la tabla superior.
	 */
	public void agregarDeTablaSuperior(){
		// Recuperar fila de tabla superior seleccionada
		int filaSeleccionada = view.getSuperiorTable().getSelectedRow();

		// Si no hay fila seleccionada, accion no disponible
		if(filaSeleccionada == -1) return;

		// Recuperar componente en la fila seleccionada
		Componente componenteSeleccionada = componentesTablaSuperior.get(filaSeleccionada);

		// Eliminar componente de la tabla superior
		componentesTablaSuperior.remove(filaSeleccionada);

		// Agregar componente a la tabla derecha
		componentesTablaDerecha.add(componenteSeleccionada);
		
		// Creacin de la expresin que iguala a la componente agregada
		expresionesTablaDerecha.add("");

		// Aadir referencia
		origenComponente.add(TABLA_SUPERIOR);

		// Modificacion
		numeroModificaciones++;
		actualizarHabilitacionDisparos();

		// ActualizarTablas
		actualizarTablaDerecha();
		actualizarTablaSuperior();
	}

	/**
	 * Eliminar una componente de la tabla derecha y agregarla a su tabla de origen
	 */
	public void borrarDeTablaDerecha(){
		// Recuperar fila de tabla inferior seleccionada
		int filaSeleccionada = view.getDerechaTable().getSelectedRow();

		// Si no hay fila seleccionada, accion no disponible
		if(filaSeleccionada == -1) return;

		// Recuperar componente en la fila seleccionada
		Componente componenteSeleccionada = componentesTablaDerecha.get(filaSeleccionada);

		// Eliminar componente de la tabla derecha
		componentesTablaDerecha.remove(filaSeleccionada);

		// Agregar componente a la tabla derecha
		if(origenComponente.get(filaSeleccionada)==TABLA_SUPERIOR){
			componentesTablaSuperior.add(componenteSeleccionada);
			actualizarTablaSuperior();
		}
		else{
			componentesTablaInferior.add(componenteSeleccionada);
			actualizarTablaInferior();
		}
		// EliminarReferencia

		origenComponente.remove(filaSeleccionada);
		
		// Eliminar expresin
		
		expresionesTablaDerecha.remove(filaSeleccionada);

		// Modificacion
		numeroModificaciones++;
		actualizarHabilitacionDisparos();

		// ActualizarTablas
		actualizarTablaDerecha();	
	}

	/**
	 * Busca las componentes que forman parte de la subcomponente original, pero no forman parte de la subcomponente derivada.
	 * Esto es: componentes que pertenecen a la subcomponente original, pero no a la derivada.
	 * Son las que se mostrarn posteriormente en la tabla inferior.
	 *
	 * @return the componente compuesta
	 */
	public ComponenteCompuesta buscarComponentesNoInsertadas(){
		ManagerComponentes d = ManagerComponentes.getInstance();
		d.actualizar();

		// Buscar la componente compuesta a la cual hace referencia la componente derivada
		String idComponenteOrigen = subestructuraDerivadaSeleccionada.getIdComponenteOrigen();
		ComponenteCompuesta componenteOriginal = (ComponenteCompuesta) d.getComponentes().get(idComponenteOrigen); 

		ComponenteCompuesta componenteOriginalClonada = null;
		try {
			componenteOriginalClonada = (ComponenteCompuesta) componenteOriginal.clone();
			/* Se va a sustraer las componentes insertadas, de las que originalmente tiene la estructura
			 * para quedarse nicamente con las que no estn insertadas. Estas componentes no insertadas
			 * son posteriormente mostradas en la tabla inferior 
			 */
			for(int i = 0; i < subestructuraDerivadaSeleccionada.getComponentes().size(); i++){
				// Se recorre todas las componentes insertadas
				Componente compoSDS = subestructuraDerivadaSeleccionada.getComponentes().get(i);
				String idCompoOriginal = compoSDS.getIdComponenteOrigen();
				Componente compoOrigen = d.getComponentes().get(idCompoOriginal);
				// Se recorre las componentes originales
				for(int j = 0; j < componenteOriginalClonada.getComponentes().size(); j++){

					Componente compoCOC = componenteOriginalClonada.getComponentes().get(j);
					// Si casan, se elimina
					if(compoCOC.getId().equals(compoOrigen.getId())){

						componenteOriginalClonada.eliminarComponente(j);		
					}

				}

			}

		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}

		return componenteOriginalClonada;

	}

	/**
	 * Cerrar ventana.
	 */
	public void cerrarVentana() {
		view.getPreviousWindow().setEnabled(true);
		view.dispose();

	}

	/**
	 * Crear selector a partir de las componentes de la taba derecha.
	 */
	public void crearSelector(){


		// Componente a la cual pertenece el selector
		Selector selector = new Selector(subestructuraDerivadaSeleccionada);
		subestructuraDerivadaSeleccionada.setSelector(selector);

		// Componentes que forman parte del selector

		selector.setComponentes(componentesTablaDerecha);
		selector.setComponentesID(new ArrayList<String>());
		for(int i = 0; i < componentesTablaDerecha.size(); i++){
			selector.getComponentesID().add(componentesTablaDerecha.get(i).getId());
		}
		
		// Expresiones que forman parte del selector
		
		selector.setExpresiones(expresionesTablaDerecha);

		// Creacin de las lneas del selector

		SelectorLinea cabecera = new SelectorLinea(selector, selector.getCcSelector().getAnidamiento()+1);
		cabecera.setInicioSelector(true);
		selector.getLineas().add(cabecera);

		for(int i = 0; i < componentesTablaDerecha.size(); i++){
			SelectorLinea linea = new SelectorLinea(selector, selector.getCcSelector().getAnidamiento()+1);
			if(i == 0){ // primera linea
				linea.setParentesisInicio(true);
				linea.setComa(true);
			}
			if(i == componentesTablaDerecha.size()-1){ // ltima lnea
				linea.setParentesisFin(true);
				linea.setComa(false);
			}
			if(i != 0 && i != componentesTablaDerecha.size()-1){
				linea.setComa(true);
			}
			linea.setContenido(componentesTablaDerecha.get(i).getNombre() + " " + expresionesTablaDerecha.get(i));
			selector.getLineas().add(linea);
		}

		// Actualizar tabla previa

		actualizarTablaEditorEstructuraDerivada();
	}

	/**
	 * Eliminar selector.
	 */
	public void eliminarSelector(){
		this.subestructuraDerivadaSeleccionada.setSelector(null);

		// Actualizar tabla previa

		actualizarTablaEditorEstructuraDerivada();
	}

	/**
	 * Guardar modificaciones.
	 */
	public void guardarModificaciones() {
		// Si la tabla derecha no tiene ninguna comopnente, no se inserta ningn tipo de selector,
		// o se borra en el caso de que la componente ya tuviese uno y se hubiesen quitado todas
		// sus componentes

		if(componentesTablaDerecha.isEmpty())
			eliminarSelector();
		else
			crearSelector();
	}

	/**
	 * Inicializar.
	 *
	 * @param editorSelector the editor selector
	 */
	public void inicializar(EditorSelector editorSelector) {
		// View actual
		view = editorSelector;

		// Controladores de las ventanas anteriores
		EditorEstructurasController eecontroller = EditorEstructurasController.getInstance();
		EditorEstructuraDerivadaController edcontroller = EditorEstructuraDerivadaController.getInstance();

		// Estructura abierta en la ventana principal del editor
		estructuraAbierta = eecontroller.getEsquema().getEstructura();

		// Actualizar label estructura		
		view.getEstructuraLabel().setText("Estructura: " + estructuraAbierta.getRaiz().getNombre());

		// Subestructura seleccionada en la ventana anterior
		int filaSeleccionada = edcontroller.getView().getEedTable().getSelectedRow();
		if(filaSeleccionada != -1){

			Linea lineaSeleccionada = edcontroller.getEstructuraDerivada().getLineas().get(filaSeleccionada);

			// Si la linea seleccionada pertenece al selector, la subestructura derivada es aquella
			// a la cual pertenece el selector seleccionado
			if(lineaSeleccionada.getClass() == SelectorLinea.class){
				SelectorLinea sl = (SelectorLinea) lineaSeleccionada;
				subestructuraDerivadaSeleccionada =  sl.getSelector().getCcSelector();
			}
			else{ // ComponenteLinea.class 
				/*  ...
				 * 	   - EstructuraDerivada
				 * 			- SubestructuraDerivada
				 * 				- Selector
				 * 				- Compo
				 * 				- Compo
				 * 					- Compo
				 * 					...
				 * 				...	
				 */
				ComponenteLinea cl = (ComponenteLinea) lineaSeleccionada;
				Componente componenteAuxiliar = cl.getComponente();
				//System.out.println("CompoSeleccionada: " + componenteAuxiliar.getNombre());
				while(componenteAuxiliar.getParent().getParent() != null &&
						componenteAuxiliar.getParent().getParent().getDerivada()==true){
					componenteAuxiliar = componenteAuxiliar.getParent();
				}
				// componenteAuxiliar en este punto tiene anidamiento == 1
				subestructuraDerivadaSeleccionada = (ComponenteCompuesta) componenteAuxiliar;		
			}
		}

		// Otras inicializaciones

		origenComponente = new ArrayList<Integer>();
		numeroModificaciones=0;

		// Elementos tabla superior
		componentesTablaSuperior = (ArrayList<Componente>) subestructuraDerivadaSeleccionada.getComponentes().clone();

		// Elementos tabla inferior

		ComponenteCompuesta componentesNoInsertadas = buscarComponentesNoInsertadas();
		componentesTablaInferior = (ArrayList<Componente>) componentesNoInsertadas.getComponentes().clone();

		// Elementos tabla derecha

		componentesTablaDerecha = new ArrayList<Componente>();
		
		// Expresiones
		
		expresionesTablaDerecha = new ArrayList<String>();

		// Si existe selector, reestablecer las componentes existentes

		if(subestructuraDerivadaSeleccionada.getSelector() != null){
			Selector selector = subestructuraDerivadaSeleccionada.getSelector();
			componentesTablaDerecha = selector.getComponentes();
			expresionesTablaDerecha = selector.getExpresiones();

			// Busqueda en la lista de la tabla de arriba
			for(int i = 0; i < componentesTablaDerecha.size(); i++){
				for(int j = componentesTablaSuperior.size()-1; j >= 0 ; j--){
					// Si uno de la tabla derecha coincide con uno de la tabla de arriba, se borra de la tabla de arriba
					if(componentesTablaDerecha.get(i).getId().equals(componentesTablaSuperior.get(j).getId())){
						componentesTablaSuperior.remove(j);
						// Recuperar referencia a tabla origen
						origenComponente.add(TABLA_SUPERIOR);
					}
				}
				for(int j = componentesTablaInferior.size()-1; j >= 0 ; j--){
					// Si uno de la tabla derecha coincide con uno de la tabla de inferior, se borra de la tabla de inferior
					if(componentesTablaDerecha.get(i).getId().equals(componentesTablaInferior.get(j).getId())){
						componentesTablaInferior.remove(j);
						// Recuperar referencia a tabla origen
						origenComponente.add(TABLA_INFERIOR);
					}
				}
			}
		}

		// Acualizar Tabas

		actualizarTablas();


	}

	/**
	 * Modificar expresin.
	 *
	 * @param nuevoValor nuevo valor de la expresin
	 * @param rowIndex ndice de la fila de la tabla de la cual se modifica la expresin
	 */
	public void modificarExpresin(String nuevoValor, int rowIndex){
		expresionesTablaDerecha.add(rowIndex, nuevoValor);
		
		numeroModificaciones++;
		
		actualizarHabilitacionDisparos();
		
		actualizarTablaDerecha();
	}

}
