package controlador;


import java.util.ArrayList;

import javax.swing.JOptionPane;

import modelo.Propiedad;
import modelo.TipoDatos;
import vista.EditorTipoDatos;

public class EditorTipoDatosController {
	private static EditorTipoDatosController INSTANCE = null;
	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorTipoDatosController();
		}
	}
	public static EditorTipoDatosController getInstance() {
		createInstance();
		return INSTANCE;
	}
	private EditorTipoDatos view;
	private Propiedad propiedad;

	private String valorLongitudMaxima;

	private String valorLongitudMinima;

	// Private constructor suppresses 
	private EditorTipoDatosController() {}



	public void actualizarCampos(){
		TipoDatos td = propiedad.getTipoDatos();

		if(td.isAlfanumerico()){
			view.getRdbtnAlfanumrico().setSelected(true);
			view.getTextFieldLongMax().setEnabled(true);
			view.getTextFieldLongMin().setEnabled(true);
			view.getChckbxNoNegativo().setEnabled(false);

		}
		if(td.isNumerico()){
			view.getRdbtnNumrico().setSelected(true);
			view.getTextFieldLongMax().setEnabled(false);
			view.getTextFieldLongMin().setEnabled(false);
			view.getChckbxNoNegativo().setEnabled(true);

		}
		if(td.isRango()){
			view.getRdbtnRango().setSelected(true);
			view.getTextFieldLongMax().setEnabled(false);
			view.getTextFieldLongMin().setEnabled(false);
			view.getChckbxNoNegativo().setEnabled(false);

		}
		if(td.isBinario()){
			view.getRdbtnBinariosino().setSelected(true);
			view.getTextFieldLongMax().setEnabled(false);
			view.getTextFieldLongMin().setEnabled(false);
			view.getChckbxNoNegativo().setEnabled(false);

		}
		if(!td.isNulo()){
			view.getChckbxNoNulo().setSelected(true);
		}
		if(td.isNoNegativo()){
			view.getChckbxNoNegativo().setSelected(true);
		}

		view.getTextFieldLongMax().setText(valorLongitudMaxima);
		view.getTextFieldLongMin().setText(valorLongitudMinima);
	}

	public void cerrarVentana() {
		if(view.getRdbtnAlfanumrico().isSelected()){
			// Si est seleccionado el alfanumrico, validacin de campos
			try {
				// Comprobacin de tipos
				int aux1 = Integer.parseInt(valorLongitudMaxima);
				int aux2 = Integer.parseInt(valorLongitudMinima);
				if(aux1 < 0 || aux2 < 0 || aux2 > aux1) throw new NumberFormatException();
				
				// Guardado de datos
				propiedad.getTipoDatos().setLongMax(aux1);
				propiedad.getTipoDatos().setLongMin(aux2);
				
				propiedad.getTipoDatos().generarExpresion();
				view.getPreviousWindow().setEnabled(true);
				view.dispose();
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(view,
						"Valores de campos longitud invlidos. " +
						"\nLos valores han de ser enteros mayores o iguales que 0. " +
						"\nAdems, la longitud mxima ha de ser mayor o igual a la longitud mnima",
						"Error de formato",
						JOptionPane.ERROR_MESSAGE);
			}
		}
		else{
			propiedad.getTipoDatos().generarExpresion();
			view.getPreviousWindow().setEnabled(true);
			view.dispose();
		}

	}

	public Propiedad getPropiedad() {
		return propiedad;
	}

	public String getValorLongitudMaxima() {
		return valorLongitudMaxima;
	}

	public String getValorLongitudMinima() {
		return valorLongitudMinima;
	}

	public EditorTipoDatos getView() {
		return view;
	}

	public void inicializar(EditorTipoDatos editorTipoDatos,
			Propiedad propiedadSeleccionada) {
		this.view = editorTipoDatos;
		this.propiedad = propiedadSeleccionada;

		/* Guarda los valores de las longitudes en variables auxiliares. Esto se hace debido
		 * a que la evaluacin de si los valores son correctos o no se realia al salir del formulario
		 * en el mtodo cerrarVentana
		 */
		valorLongitudMaxima = propiedad.getTipoDatos().getLongMax()+"";
		valorLongitudMinima = propiedad.getTipoDatos().getLongMin()+"";

		actualizarCampos();

	}

	public void setPropiedad(Propiedad propiedad) {
		this.propiedad = propiedad;
	}

	public void setValorLongitudMaxima(String valorLongitudMaxima) {
		this.valorLongitudMaxima = valorLongitudMaxima;
	}

	public void setValorLongitudMinima(String valorLongitudMinima) {
		this.valorLongitudMinima = valorLongitudMinima;
	}

	public void setView(EditorTipoDatos view) {
		this.view = view;
	}


}