package controlador;

import java.awt.Desktop;
import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import modelo.Presentacion;
import modelo.prototypes.PrototipoListaPropiedades;

import utils.ManagerPresentaciones;
import utils.tablemodels.ModeloEditorTipoPresentaciones;
import vista.EditorEstructuras;
import vista.EditorPresentaciones;
import vista.EditorTipoPresentacion;

public class EditorTipoPresentacionController {

	private EditorTipoPresentacion view;
	private ManagerPresentaciones manager;

	private static EditorTipoPresentacionController INSTANCE = null;

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EditorTipoPresentacionController();
		}
	}

	public static EditorTipoPresentacionController getInstance() {
		createInstance();
		return INSTANCE;
	}

	// Private constructor suppresses 
	private EditorTipoPresentacionController() {
		manager = ManagerPresentaciones.getInstance();

	}



	public void abrirNuevaPresentacion(int tipo){
		EditorPresentaciones et = new EditorPresentaciones(view, null, tipo);
		et.setVisible(true);
		et.setAlwaysOnTop(true);
		et.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
	}

	public void abrirModificarPresentacion(int tipo){

		// Seleccin de la presentacin a modificar
		Presentacion p;
		int filaSeleccionada;
		if(tipo == Presentacion.PRIVADA){
			filaSeleccionada = view.getPrivadasTable().getSelectedRow();
			p = manager.getPresentacionesPrivadas().get(filaSeleccionada);
		}
		else{
			filaSeleccionada = view.getPublicasTable().getSelectedRow();
			p = manager.getPresentacionesPublicas().get(filaSeleccionada);
		}

		// Llamada de la nueva ventana

		EditorPresentaciones et = new EditorPresentaciones(view, p, tipo);
		et.setVisible(true);
		et.setAlwaysOnTop(true);
		et.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
	}

	public void actualizarHabilitacionDisparos(){

		// Fila seleccionada en tabla publica
		int filaSeleccionadaPublica = view.getPublicasTable().getSelectedRow();

		// Fila seleccionada en tabla privada
		int filaSeleccionadaPrivada = view.getPrivadasTable().getSelectedRow();

		// Inicialmente, habilitado
		view.getDetp2().setEnabled(true); // arriba
		view.getDetp3().setEnabled(true); // abajo
		view.getDlppb2().setEnabled(true); // eliminar
		view.getDlppb3().setEnabled(true); // modificar
		view.getDlppb4().setEnabled(true); // default
		view.getDlppv2().setEnabled(true); // eliminar
		view.getDlppv3().setEnabled(true); // modificar

		// Si no hay fila seleccionada en tabla publica
		if(filaSeleccionadaPublica < 0){
			view.getDetp3().setEnabled(false);
			view.getDlppb2().setEnabled(false); 
			view.getDlppb3().setEnabled(false); 
			view.getDlppb4().setEnabled(false); 
		}

		// Si no hay fila seleccionada en tabla privada
		if(filaSeleccionadaPrivada < 0){
			view.getDetp2().setEnabled(false);
			view.getDlppv2().setEnabled(false); 
			view.getDlppv3().setEnabled(false); 
		}

		// Si la presentacion publica seleccionada es default...
		if(filaSeleccionadaPublica >= 0){
			Presentacion publicaSeleccionada = manager.getPresentacionesPublicas().get(filaSeleccionadaPublica);
			Presentacion presentacionDefault = manager.getPresentacionDefault();
			if(publicaSeleccionada.getId().equals(presentacionDefault.getId())){
				view.getDetp3().setEnabled(false); // flecha hacia abajo
				view.getDlppb4().setEnabled(false); // default
				view.getDlppb2().setEnabled(false); // eliminar
			}
		}
	}


	public void actualizarTablaPrivada(){
		// Recuperar presentaciones publicas
		ArrayList<Presentacion> privadas = manager.getPresentacionesPrivadas();
		// Crear Modelo de tabla
		ModeloEditorTipoPresentaciones modelo = new ModeloEditorTipoPresentaciones(privadas);
		// Actualizar tabla
		view.getPrivadasTable().setModel(modelo);
	}

	public void actualizarTablaPublica(){
		// Recuperar presentaciones publicas
		ArrayList<Presentacion> publicas = manager.getPresentacionesPublicas();
		// Crear Modelo de tabla
		ModeloEditorTipoPresentaciones modelo = new ModeloEditorTipoPresentaciones(publicas);
		// Actualizar tabla
		view.getPublicasTable().setModel(modelo);
	}

	public void actualizarTablas(){
		actualizarTablaPublica();
		actualizarTablaPrivada();
	}

	public void cerrarVentana() {
		EditorEstructurasController controller = EditorEstructurasController.getInstance();
		controller.actualizarListaPresentaciones();

		// Inicialmente no hay estructura. Dara error al actualizar la tabla.
		if(controller.getEsquema().getEstructura()!=null)
			controller.actualizarTabla();

		view.getPreviousWindow().setEnabled(true);
		view.dispose();

	}

	public EditorTipoPresentacion getView() {
		return view;
	}

	public void inicializar() {
		actualizarTablas();
		actualizarHabilitacionDisparos();
		PrototipoListaPropiedades plp = new PrototipoListaPropiedades();
		plp.abrir();
	}

	public void marcarPresentacionDefault(){
		// Fila seleccionada en tabla publica
		int filaSeleccionadaPublica = view.getPublicasTable().getSelectedRow();

		// Desmarcar la default actual
		Presentacion defaultActual = manager.getPresentacionDefault();
		defaultActual.setTipo(Presentacion.PUBLICA);
		manager.modificarPresentacion(defaultActual);

		// Recuperar de presentaciones publicas y marcar como default
		Presentacion presentacionSeleccionada = manager.getPresentacionesPublicas().get(filaSeleccionadaPublica);
		manager.eliminarPresentacionPublica(presentacionSeleccionada);
		presentacionSeleccionada.setTipo(Presentacion.DEFAULT);
		manager.agregarPresentacionDefault(presentacionSeleccionada);

		// Actualizar tabla superior
		actualizarTablaPublica();
	}

	public void moverAbajo(){

		// Fila seleccionada en tabla publica
		int filaSeleccionadaPublica = view.getPublicasTable().getSelectedRow();

		// Recuperar y borrar de presentaciones publicas
		Presentacion presentacionSeleccionada = manager.getPresentacionesPublicas().get(filaSeleccionadaPublica);
		manager.eliminarPresentacionPublica(presentacionSeleccionada);

		// Aadir a presentaciones privadas
		manager.agregarPresentacionPrivada(presentacionSeleccionada);

		// Actualizar la tabla
		actualizarTablas();
	}


	public void moverArriba(){

		// Fila seleccionada en tabla privada
		int filaSeleccionadaPrivada = view.getPrivadasTable().getSelectedRow();

		// Recuperar y borrar de presentaciones publicas
		Presentacion presentacionSeleccionada = manager.getPresentacionesPrivadas().get(filaSeleccionadaPrivada);
		manager.eliminarPresentacionPrivada(presentacionSeleccionada);

		// Aadir a presentaciones publicas
		manager.agregarPresentacionPublica(presentacionSeleccionada);

		// Actualizar la tabla
		actualizarTablas();
	}

	public void setView(EditorTipoPresentacion view) {
		this.view = view;
	}

	public void eliminarPresentacionPublica(){
		// Seleccionar presentacion
		int filaSeleccionadaPublica = view.getPublicasTable().getSelectedRow();
		Presentacion presentacionSeleccionada = manager.getPresentacionesPublicas().get(filaSeleccionadaPublica);

		// Mensaje de alerta
		int opcion = JOptionPane.showConfirmDialog(
				view,
				"Eliminar la presentacin " + presentacionSeleccionada.getNombre() + " ?",
				"Eliminar",
				JOptionPane.YES_NO_OPTION);

		// Borrado y actualizacin
		if(opcion == JOptionPane.YES_OPTION){
			manager.eliminarPresentacionPublica(presentacionSeleccionada);
			actualizarTablaPublica();

			// Si la presentacin eliminada era la que haba seleccionada en el editor
			// Se sustituye por la default

			EditorEstructurasController eec = EditorEstructurasController.getInstance();
			Presentacion presentacionEditor = eec.getEsquema().getPresentacion();
			if(presentacionEditor==presentacionSeleccionada){
				eec.getEsquema().setPresentacion(manager.getPresentacionDefault());
				eec.actualizarTabla();
			}

		}
	}

	public void eliminarPresentacionPrivada(){
		// Seleccionar presentacion
		int filaSeleccionadaPrivada = view.getPrivadasTable().getSelectedRow();
		Presentacion presentacionSeleccionada = manager.getPresentacionesPrivadas().get(filaSeleccionadaPrivada);

		// Mensaje de alerta
		int opcion = JOptionPane.showConfirmDialog(
				view,
				"Eliminar la presentacin " + presentacionSeleccionada.getNombre() + " ?",
				"Eliminar",
				JOptionPane.YES_NO_OPTION);

		// Borrado y actualizacin
		if(opcion == JOptionPane.YES_OPTION){
			manager.eliminarPresentacionPrivada(presentacionSeleccionada);
			actualizarTablaPrivada();

			// Si la presentacin eliminada era la que haba seleccionada en el editor
			// Se sustituye por la default

			EditorEstructurasController eec = EditorEstructurasController.getInstance();
			Presentacion presentacionEditor = eec.getEsquema().getPresentacion();
			if(presentacionEditor==presentacionSeleccionada){
				eec.getEsquema().setPresentacion(manager.getPresentacionDefault());
				eec.actualizarTabla();
			}

		}
	}

	public void abrirManual() {
		Desktop desktop;
		String direccion = System.getProperty("user.dir")+"\\manual\\presentaciones\\";
		String filename = "ee2_f1.html";
		// minimizar ventanas
		view.setState(Frame.ICONIFIED);
		view.getPreviousWindow().setState(Frame.ICONIFIED);

		File file = new File(direccion+filename); 

		if (Desktop.isDesktopSupported()){// si ste Host soporta esta API 
			desktop = Desktop.getDesktop();//objtengo una instancia del Desktop(Escritorio)de mi host 
			try {
				desktop.open(file);//abro el archivo con el programa predeterminado
			}
			catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		else{ JOptionPane.showMessageDialog(null,"No es posible mostrar el manual de usuario en esta mquina.");
		}
	}


}
