package controlador;

import java.util.SortedSet;

import javax.swing.JDialog;

import modelo.Usuario;

import utils.ManagerUsuarios;
import utils.tablemodels.ModeloGestionUsuarios;
import vista.GestionUsuarios;
import vista.NuevoUsuario;

public class GestionUsuariosController {
	private static GestionUsuariosController INSTANCE = null;
	private GestionUsuarios view;
	private ManagerUsuarios manager;

	// Private constructor suppresses 
	private GestionUsuariosController() {}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new GestionUsuariosController();
		}
	}

	public static GestionUsuariosController getInstance() {
		createInstance();
		return INSTANCE;
	}

	public void inicializar(GestionUsuarios view){
		this.view = view;
		manager = ManagerUsuarios.getInstance();

		actualizarTabla();
		deshabilitarCampos();
	}

	public void actualizarTabla() {

		// Obtener listado Usuarios

		SortedSet<String> listado = manager.obtenerListaNombresUsuarios();

		// Crear modelo

		ModeloGestionUsuarios modelo = new ModeloGestionUsuarios(listado);

		// Aplicar modelo

		view.getTable().setModel(modelo);

	}

	public void cerrarVentana(){
		// Guardar usuarios
		manager.guardarUsuarios();

		view.getPreviousWindow().setEnabled(true);
		view.dispose();
	}

	public void habilitarCampos() {
		view.getTextFieldMail().setEnabled(true);
		view.getTextFieldNombreReal().setEnabled(true);
		view.getComboBoxTipoUsuario().setEnabled(true);

	}
	
	public void deshabilitarCampos() {
		view.getTextFieldMail().setEnabled(false);
		view.getTextFieldNombreReal().setEnabled(false);
		view.getComboBoxTipoUsuario().setEnabled(false);

	}

	public void actualizarCampos() {
		// Obtener nombre de usuario de la tabla
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada >= 0){
			String usuarioSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);

			// Obtener objeto usuario del manager
			Usuario usuario = manager.obtenerUsuarioPorNombre(usuarioSeleccionado);

			// Actualizar campos

			view.getTextFieldMail().setText(usuario.getCorreoElectronico());
			view.getTextFieldNombreReal().setText(usuario.getNombreReal());
			view.getTextFieldNombreUsuario().setText(usuario.getNombreUsuario());
			switch(usuario.getTipo()){
			case Usuario.TIPO_NORMAL:
				view.getComboBoxTipoUsuario().setSelectedItem("Normal");
				break;
			case Usuario.TIPO_ADMINISTRADOR:
				view.getComboBoxTipoUsuario().setSelectedItem("Administrador");
				break;
			}
		}
		else{
			view.getTextFieldMail().setText("");
			view.getTextFieldNombreReal().setText("");
			view.getTextFieldNombreUsuario().setText("");
			view.getComboBoxTipoUsuario().setSelectedIndex(-1);
		}

	}

	public void agregarUsuario(){
		NuevoUsuario dialog = new NuevoUsuario();
		dialog.setVisible(true);
		dialog.setAlwaysOnTop(true);
		dialog.setPreviousWindow(view);

		view.setEnabled(false);
	}

	public void eliminarUsuario(){
		// Obtener nombre de usuario de la tabla
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada >= 0){

			String usuarioSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);

			// borrar
			manager.eliminarUsuarioPorNombre(usuarioSeleccionado);

			// actualizar

			actualizarTabla();
			
			// Deshabilitar campos
			deshabilitarCampos();

		}
	}

	public void cambiarNombre(){
		// Obtener usuario
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada > -1){
			String usuarioSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);
			Usuario usuario = manager.obtenerUsuarioPorNombre(usuarioSeleccionado);

			// Obtener datos
			String nombreReal = view.getTextFieldNombreReal().getText();

			// Actualizar
			usuario.setNombreReal(nombreReal);
		}

	}
	public void cambiarMail(){
		// Obtener usuario
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada > -1){
			String usuarioSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);
			Usuario usuario = manager.obtenerUsuarioPorNombre(usuarioSeleccionado);

			// Obtener datos
			String mail = view.getTextFieldMail().getText();

			// Actualizar
			usuario.setCorreoElectronico(mail);
		}
	}
	public void cambiarTipo(){
		// Obtener usuario
		int filaSeleccionada = view.getTable().getSelectedRow();
		if(filaSeleccionada > -1){
			String usuarioSeleccionado = (String) view.getTable().getModel().getValueAt(filaSeleccionada, 0);
			Usuario usuario = manager.obtenerUsuarioPorNombre(usuarioSeleccionado);

			// Obtener datos
			int tipo = view.getComboBoxTipoUsuario().getSelectedIndex();

			// Actualizar
			usuario.setTipo(tipo);
		}

	}
}