package controlador;

import java.util.HashSet;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.ListSelectionModel;

import modelo.Grupo;
import modelo.Usuario;
import utils.ManagerUsuarios;
import utils.tablemodels.ModeloGestionUsuarios;
import vista.GestionUsuariosGrupo;

public class GestionUsuariosGrupoController {
    private static GestionUsuariosGrupoController INSTANCE = null;
    private GestionUsuariosGrupo view;
    private Grupo grupo;
    private SortedSet<String> usuariosIzquierda;
    private SortedSet<String> usuariosDerecha;
    private ManagerUsuarios manager;
    private int ultimaFilaSeleccionadaIzquierda;
    private int ultimaFilaSeleccionadaDerecha;
 
    // Private constructor suppresses 
    private GestionUsuariosGrupoController() {}
 
    // creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new GestionUsuariosGrupoController();
        }
    }
 
    public static GestionUsuariosGrupoController getInstance() {
        createInstance();
        return INSTANCE;
    }
    
    public void inicializar(GestionUsuariosGrupo view, Grupo grupo){
    	this.view = view;
    	this.grupo = grupo;
    	
    	ultimaFilaSeleccionadaIzquierda = -1;
    	ultimaFilaSeleccionadaDerecha = -1;
    	
    	usuariosIzquierda = new TreeSet();
    	usuariosDerecha = new TreeSet();
    	
    	// Usuarios tabla derecha
		for(Usuario u: grupo.getUsuarios()){
			usuariosDerecha.add(u.getNombreUsuario());
		}
		
		// Usuarios tabla izquierda
		manager = ManagerUsuarios.getInstance();
		usuariosIzquierda.addAll(manager.obtenerListaNombresUsuarios());
		
		for(Usuario u: grupo.getUsuarios()){
			usuariosIzquierda.remove(u.getNombreUsuario());
		}
		
    	
    	actualizarTablas();
    	habilitarDisparos();
    }
    
    public void cerrarVentana(){
		view.getPreviousWindow().setEnabled(true);
		view.dispose();
    }
    
    public void actualizarTablas(){
    	actualizarTablaIzquierda();
    	actualizarTablaDerecha();
    }

	private void actualizarTablaDerecha() {
		
		// Selection model
		ultimaFilaSeleccionadaDerecha = view.getTableLUG().getSelectedRow();
		
		// Modelo
		ModeloGestionUsuarios modelo = new ModeloGestionUsuarios(usuariosDerecha);
		
		// Tabla
		view.getTableLUG().setModel(modelo);
		
		if(ultimaFilaSeleccionadaDerecha < 0)
			ultimaFilaSeleccionadaDerecha = 0;
		if(ultimaFilaSeleccionadaDerecha >= view.getTableLUG().getRowCount())
			ultimaFilaSeleccionadaDerecha = view.getTableLUG().getRowCount()-1;
		
		ListSelectionModel sm = view.getTableLUG().getSelectionModel();
		sm.setSelectionInterval(ultimaFilaSeleccionadaDerecha, ultimaFilaSeleccionadaDerecha);
		
	}

	private void actualizarTablaIzquierda() {
		
		// Selection model
		ultimaFilaSeleccionadaIzquierda = view.getTableLUNG().getSelectedRow();
		
		// Modelo
		ModeloGestionUsuarios modelo = new ModeloGestionUsuarios(usuariosIzquierda);
		
		// Tabla
		view.getTableLUNG().setModel(modelo);
		
		if(ultimaFilaSeleccionadaIzquierda < 0)
			ultimaFilaSeleccionadaIzquierda = 0;
		if(ultimaFilaSeleccionadaIzquierda >= view.getTableLUNG().getRowCount())
			ultimaFilaSeleccionadaIzquierda = view.getTableLUNG().getRowCount()-1;
		
		ListSelectionModel sm = view.getTableLUNG().getSelectionModel();
		sm.setSelectionInterval(ultimaFilaSeleccionadaIzquierda, ultimaFilaSeleccionadaIzquierda);
		
	}
	
	public void moverDerecha(){
		
		int filaSeleccionada = view.getTableLUNG().getSelectedRow();
		
		if(filaSeleccionada > -1){
			
			String nombreUsuario = (String) view.getTableLUNG().getModel().getValueAt(filaSeleccionada, 0);
			
			Usuario usuario = manager.obtenerUsuarioPorNombre(nombreUsuario);
			
			// Eliminar de la izquierda
			usuariosIzquierda.remove(usuario.getNombreUsuario());
		
			// Aadir a la derecha
			usuariosDerecha.add(usuario.getNombreUsuario());
			
			// Aadir al grupo
			grupo.getUsuarios().add(usuario);
			grupo.getIdsUsuarios().add(usuario.getId());
			
			// Actualizar tablas
			actualizarTablas();			
		}
	}
	
	public void moverIzquierda(){
		
		int filaSeleccionada = view.getTableLUG().getSelectedRow();
		
		if(filaSeleccionada > -1){
			
			String nombreUsuario = (String) view.getTableLUG().getModel().getValueAt(filaSeleccionada, 0);
			
			Usuario usuario = manager.obtenerUsuarioPorNombre(nombreUsuario);
			
			// Eliminar de la derecha
			usuariosDerecha.remove(usuario.getNombreUsuario());
		
			// Aadir a la izquierda
			usuariosIzquierda.add(usuario.getNombreUsuario());
			
			// Eliminar del  grupo
			grupo.getUsuarios().remove(usuario);
			grupo.getIdsUsuarios().remove(usuario.getId());
			
			// Actualizar tablas
			actualizarTablas();
			
		}
	}
	
	public void habilitarDisparos(){
		
		int filaIzquierda = view.getTableLUNG().getSelectedRow();
		int filaDerecha = view.getTableLUG().getSelectedRow();
		
		if(filaIzquierda < 0)
			view.getDgg3button().setEnabled(false);
		else
			view.getDgg3button().setEnabled(true);
		
		if(filaDerecha < 0)
			view.getDgg4button().setEnabled(false);
		else
			view.getDgg4button().setEnabled(true);
		
	}
	
	
    
    
}