package controlador;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import modelo.Area;
import modelo.Esquema;
import modelo.Estructura;

import utils.ManagerAreas;
import utils.ManagerEstructuras;
import utils.tablemodels.ModeloLocalizarAreas;
import utils.tablemodels.ModeloLocalizarEsquemas;
import utils.tablemodels.RXTable;
import vista.EditorAreas;
import vista.EditorEsquemas;
import vista.EditorEstructuras;
import vista.EditorTipoPresentacion;
import vista.LocalizadorAreasEsquemas;

public class LocalizadorAreasEsquemasController {


	private static LocalizadorAreasEsquemasController INSTANCE = null;
	private LocalizadorAreasEsquemas view;
	private ManagerAreas managerAreas = ManagerAreas.getInstance();

	// Private constructor suppresses 
	private LocalizadorAreasEsquemasController() {}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new LocalizadorAreasEsquemasController();
		}
	}

	public static LocalizadorAreasEsquemasController getInstance() {
		createInstance();
		return INSTANCE;
	}

	public void inicializar(LocalizadorAreasEsquemas view){

		this.view = view;

		actualizarTablas();

		habilitarDisparos();
	}

	public void habilitarDisparos() {

		int filaAreas = view.getTableAreas().getSelectedRow();

		int filaEsquemas = view.getTableEsquemas().getSelectedRow();

		if(filaAreas < 0){
			view.getDla2button().setEnabled(false);
			view.getDla3button().setEnabled(false);
			view.getDle1button().setEnabled(false);
		}
		else{
			view.getDla2button().setEnabled(true);
			view.getDla3button().setEnabled(true);
			view.getDle1button().setEnabled(true);
		}
		if(filaEsquemas < 0){
			view.getDle2button().setEnabled(false);
			view.getDle3button().setEnabled(false);
		}
		else{
			view.getDle2button().setEnabled(true);
			view.getDle3button().setEnabled(true);
		}

	}

	public void actualizarTablas() {

		actualizarTablaAreas();
		actualizarTablaEsquemas();

	}

	public void actualizarTablaEsquemas() {
		// CAMBIAR POR EL USUARIO ACTUAL

		// Fila seleccionada en el listado de esquemas
		int selectedRow = view.getTableEsquemas().getSelectedRow();


		int filaSeleccionada = view.getTableAreas().getSelectedRow();
		SortedSet listadoEsquemas = new TreeSet();


		// Si hay fila seleccionada, se aaden las areas, si no, se le pasa al modelo un conjunto vaco
		if(filaSeleccionada > -1){

			String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaSeleccionada, 0);

			Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

			listadoEsquemas.addAll(areaSeleccionada.getEsquemas());

		}

		// modelo
		ModeloLocalizarEsquemas modelo = new ModeloLocalizarEsquemas(listadoEsquemas);

		// tabla
		view.getTableEsquemas().setModel(modelo);

		// Volver a poner bien la fila seleccionada
		selectedRow = Math.min(view.getTableEsquemas().getRowCount()-1, selectedRow);

		view.getTableEsquemas().getSelectionModel().setSelectionInterval(selectedRow, selectedRow);

	}

	private void actualizarTablaAreas() {

		int selectedRow = view.getTableAreas().getSelectedRow();

		// CAMBIAR POR EL USUARIO ACTUAL

		SortedSet listadoAreas = managerAreas.listarNombresAreas();

		//System.out.println(listadoAreas.size());

		// modelo
		ModeloLocalizarAreas modelo = new ModeloLocalizarAreas(listadoAreas);

		// tabla
		view.getTableAreas().setModel(modelo);

		// Volver a poner bien la fila seleccionada
		selectedRow = Math.min(view.getTableAreas().getRowCount()-1, selectedRow);
		view.getTableAreas().getSelectionModel().setSelectionInterval(selectedRow, selectedRow);

	}

	public void cerrarVentana(){
		view.getPreviousWindow().setEnabled(true);
		view.dispose();

		EditorEstructurasController eecontroller = EditorEstructurasController.getInstance();
		eecontroller.actualizarArbolLAE();

		// guardar
		managerAreas.guardarAreas();

	}

	public void modificarArea(){

		int filaSeleccionada = view.getTableAreas().getSelectedRow();
		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaSeleccionada, 0);
		Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		EditorAreas ea = new EditorAreas(view, areaSeleccionada, EditorAreasController.MODIFICAR);
		ea.setVisible(true);
		ea.setAlwaysOnTop(true);
		ea.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
	}

	public void nuevaArea(){
		int i = 2;
		String nombre = "Area";

		while(managerAreas.existeNombreArea(nombre)){
			nombre = "Area"+i;
			i++;
		}

		Area nueva = managerAreas.nuevaArea(nombre);

		EditorAreas ea = new EditorAreas(view, nueva, EditorAreasController.NUEVA);
		ea.setVisible(true);
		ea.setAlwaysOnTop(true);
		ea.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
	}

	public void eliminarArea(){

		// Busqueda
		int filaSeleccionada = view.getTableAreas().getSelectedRow();

		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaSeleccionada, 0);

		Area area = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		// Aviso
		if(area != null){
			int opcion = JOptionPane.showConfirmDialog(
					view,
					"Eliminar el rea " + area.getNombre() + " ?\nImportante: Se borrarn todas las estructuras del rea. Posteriormente no ser posible deshacer esta operacin",
					"Eliminar",
					JOptionPane.YES_NO_OPTION);


			if(opcion == JOptionPane.YES_OPTION){
				// Eliminar rea
				managerAreas.eliminarAreaPorID(area.getId());

				// Eliminar estructuras

				ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();
				for(Estructura e: area.getEstructuras()){
					managerEstructuras.eliminar(e);
				}

				// Actualizar tablas

				actualizarTablas();
			}
		}

	}

	public void nuevoEsquema(){

		// Area seleccionada
		int filaSeleccionada = view.getTableAreas().getSelectedRow();
		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaSeleccionada, 0);
		Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		int i = 2;
		String nombre = "Esquema";
		boolean existe = true;

		// Generacin de nombre no repetido
		while(existe){
			existe = false;
			for(Esquema esquema: areaSeleccionada.getEsquemas()){
				if(esquema.getNombre().equals(nombre))
					existe = true;
			}
			if(existe){
				nombre = "Esquema"+i;
				i++;
			}
		}

		// Creacin
		Esquema esquema = new Esquema(nombre, areaSeleccionada.getId());

		// Insercin en rea
		areaSeleccionada.getEsquemas().add(esquema);

		EditorEsquemas ee = new EditorEsquemas(view, esquema, EditorEsquemasController.NUEVA);
		ee.setVisible(true);
		ee.setAlwaysOnTop(true);
		ee.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);	
	}

	public void modificarEsquema(){

		// Area seleccionada
		int filaSeleccionada = view.getTableAreas().getSelectedRow();
		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaSeleccionada, 0);
		Area areaSeleccionada = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		// Esquema seleccionado
		filaSeleccionada = view.getTableEsquemas().getSelectedRow();
		String nombreEsquemaSeleccionado = (String) view.getTableEsquemas().getValueAt(filaSeleccionada, 0);
		Esquema esquemaSeleccionado = null;

		for(Esquema e: areaSeleccionada.getEsquemas()){
			if(e.getNombre().equals(nombreEsquemaSeleccionado)){
				esquemaSeleccionado = e;
			}
		}

		EditorEsquemas ee = new EditorEsquemas(view, esquemaSeleccionado, EditorEsquemasController.NUEVA);
		ee.setVisible(true);
		ee.setAlwaysOnTop(true);
		ee.setEnabled(true);

		// Desactivacin del JFrame actual
		view.setEnabled(false);
	}

	public void eliminarEsquema(){

		// Busqueda
		int filaSeleccionada = view.getTableAreas().getSelectedRow();
		String nombreAreaSeleccionada = (String) view.getTableAreas().getValueAt(filaSeleccionada, 0);
		Area area = managerAreas.obtenerAreaPorNombre(nombreAreaSeleccionada);

		filaSeleccionada = view.getTableEsquemas().getSelectedRow();
		String nombreEsquemaSeleccionado = (String) view.getTableEsquemas().getValueAt(filaSeleccionada, 0);
		Esquema esquema = null;
		for(Esquema e: area.getEsquemas()){
			if(e.getNombre().equals(nombreEsquemaSeleccionado))
				esquema = e;
		}

		// Aviso
		if(area != null){
			int opcion = JOptionPane.showConfirmDialog(
					view,
					"Eliminar el esquema " + esquema.getNombre() + " del rea " + area.getNombre() + " ?",
					"Eliminar",
					JOptionPane.YES_NO_OPTION);


			if(opcion == JOptionPane.YES_OPTION){
				// Eliminar esquema
				area.getEsquemas().remove(esquema);

				// Actualizar tablas

				actualizarTablas();
			}
		}

	}

}
