package controlador.undo;

import java.util.ArrayList;

import modelo.Componente;
import modelo.ComponenteCompuesta;

// TODO: Auto-generated Javadoc
/**
 * The Class DesplazarHaciaDerechaCommand.
 * Encapsulacin de la operacin desplazamiento de una componente hacia un nivel de anidamiento superior
 */
public class DesplazarHaciaDerechaCommand extends Command{

	/** Componente sobre la cual se aplica el comando. */
	private Componente componente;
	
	/** Parent de la componente. */
	private ComponenteCompuesta parent;
	
	/** Conjunto de hermanos de la componente. */
	private ArrayList<Componente> componentesParent = null;
	
	/** Tipo de desplazamiento hacia la derecha que se ha realizado:
	 * 1: se ha integrado en la componente superior
	 * 2: se ha integrado en la componente inferior
	 * 3: se convierte la componente superior en una componente compuesta y se integra la componente actual en ella. */
	private int tipoDesplazamiento;
	
	/* (non-Javadoc)
	 * @see controller.undo.Command#ejecutar()
	 */
	@Override
	public void ejecutar() {
		tipoDesplazamiento = componente.desplazarHaciaDerecha();
		
	}

	/* (non-Javadoc)
	 * @see controller.undo.Command#deshacer()
	 */
	@Override
	public void deshacer() {
		switch(tipoDesplazamiento){
		case 1:
			componente.desplazarHaciaIzquierda();
			
			break;
		case 2:
			componente.desplazarHaciaIzquierda();
			componente.desplazarHaciaArriba();
			break;
		case 3:
			componente.desplazarHaciaIzquierda();
			int posicion = componente.getParent().getPosicionComponente(componente.getId()); // eliminar la componente compleja que se crea alrededor de la elemental
			componente.getParent().eliminarComponente(posicion-1);
		}
	}

	/**
	 * Recupera componente.
	 *
	 * @return el componente
	 */
	public Componente getComponente() {
		return componente;
	}

	/**
	 * Establece componente.
	 *
	 * @param componente el nuevo componente
	 */
	public void setComponente(Componente componente) {
		this.componente = componente;
	}

	/**
	 * Recupera tipo desplazamiento.
	 *
	 * @return el tipo desplazamiento
	 */
	public int getTipoDesplazamiento() {
		return tipoDesplazamiento;
	}

	/**
	 * Establece tipo desplazamiento.
	 *
	 * @param tipoDesplazamiento el nuevo tipo desplazamiento
	 */
	public void setTipoDesplazamiento(int tipoDesplazamiento) {
		this.tipoDesplazamiento = tipoDesplazamiento;
	}
	
	

}
