package controlador.undo;

import modelo.Componente;
import modelo.ComponenteCompuesta;

// TODO: Auto-generated Javadoc
/**
 * The Class ModificarClaseCommand.
 * Encapsulacin de la operacin de modificacin de de la clase de una componente, y de su operacin opuesta.
 */
public class ModificarClaseCommand extends Command{

	/** Componente sobre la cual se aplica el comando. */
	private Componente componente;
	
	/** Componente creada a la hora de aplicar la operacin de ejecutar. Necesario para la operacin de rehacer. */
	private Componente componenteCreada = null;
	
	/** Posicin de la componente. */
	private int posicion;
	
	/* Se realiza la ejecucin a partir del parent, ya que es imposible hacerlo pasandole la componente
	 * Al modificar la clase de la componente, el objeto se elimina y se crea uno nuevo de la otra clase,
	 * por lo tanto no sirve para realizar la operacin redo.
	 * (non-Javadoc)
	 * @see controller.undo.Command#ejecutar()
	 */
	
	@Override
	public
	void ejecutar() {
		if(componenteCreada == null){ // se ejecuta por primera vez
			posicion = componente.getParent().getPosicionComponente(componente.getId());
			componente.modificarClase();
		}
		else{ // si se llamase otra vez a modificarClase, se volvera a crear un objeto nuevo, con lo cual no se podra realizar ninguna accin de undo/redo ms sobre el objeto
			componente.getParent().eliminarComponente(posicion);
			componente.getParent().agregarComponente(componenteCreada, posicion);
			componente.getParent().actualizarSimbolos();
		}
	}

	/* (non-Javadoc)
	 * @see controller.undo.Command#deshacer()
	 */
	@Override
	public
	void deshacer() {
		// Se restaura la componente borrada
		componenteCreada = componente.getParent().getComponentes().get(posicion);
		componente.getParent().eliminarComponente(posicion);
		componente.getParent().agregarComponente(componente, posicion);
		componente.getParent().actualizarSimbolos();
	}

	/**
	 * Recupera componente.
	 *
	 * @return el componente
	 */
	public Componente getComponente() {
		return componente;
	}

	/**
	 * Establece componente.
	 *
	 * @param componente el nuevo componente
	 */
	public void setComponente(Componente componente) {
		this.componente = componente;
	}

	/**
	 * Recupera posicion.
	 *
	 * @return el posicion
	 */
	public int getPosicion() {
		return posicion;
	}

	/**
	 * Establece posicion.
	 *
	 * @param posicion el nuevo posicion
	 */
	public void setPosicion(int posicion) {
		this.posicion = posicion;
	}

	

}
