package controlador.undo;

import modelo.ComponenteCompuesta;

// TODO: Auto-generated Javadoc
/**
 * The Class ModificarEstructuraDerivadaCommand.
 * Modificar una estructura derivada ya existente
 */
public class ModificarEstructuraDerivadaCommand extends Command{

	/** Componente en la cual se insertar la estructura derivada. */
	private ComponenteCompuesta componente;
	
	/** Estructura derivada antes de realizar las modificacciones */
	private ComponenteCompuesta estructuraDerivadaOriginal;
	
	/** Estructura derivada despus de realizar las modificaciones*/
	private ComponenteCompuesta estructuraDerivadaModificada;
	
	/** Posicin en la componnte en la cual est la estructura derivada. */
	private int posicion;
	
	/* (non-Javadoc)
	 * @see controlador.undo.Command#ejecutar()
	 */
	@Override
	public void ejecutar() {
		estructuraDerivadaOriginal = (ComponenteCompuesta) componente.getComponentes().get(posicion);
		componente.eliminarComponente(posicion);
		componente.agregarComponente(estructuraDerivadaModificada, posicion);
	}

	/* (non-Javadoc)
	 * @see controlador.undo.Command#deshacer()
	 */
	@Override
	public void deshacer() {
		estructuraDerivadaModificada = (ComponenteCompuesta) componente.getComponentes().get(posicion);
		componente.eliminarComponente(posicion);
		componente.agregarComponente(estructuraDerivadaOriginal, posicion);		
	}

	/**
	 * Recupera componente.
	 *
	 * @return el componente
	 */
	public ComponenteCompuesta getComponente() {
		return componente;
	}

	/**
	 * Establece componente.
	 *
	 * @param componente el nuevo componente
	 */
	public void setComponente(ComponenteCompuesta componente) {
		this.componente = componente;
	}

	/**
	 * Recupera estructura derivada original.
	 *
	 * @return el estructura derivada original
	 */
	public ComponenteCompuesta getEstructuraDerivadaOriginal() {
		return estructuraDerivadaOriginal;
	}

	/**
	 * Establece estructura derivada original.
	 *
	 * @param estructuraDerivadaOriginal el nuevo estructura derivada original
	 */
	public void setEstructuraDerivadaOriginal(
			ComponenteCompuesta estructuraDerivadaOriginal) {
		this.estructuraDerivadaOriginal = estructuraDerivadaOriginal;
	}

	/**
	 * Recupera estructura derivada modificada.
	 *
	 * @return el estructura derivada modificada
	 */
	public ComponenteCompuesta getEstructuraDerivadaModificada() {
		return estructuraDerivadaModificada;
	}

	/**
	 * Establece estructura derivada modificada.
	 *
	 * @param estructuraDerivadaModificada el nuevo estructura derivada modificada
	 */
	public void setEstructuraDerivadaModificada(
			ComponenteCompuesta estructuraDerivadaModificada) {
		this.estructuraDerivadaModificada = estructuraDerivadaModificada;
	}

	/**
	 * Recupera posicion.
	 *
	 * @return el posicion
	 */
	public int getPosicion() {
		return posicion;
	}

	/**
	 * Establece posicion.
	 *
	 * @param posicion el nuevo posicion
	 */
	public void setPosicion(int posicion) {
		this.posicion = posicion;
	}

}
