package datos;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SealedObject;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;

import modelo.Area;
import modelo.Estructura;
import modelo.Grupo;
import modelo.Usuario;

public class AreasDAO {
	private static AreasDAO INSTANCE = null;
	private static final String password = "kepa.adam@gmail.com";

	// Private constructor suppresses 
	private AreasDAO() {}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new AreasDAO();
		}
	}

	public static AreasDAO getInstance() {
		createInstance();
		return INSTANCE;
	}

	public void guardarAreas(ArrayList<Area> areas) throws Exception{

		// Preproceso
		/* A la hora de serializar el objeto, no se tienen en cuenta las colecciones de estructuras y de estructuras importadas.
		 * Por eso, a la hora de serializar el objeto, ste se resetea y se vuelve a cargar su estado ms tarde.
		 */

		ArrayList<ArrayList<Estructura>> auxEstructuras = new ArrayList<ArrayList<Estructura>>();

		for(Area a: areas){
			auxEstructuras.add(a.getEstructuras());
			a.setEstructuras(null);
		}


		// Create Key
		byte key[] = password.getBytes();
		DESKeySpec desKeySpec = new DESKeySpec(key);
		SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
		SecretKey secretKey = keyFactory.generateSecret(desKeySpec);

		// Create Cipher
		Cipher desCipher = Cipher.getInstance("DES/ECB/PKCS5Padding");
		desCipher.init(Cipher.ENCRYPT_MODE, secretKey);

		// Seal object

		SealedObject sealedObject = new SealedObject(areas, desCipher);

		// Serialize to a file
		String direccion = System.getProperty("user.dir")+"\\data\\";
		String file = "area.dat";
		File path = new File(direccion);
		if(!path.exists())
			path.mkdirs();

		File fichero = new File(direccion + file);

		ObjectOutput out = new ObjectOutputStream(new FileOutputStream(fichero));
		out.writeObject(sealedObject);
		out.close();

		// Postproceso

		for(int i = 0; i < areas.size(); i++){
			areas.get(i).setEstructuras(auxEstructuras.get(i));
		}

	}

	public ArrayList<Area> cargarAreas() throws Exception{
		// Create Key
		byte key[] = password.getBytes();
		DESKeySpec desKeySpec = new DESKeySpec(key);
		SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
		SecretKey secretKey = keyFactory.generateSecret(desKeySpec);

		// Deserialize from a file
		String direccion = System.getProperty("user.dir")+"\\data\\area.dat";
		File file = new File(direccion);
		if(file.exists()){
			ObjectInputStream in = new ObjectInputStream(new FileInputStream(file));

			// Deserialize the object
			SealedObject sealedObject = (SealedObject) in.readObject();
			in.close();

			// Unseal object
			ArrayList<Area> areas = (ArrayList<Area>)sealedObject.getObject(secretKey);

			return areas;
		}
		else{
			return new ArrayList<Area>();
		}
	}



}