package datos;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.PropiedadLinea;
import modelo.Selector;
import modelo.SelectorLinea;
import controlador.EditorEstructurasController;
// TODO: Auto-generated Javadoc
/**
 * Data Access Object
 * La clase se ocupa de la serializacin y deserializacin de los objetos de la capa de negocio.
 */
public class EstructuraDAO {

	/** nica instancia del DAO. */
	private static EstructuraDAO INSTANCE = null;

	/** Controlador del Editor de Estructuras. */
	EditorEstructurasController controller;

	/**
	 * Gets the single instance of EditorEstructurasDAO.
	 *
	 * @return single instance of EditorEstructurasDAO
	 */
	public static EstructuraDAO getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}
	// Patrn singleton
	/**
	 * Creacin de la instancia.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new EstructuraDAO();
		}
	}

	/**
	 * Inicializa un objeto de tipo editor estructuras dao.
	 */
	public EstructuraDAO() {
		controller = EditorEstructurasController.getInstance();
	}


	public Boolean guardarEstructura(Estructura e){

		String direccion = System.getProperty("user.dir")+"\\EstructurasXML\\";
		String filename = e.getNombre()+".xml";

		File ruta = new File(direccion);
		if(!ruta.exists())
			ruta.mkdirs();

		File file = new File(direccion+filename);

		JAXBContext contextObj;
		try {
			contextObj = JAXBContext.newInstance(Estructura.class, Componente.class, ComponenteCompuesta.class, ComponenteElemental.class, PropiedadLinea.class, ComponenteLinea.class, Linea.class, SelectorLinea.class, Selector.class);


			Marshaller marshallerObj = contextObj.createMarshaller();
			marshallerObj.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
			marshallerObj.marshal(e, file);
			return true;
		} catch (JAXBException ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
			return false;
		}

	}


	public Estructura abrirEstructura(File selectedFile) {
		try {
			JAXBContext jc = JAXBContext.newInstance (Estructura.class, Componente.class, ComponenteCompuesta.class, ComponenteElemental.class, PropiedadLinea.class, ComponenteLinea.class, Linea.class, SelectorLinea.class, Selector.class);

			Unmarshaller u = jc.createUnmarshaller ();

			Estructura estructura = (Estructura) u.unmarshal (selectedFile);

			// Referencias
			estructura.getRaiz().setEstructura(estructura);
			estructura.getRaiz().recuperarReferencias();

			return estructura;
		} catch (JAXBException e) {
			e.printStackTrace ();
			return null;
		}

	}

	public ArrayList<Estructura> abrirEstructuras(){

		ArrayList<Estructura> listaEstructuras = new ArrayList<Estructura>();
		
		String direccion = System.getProperty("user.dir")+"\\EstructurasXML\\";

		// Si no existe, se crea
		File ruta = new File(direccion);
		if(!ruta.exists()){
			ruta.mkdirs();
			ruta = new File(direccion);
		}

		/*
		 * Comprueba todos los ficheros de la carpeta e intenta abrir todos los que sean ficheros
		 * y tengan extensin xml
		 */
		File [] files = ruta.listFiles();

		for(File f: files){

			if(f.isFile()){
				
				//Comprobacin de la extensin
				String fname="";
				String ext="";
				int mid= f.getName().lastIndexOf(".");
				ext=f.getName().substring(mid+1,f.getName().length());

				if(ext.equals("xml")){
					Estructura e = this.abrirEstructura(f);
					if(e != null)
						listaEstructuras.add(e);
				}

			}
		}
		
		return listaEstructuras;

	}
	
	public void eliminarEstructura(Estructura e){
		
		String direccion = System.getProperty("user.dir")+"\\EstructurasXML\\";
		String filename = e.getNombre()+".xml";
		
		File file = new File(direccion+filename);
		
		if(file.exists()){
			file.delete();
		}
		
	}
}
