package datos;

import java.io.File;
import java.util.ArrayList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import modelo.Acabado;
import modelo.Aspecto;
import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.Presentacion;
import modelo.Propiedad;
import modelo.PropiedadLinea;
import modelo.Selector;
import modelo.SelectorLinea;
import modelo.TipoDatos;
import modelo.tipoDatos.Alfanumerico;
import modelo.tipoDatos.Binario;
import modelo.tipoDatos.Numerico;
import modelo.tipoDatos.Rango;
import controlador.EditorEstructurasController;
// TODO: Auto-generated Javadoc
/**
 * Data Access Object
 * La clase se ocupa de la serializacin y deserializacin de los objetos de la capa de negocio.
 */
public class PresentacionesDAO {
	
	/** nica instancia del DAO. */
	private static PresentacionesDAO INSTANCE = null;
	
	/**
	 * Gets the single instance of PresentacionesDAO.
	 *
	 * @return single instance of PresentacionesDAO
	 */
	public static PresentacionesDAO getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}
	// Patrn singleton
	/**
	 * Creacin de la instancia.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new PresentacionesDAO();
		}
	}
	

	/**
	 * Inicializa un objeto de tipo presentaciones dao.
	 */
	public PresentacionesDAO() {

	}

		


	/**
	 * Guardar presentacin en el workspace de la aplicacin
	 *
	 * @param presentacion la presentacin a guardar
	 * @return si se ha realizado con xito
	 */
	public Boolean guardarPresentacin(Presentacion presentacion){
		
		String direccion = System.getProperty("user.dir")+"\\Presentaciones\\";
		String filename = presentacion.getNombre()+".xml";

		File file = new File(direccion+filename);
		JAXBContext contextObj;
		try {
			contextObj = JAXBContext.newInstance(Presentacion.class, Propiedad.class, 
					TipoDatos.class, Aspecto.class, Acabado.class, Numerico.class, Alfanumerico.class, Binario.class, Rango.class);


			Marshaller marshallerObj = contextObj.createMarshaller();
			marshallerObj.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
			marshallerObj.marshal(presentacion, file);
			return true;
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

	}
	

	/**
	 * Cargar presentacion de disco
	 *
	 * @param file el fichero que contiene la presentacin a cargar
	 * @return la presentacin deserializada
	 */
	public Presentacion cargarPresentacion(File file) {
	
		
        try {
            JAXBContext jc = JAXBContext.newInstance(Presentacion.class, Propiedad.class, 
					TipoDatos.class, Aspecto.class, Acabado.class, Numerico.class, Alfanumerico.class, Binario.class, Rango.class);

            Unmarshaller u = jc.createUnmarshaller ();

           Presentacion presentacion = (Presentacion) u.unmarshal (file);
           


           return presentacion;
       } catch (JAXBException e) {
           e.printStackTrace ();
           return null;
       }
		
	}
	
	/**
	 * Cargar todas las presentaciones disponibles en el workspace de la aplicacin.
	 *
	 * @return listado con todas las presentaciones disponibles.
	 */
	public ArrayList<Presentacion> cargarPresentaciones(){
		ArrayList<Presentacion> presentaciones = new ArrayList<Presentacion>();
		
		String direccion = System.getProperty("user.dir")+"\\Presentaciones\\";
		File carpeta = new File(direccion);
		
		// Si la carpeta no existe, se crea
		if(!carpeta.exists()){
			carpeta.mkdirs();
		}

		File[] ficheros = carpeta.listFiles();
		
		for(File f: ficheros){
			if(f.isFile()){
				presentaciones.add(this.cargarPresentacion(f));
			}
		}
		
		
		return presentaciones;
	}
	
	/**
	 * Eliminar presentacion.
	 *
	 * @param nombre the nombre
	 */
	public void eliminarPresentacion(String nombre){
		
		String direccion = System.getProperty("user.dir")+"\\Presentaciones\\";
		String filename = nombre+".xml";
		
		File file = new File(direccion+filename);
		
		if(file.exists()){
			file.delete();
		}
		
	}
}
