package datos;

import java.io.File;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.Propiedad;
import modelo.PropiedadLinea;
import modelo.Selector;
import modelo.SelectorLinea;
import modelo.TipoDatos;
import modelo.prototypes.PrototipoListaPropiedades;
import modelo.tipoDatos.Alfanumerico;
import modelo.tipoDatos.Binario;
import modelo.tipoDatos.Numerico;
import modelo.tipoDatos.Rango;
import controlador.EditorEstructurasController;
// TODO: Auto-generated Javadoc
/**
 * Data Access Object
 * La clase se ocupa de la serializacin y deserializacin de los objetos de la capa de negocio.
 */
public class PrototipoPropiedadesDAO {

	/** nica instancia del DAO. */
	private static PrototipoPropiedadesDAO INSTANCE = null;

	/** Controlador del Editor de Estructuras. */
	EditorEstructurasController controller;

	/**
	 * Gets the single instance of PrototipoPropiedadesDAO.
	 *
	 * @return single instance of PrototipoPropiedadesDAO
	 */
	public static PrototipoPropiedadesDAO getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}
	// Patrn singleton
	/**
	 * Creacin de la instancia.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new PrototipoPropiedadesDAO();
		}
	}

	/**
	 * Inicializa un objeto de tipo PrototipoPropiedadesDAO.
	 */
	public PrototipoPropiedadesDAO() {
		controller = EditorEstructurasController.getInstance();
	}




	/**
	 * Guardar prototipo en la carpeta configuracin del workspace de la aplicacin
	 *
	 * @param prototipo el prototipo a guardar
	 * @return si se ha realizado la operacin con xito.
	 */
	public Boolean guardarPrototipo(PrototipoListaPropiedades prototipo){
		String direccion = System.getProperty("user.dir")+"\\configuracion\\";
		String filename = "listadoPropiedades.xml";

		File file = new File(direccion+filename);
		File ruta = new File(direccion);
		if(!ruta.exists()){
			ruta.mkdirs();
		}

		JAXBContext contextObj;
		try {
			contextObj = JAXBContext.newInstance (PrototipoListaPropiedades.class, Propiedad.class,
					TipoDatos.class, Alfanumerico.class, Binario.class, Numerico.class, Rango.class);


			Marshaller marshallerObj = contextObj.createMarshaller();
			marshallerObj.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
			marshallerObj.marshal(prototipo, file);
			return true;
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

	}

	/**
	 * Abrir prototipo desde workspace.
	 *
	 * @return el listado de propiedades por defecto de la aplicacin
	 * @throws JAXBException si falla la deserializacin del objeto
	 */
	public PrototipoListaPropiedades abrirPrototipo() throws JAXBException {

		/* 
		 * La excepcin se propaga, y as en el prototype, en el caso de que no exista lista de
		 * propiedades, sta se genera de forma automtica. En el caso de que la lista est corrompida
		 * se vuelve a sobreescribir.
		 */
		
		String direccion = System.getProperty("user.dir")+"\\configuracion\\";
		String filename = "listadoPropiedades.xml";

		File file = new File(direccion+filename);


		JAXBContext jc = JAXBContext.newInstance (PrototipoListaPropiedades.class, Propiedad.class,
				TipoDatos.class, Alfanumerico.class, Binario.class, Numerico.class, Rango.class);

		Unmarshaller u = jc.createUnmarshaller ();

		PrototipoListaPropiedades plp = (PrototipoListaPropiedades) u.unmarshal (file);

		return plp;

	}


}
