package modelo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.UUID;

import utils.ManagerEstructuras;

public class Area implements Comparable, Serializable{
	
	public final static int SIN_ACCESO = 0;
	public final static int LECTURA = 1;
	public final static int ESCRITURA = 2;
	
	private String id = UUID.randomUUID().toString();
	private String nombre;
	private String descripcion;
	
	private Calendar fechaCreacion;
	private Calendar fechaModificacion;
	private String idUsuarioCreacion;
	private String idUsuarioModificacion;
	
	HashMap <String, Integer> permisos; // <id_grupo, permiso>
	
	private ArrayList <Estructura> estructuras;
	private ArrayList <String> idEstructuras;
	
	private ArrayList <String> idEstructurasImportadas;
	
	private ArrayList <Esquema> esquema;


	public Area() { 
		estructuras = new ArrayList<Estructura>();
		idEstructuras = new ArrayList<String>();
		
		idEstructurasImportadas = new ArrayList<String>();
		
		esquema = new ArrayList<Esquema>();
		
		permisos = new HashMap<String, Integer>();
	}


	public String getNombre() {
		return nombre;
	}


	public void setNombre(String nombre) {
		this.nombre = nombre;
	}


	public ArrayList<Estructura> getEstructuras() {
		return estructuras;
	}


	public void setEstructuras(ArrayList<Estructura> estructuras) {
		this.estructuras = estructuras;
	}


	public String getId() {
		return id;
	}


	public void setId(String id) {
		this.id = id;
	}


	public ArrayList<String> getIdEstructuras() {
		return idEstructuras;
	}


	public void setIdEstructuras(ArrayList<String> idEstructuras) {
		this.idEstructuras = idEstructuras;
	}


	public ArrayList<Estructura> obtenerEstructurasImportadas() {
		// EN VEZ DE MANTERNER LA INFORMACION, SE DERIVA CADA VEZ	
		ArrayList<Estructura> importadas = new ArrayList<Estructura>();
		ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();
		
		for(int i = idEstructurasImportadas.size()-1; i >= 0; i--){
			
			String id = idEstructurasImportadas.get(i);
			
			Estructura e = managerEstructuras.getEstructurasID().get(id);
			
			if(e != null){
				importadas.add(e);
			}
			else{ // Significa que la estructura importada ya no existe!!!
				idEstructurasImportadas.remove(i);
			}
		}
		// Se devuelve al orden original
		Collections.reverse(importadas);
		return importadas;
	}


	public ArrayList<String> getIdEstructurasImportadas() {
		return idEstructurasImportadas;
	}


	public void setIdEstructurasImportadas(
			ArrayList<String> idEstructurasImportadas) {
		this.idEstructurasImportadas = idEstructurasImportadas;
	}


	public ArrayList<Esquema> getEsquemas() {
		return esquema;
	}


	public void setEsquemas(ArrayList<Esquema> esquema) {
		this.esquema = esquema;
	}


	public Area(String nombre) {
		super();
		this.nombre = nombre;
		
		estructuras = new ArrayList<Estructura>();
		idEstructuras = new ArrayList<String>();
		
		idEstructurasImportadas = new ArrayList<String>();
		
		esquema = new ArrayList<Esquema>();
		
		permisos = new HashMap<String, Integer>();
		
		fechaCreacion = Calendar.getInstance();
		fechaModificacion = Calendar.getInstance();
	}


	public HashMap<String, Integer> getPermisos() {
		return permisos;
	}


	public void setPermisos(HashMap<String, Integer> permisos) {
		this.permisos = permisos;
	}


	public String getDescripcion() {
		return descripcion;
	}


	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}


	public Calendar getFechaCreacion() {
		return fechaCreacion;
	}


	public void setFechaCreacion(Calendar fechaCreacion) {
		this.fechaCreacion = fechaCreacion;
	}


	public Calendar getFechaModificacion() {
		return fechaModificacion;
	}


	public void setFechaModificacion(Calendar fechaModificacion) {
		this.fechaModificacion = fechaModificacion;
	}


	public String getIdUsuarioCreacion() {
		return idUsuarioCreacion;
	}


	public void setIdUsuarioCreacion(String idUsuarioCreacion) {
		this.idUsuarioCreacion = idUsuarioCreacion;
	}


	public String getIdUsuarioModificacion() {
		return idUsuarioModificacion;
	}


	public void setIdUsuarioModificacion(String idUsuarioModificacion) {
		this.idUsuarioModificacion = idUsuarioModificacion;
	}



	@Override
	public int compareTo(Object arg0) {
		Area area = (Area) arg0;
		return this.getNombre().compareTo( area.getNombre());
	}


	@Override
	public String toString() {
		return this.getNombre();
	}
	
	public ArrayList<String> listarNombresEsquemas(){
		ArrayList<String> nombres = new ArrayList<String>();
		
		for(Esquema e: this.getEsquemas()){
			nombres.add(e.getNombre());
		}
		
		return nombres;
	}
	
	
	
}
