package modelo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.regex.Pattern;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

import utils.ManagerComponentes;

import controlador.EditorEstructurasController;

// TODO: Auto-generated Javadoc
/**
 * The Class ComponenteCompuesta.
 */
@XmlAccessorType(XmlAccessType.NONE)
public class ComponenteCompuesta extends Componente implements Cloneable{

	/** Constante AGREGACION. */
	public static final int AGREGACION = 0;

	/** Constante ESPECIALIZACION. */
	public static final int ESPECIALIZACION = 1;

	/** Constante SIN_ITERADOR. */
	public static final int SIN_ITERADOR = 0;

	/** Constante ITERADOR_INTERIOR. */
	public static final int ITERADOR_INTERIOR = 1;

	/** Constante ITERADOR_EXTERIOR. */
	public static final int ITERADOR_EXTERIOR = 2;

	/** Tipo de iteracin de la componente compleja. (0 = sin iterador, 1 = iterador interior, 2 = iterador exterior)*/
	@XmlAttribute(name="Iteracion")
	private int tipoIteracion;

	/** Tipo de la componente compleja (0 = Agregacion, 1 = Iteracion). */
	@XmlAttribute(name="Tipo")
	private int tipoComponente;

	/** Lnea de cabecera de la componente. */
	@XmlElement(name="Cabecera")
	private ComponenteLinea cabecera;

	/** Lnea que contiene los smbolos iniciales de la componente. */
	@XmlElement(name="LineaSimbolosInicio")
	private ComponenteLinea simbolosInicio;

	/** Lnea que contiene los smbolos finales de la componente. */
	@XmlElement(name="LineaSimbolosFin")
	private ComponenteLinea simbolosFin;

	/** Conjunto de componentes contenidas en la componente compleja actual. */
	@XmlElement(name="Componentes")
	private ArrayList<Componente> componentes;

	/** Selector. nicamente usado cuando la componente es derivada. */
	@XmlElement(name="Selector")
	private Selector selector = null;

	/**
	 * Inicializa un objeto de tipo componente compuesta.
	 */
	public ComponenteCompuesta() {
		super();
	}


	/**
	 * Inicializa un objeto de tipo componente compuesta.
	 *
	 * @param nombre el nombre de la componente compuesta
	 * @param estructura la estructura a la cual pertenece la componente
	 * @param parent componente compleja en la cual est contenida la componente actual
	 */
	public ComponenteCompuesta(String nombre, Estructura estructura, ComponenteCompuesta parent) {
		super();
		super.setNombre(nombre);
		super.setAnidamiento(0);
		super.setDerivada(false);
		setEstructura(estructura);
		super.setOpcional(false);
		super.setParent(parent);
		this.tipoComponente = AGREGACION;
		this.tipoIteracion = SIN_ITERADOR;


		this.componentes = new ArrayList<Componente>();

		// Agregar una componente elemental inicial
		//		ComponenteElemental ce = new ComponenteElemental("CompoElemental", getEstructura(), this);
		//		ce.setAnidamiento(getAnidamiento()+1);
		//		componentes.add(ce);

		// Creacin de la cabecera
		cabecera = new ComponenteLinea(this, getAnidamiento());
		cabecera.setContenido(nombre);
		cabecera.setIgualFinal(true);

		// Creacin de la lnea de smbolos inicial

		simbolosInicio = new ComponenteLinea(this, getAnidamiento()+1);

		// Creacin de la lnea de smbolos final

		simbolosFin = new ComponenteLinea(this, getAnidamiento()+1);

		// Inicialmente, es una agregacn

		setAgregacion();

		//		System.out.println(cabecera);
		//		System.out.println(simbolosInicio);
		//		System.out.println(simbolosFin);

	}



	/**
	 * Realiza un redibujado de los smbolos tras una operacin editorial que haya afectado a la estructura.
	 */
	public void actualizarSimbolos(){
		if(tipoComponente == AGREGACION) setAgregacion();
		else setEspecializacion();
	}



	/**
	 * Agregar componente.
	 *
	 * @param componente the componente
	 * @param posicion the posicion
	 */
	public void agregarComponente(Componente componente, int posicion) { // id de la componente a continuacin de la cual se inserta el elemento
		componente.setAnidamiento(getAnidamiento()+1);
		componente.setParent(this);
		componente.setEstructura(getEstructura());
		componentes.add(posicion, componente);
		actualizarSimbolos();

	}






	/**
	 * Agregacin de una nueva componente elemental a la componente compleja actual.
	 *
	 * @param posicion la posicin en la cual se inserta la componente
	 */
	public void agregarComponente(int posicion) { // id de la componente a continuacin de la cual se inserta el elemento
		// Creacin de una nueva componente elemental
		ComponenteElemental ce = new ComponenteElemental("", getEstructura(), this);
		ce.setAnidamiento(getAnidamiento()+1);

		// Aadir la componente al rbol
		componentes.add(posicion, ce);
		actualizarSimbolos();

	}


	/* (non-Javadoc)
	 * @see modelo.Componente#clone()
	 */
	@Override
	public Object clone() throws CloneNotSupportedException {
		// Creacin de la componente y acualizacin de las propiedades
		ComponenteCompuesta ncc = new ComponenteCompuesta(getNombre(), getEstructura(), getParent());
		ncc.setAnidamiento(getAnidamiento());
		ncc.setTipoComponente(getTipoComponente());
		ncc.setTipoIteracion(getTipoIteracion());
		ncc.setOpcional(getOpcional());
		ncc.setIdComponenteOrigen(getIdComponenteOrigen());
		ncc.setIdEstructuraOrigen(getIdEstructuraOrigen());
		ncc.setId(getId());
		// La componente clonada es derivada!!
		ncc.setDerivada(getDerivada());

		//		System.out.println("ComponenteClonada: " + ncc.getNombre());
		//		System.out.println("\tCompoID: " + ncc.getIdComponenteOrigen());
		//		System.out.println("\tEstructuraID: " + ncc.getIdEstructuraOrigen());

		// Clonacin de las lneas
		ComponenteLinea nuevaCabecera = (ComponenteLinea) cabecera.clone();
		ComponenteLinea nuevaSimbolosInicio = (ComponenteLinea) simbolosInicio.clone();
		ComponenteLinea nuevaSimbolosFin = (ComponenteLinea) simbolosFin.clone();

		// Actualizacion referencia lineas clonadas

		nuevaCabecera.setComponente(ncc);
		nuevaSimbolosInicio.setComponente(ncc);
		nuevaSimbolosFin.setComponente(ncc);

		// Esablecer Lineas
		ncc.setCabecera(nuevaCabecera);
		ncc.setSimbolosInicio(nuevaSimbolosInicio);
		ncc.setSimbolosFin(nuevaSimbolosFin);

		// Clonacin de las subcomponentes
		for(int i = 0; i < getComponentes().size(); i++){
			Componente c = (Componente) getComponentes().get(i).clone();
			c.setParent(ncc);
			ncc.getComponentes().add(c);
		}
		// Clonacin de las propiedades
		for(int i = 0; i < getPropiedades().size(); i++){
			PropiedadLinea p = (PropiedadLinea) getPropiedades().get(i).clone();
			p.setComponente(ncc);
			ncc.getPropiedades().add(p);
		}

		// Se mantiene el selector, sin clonacin
		ncc.setSelector(getSelector());

		// Actualizar smbolos
		ncc.actualizarSimbolos();
		return ncc;
	}


	/**
	 * Eliminar componente.
	 *
	 * @param posicion the posicion
	 */
	public void eliminarComponente(int posicion) {
		// Eliminacin de la componente
		componentes.remove(posicion);
		actualizarSimbolos();

	}


	/* (non-Javadoc)
	 * @see modelo.Componente#generarReferenciasDerivacion(modelo.Componente)
	 */
	@Override
	public void generarReferenciasDerivacion(Componente componente) {
		super.generarReferenciasDerivacion(componente);

		// propagacin
		ComponenteCompuesta cc = (ComponenteCompuesta) componente;

		for(int i = 0; i < getComponentes().size(); i++){
			getComponentes().get(i).generarReferenciasDerivacion(cc.getComponentes().get(i));
		}
	}

	/**
	 * Recupera cabecera.
	 *
	 * @return el cabecera
	 */
	public ComponenteLinea getCabecera() {
		return cabecera;
	}

	/**
	 * Recupera componentes.
	 *
	 * @return el componentes
	 */
	public ArrayList<Componente> getComponentes() {
		return componentes;
	}

	/* (non-Javadoc)
	 * @see logic.Componente#getLineas()
	 */
	@Override

	// Recorre todas las subcomponentes de forma ordenada, y adems, aade sus propias lneas
	public ArrayList<Linea> getLineas() {
		ArrayList<Linea> lineas = new ArrayList<Linea>();

		// Aade la lnea de cabecera y la lnea inicial de smbolos
		lineas.add(cabecera);
		lineas.add(simbolosInicio);

		// Si es derivada y hay selector, aade las lneas del selector
		if(getDerivada()&&(getSelector()!=null)){
			lineas.addAll(getSelector().getLineas());
		}

		// Recorre el subrbol de componentes
		for(int i = 0; i < componentes.size(); i++){
			lineas.addAll(componentes.get(i).getLineas());
		}
		// Aade la lnea final de smbolos
		lineas.add(simbolosFin);
		return lineas;
	}


	/* (non-Javadoc)
	 * @see modelo.Componente#getLineasSinSimbolos()
	 */
	@Override
	public ArrayList<ComponenteLinea> getLineasSinSimbolos() {

		ArrayList<ComponenteLinea> lineas = new ArrayList<ComponenteLinea>();

		// Se aade a si mismo
		lineas.add(cabecera);

		// Aade subcomponentes
		for(int i = 0; i < componentes.size(); i++){
			lineas.addAll(componentes.get(i).getLineasSinSimbolos());
		}

		return lineas;
	}


	/**
	 * Recupera posicion componente en el subrbol a partir de la componente compleja actual.
	 *
	 * @param ID the iD
	 * @return el posicion componente, o -1 en el caso de que sta no est includa en el subrbol
	 */
	public int getPosicionComponente(String ID){
		for(int i = 0; (i < componentes.size()); i++)
			if(componentes.get(i).getId().equals(ID))
				return i;
		return -1;
	}


	/**
	 * Recupera selector.
	 *
	 * @return el selector
	 */
	public Selector getSelector() {
		return selector;
	}


	/**
	 * Recupera simbolos fin.
	 *
	 * @return el simbolos fin
	 */
	public ComponenteLinea getSimbolosFin() {
		return simbolosFin;
	}


	/**
	 * Recupera simbolos inicio.
	 *
	 * @return el simbolos inicio
	 */
	public ComponenteLinea getSimbolosInicio() {
		return simbolosInicio;
	}

	/**
	 * Recupera tipo componente.
	 *
	 * @return el tipo componente
	 */
	public int getTipoComponente() {
		return tipoComponente;
	}


	/**
	 * Recupera tipo iteracion.
	 *
	 * @return el tipo iteracion
	 */
	public int getTipoIteracion() {
		return tipoIteracion;
	}	

	/* (non-Javadoc)
	 * @see modelo.Componente#insertarEnDiccionario()
	 */
	@Override
	public void insertarEnDiccionario() {
		ManagerComponentes d = ManagerComponentes.getInstance();
		d.getComponentes().put(getId(), this);
		for(int i = 0; i < getComponentes().size(); i++){
			getComponentes().get(i).insertarEnDiccionario();
		}
	}


	/* (non-Javadoc)
	 * @see logic.Componente#modicarOpcionalidad()
	 */
	@Override
	public void modicarOpcionalidad() {
		if(getOpcional() == true){ 
			setOpcional(false);
			simbolosFin.setOpcFinal(false);
			cabecera.setOpcInicial(false);
		}
		else{
			setOpcional(true);
			simbolosFin.setOpcFinal(true);
			cabecera.setOpcInicial(true);
		}

	}


	/* (non-Javadoc)
	 * @see logic.Componente#modificarClase()
	 */
	@Override
	public void modificarClase() {

		// Sustitucin de una componente compuesta por una componente elemental
		// Bsqueda de si mismo en parent
		int posicion = getParent().getPosicionComponente(getId());
		// Borrado
		getParent().getComponentes().remove(posicion);
		// Creacin de componente elemental que sustituir a la componente compleja
		ComponenteElemental ce = new ComponenteElemental(getNombre(), getEstructura(), getParent());
		ce.setAnidamiento(getAnidamiento());
		// Mantener la opcionalidad
		if(getOpcional()) ce.modicarOpcionalidad();
		// Las propiedades se mantienen
		ce.setPropiedades(getPropiedades());
		// Mantener el id
		ce.setId(getId());
		// Insercin de la componente elemental
		getParent().getComponentes().add(posicion, ce);
		// Insercin de las subcomponentes de la componente compleja a eliminar
		int pos = posicion + 1; // siguiente posicin a insertar
		for(int j = 0; j < getComponentes().size(); j++){
			Componente c = getComponentes().get(j);
			c.setAnidamiento(getAnidamiento()); // actualizacin del nivel de anidamiento
			getParent().getComponentes().add(pos, c);
			pos++;
		}
		// Actualizar los smbolos de parent
		getParent().actualizarSimbolos();
	}

	/**
	 * Modificar iteracion.
	 */
	public void modificarIteracion(){
		if(tipoIteracion == this.SIN_ITERADOR){
			tipoIteracion = this.ITERADOR_INTERIOR;
			simbolosInicio.setIterInicial(true);
			simbolosFin.setIterFinal(true);
		}
		else if (tipoIteracion == this.ITERADOR_INTERIOR){
			tipoIteracion = this.ITERADOR_EXTERIOR;
			simbolosInicio.setIterInicial(false);
			cabecera.setIterInicial(true);
		}
		else{ // tipoIteracion == this.ITERADOR_EXTERIOR
			tipoIteracion = this.SIN_ITERADOR;
			cabecera.setIterInicial(false);
			simbolosFin.setIterFinal(false);
		}
	}

	/**
	 * Convertir una agregacin en especializacin, y viceversa.
	 */
	public void modificarTipo(){
		if(tipoComponente == AGREGACION)
			tipoComponente = ESPECIALIZACION;
		else 
			tipoComponente = AGREGACION;
		actualizarSimbolos();
	}

	/* (non-Javadoc)
	 * @see modelo.Componente#obtenerSubArbol()
	 */
	@Override
	public DefaultMutableTreeNode obtenerSubArbol() {
		DefaultMutableTreeNode nodo = new DefaultMutableTreeNode(this);

		for(int i = 0; i < this.getComponentes().size(); i++){
			nodo.add(getComponentes().get(i).obtenerSubArbol());
		}
		return nodo;
	}

	/* (non-Javadoc)
	 * @see logic.Componente#recuperarReferencias()
	 */
	@Override
	public void recuperarReferencias() {
		//System.out.println("CCompuesta.recuperar");
		// Recuperar la referencia de los objetos de tipo LineaComponente
		cabecera.setComponente(this);
		simbolosInicio.setComponente(this);
		simbolosFin.setComponente(this);

		//Recuperar las referencias del selector
		if(selector != null){

			selector.setCcSelector(this);

			// Lineas selector

			for(int i = 0; i < selector.getLineas().size(); i++){
				selector.getLineas().get(i).setSelector(selector);
			}

			// Restaurar componentes

			ManagerComponentes diccionario = ManagerComponentes.getInstance();
			diccionario.actualizar();

			selector.setComponentes(new ArrayList<Componente>());

			for(int i = 0; i < selector.getComponentesID().size(); i++){
				String id = selector.getComponentesID().get(i);
				Componente c = diccionario.getComponentes().get(id);
				selector.getComponentes().add(c);
			}
		}

		// Recuperar las referencias de las propiedades
		for(int i = 0; i < getPropiedades().size(); i++){
			getPropiedades().get(i).setComponente(this);
		}

		// Recuperar la referencia de las sub componentes.

		if(componentes == null){
			componentes = new ArrayList<Componente>();
		}

		for(int i = 0; i < componentes.size(); i++){
			componentes.get(i).setParent(this);
			componentes.get(i).setEstructura(getEstructura());
			componentes.get(i).recuperarReferencias();
		}


	}


	/**
	 * Actua sobre la componente actual y las subcomponentes, de forma que stas representen una agregacin.
	 */
	public void setAgregacion(){
		// modificar el tipo propio
		//setTipoComponente(AGREGACION);
		// modificar los simbolos propios
		simbolosInicio.setEspInicial(false);
		simbolosInicio.setAgrInicial(true);

		simbolosFin.setEspFinal(false);
		simbolosFin.setAgrFinal(true);

		// Por si se borra el smbolo + o | al hacer el borrado en simbolosFin.clear()
		//if(getParent()!=null) getParent().actualizarSimbolos();

		// modificar los simbolos de todas las componentes

		for(int i = 0; i < componentes.size(); i++){ 
			if(componentes.get(i).getClass() == ComponenteElemental.class){
				((ComponenteElemental) componentes.get(i)).getLinea().setEspIntermedio(false);
				if(i < componentes.size() - 1) // la ltima lnea no tiene smbolos
					((ComponenteElemental) componentes.get(i)).getLinea().setAgrIntermedio(true);
				else
					((ComponenteElemental) componentes.get(i)).getLinea().setAgrIntermedio(false);
			}
			if(componentes.get(i).getClass() == ComponenteCompuesta.class){
				((ComponenteCompuesta) componentes.get(i)).getSimbolosFin().setEspIntermedio(false);
				if(i < componentes.size() - 1) // la ltima lnea no tiene smbolos
					((ComponenteCompuesta) componentes.get(i)).getSimbolosFin().setAgrIntermedio(true);
				else
					((ComponenteCompuesta) componentes.get(i)).getSimbolosFin().setAgrIntermedio(false);
			}	
		}


	}


	/* (non-Javadoc)
	 * @see logic.Componente#setAnidamiento(int)
	 */
	@Override
	public void setAnidamiento(int anidamiento) {
		super.setAnidamiento(anidamiento);

		// actualizacin de lneas propias
		cabecera.setAnidamiento(anidamiento);
		simbolosInicio.setAnidamiento(anidamiento+1);
		simbolosFin.setAnidamiento(anidamiento+1);

		// actualizacin de las componentes contenidas

		for(int i = 0; i < componentes.size(); i++){
			componentes.get(i).setAnidamiento(anidamiento+1);
		}

		// actualizacin del selector

		if(getSelector()!=null)
			selector.setAnidamiento(anidamiento+1);

	}


	/**
	 * Establece cabecera.
	 *
	 * @param cabecera el nuevo cabecera
	 */
	public void setCabecera(ComponenteLinea cabecera) {
		this.cabecera = cabecera;
	}


	/**
	 * Establece componentes.
	 *
	 * @param componentes el nuevo componentes
	 */
	public void setComponentes(ArrayList<Componente> componentes) {
		this.componentes = componentes;
	}


	/**
	 * Actua sobre la componente actual y las subcomponentes, de forma que stas representen una especializacin.
	 */
	public void setEspecializacion(){
		// modificar el tipo propio
		//setTipoComponente(ESPECIALIZACION);
		// modificar los simbolos propios
		simbolosInicio.setAgrInicial(false);
		simbolosInicio.setEspInicial(true);

		simbolosFin.setAgrFinal(false);
		simbolosFin.setEspFinal(true);

		// Por si se borra el smbolo + o | al hacer el borrado en simbolosFin.clear()
		//if(getParent()!=null) getParent().actualizarSimbolos();

		// modificar los simbolos de todas las componentes

		for(int i = 0; i < componentes.size(); i++){
			if(componentes.get(i).getClass() == ComponenteElemental.class){
				((ComponenteElemental) componentes.get(i)).getLinea().setAgrIntermedio(false);
				if(i < componentes.size() - 1) // la ltima lnea no tiene smbolos
					((ComponenteElemental) componentes.get(i)).getLinea().setEspIntermedio(true);
				else
					((ComponenteElemental) componentes.get(i)).getLinea().setEspIntermedio(false);
			}
			if(componentes.get(i).getClass() == ComponenteCompuesta.class){
				((ComponenteCompuesta) componentes.get(i)).getSimbolosFin().setAgrIntermedio(false);
				if(i < componentes.size() - 1) // la ltima lnea no tiene smbolos
					((ComponenteCompuesta) componentes.get(i)).getSimbolosFin().setEspIntermedio(true);
				else
					((ComponenteCompuesta) componentes.get(i)).getSimbolosFin().setEspIntermedio(false);
			}	
		}
	}


	/* (non-Javadoc)
	 * @see logic.Componente#setNombre(java.lang.String)
	 */
	@Override
	public void setNombre(String nombre) {
		super.setNombre(nombre);
		cabecera.setContenido(nombre);
		if(getParent() == null) // el nombre de la componente compuesta es tambin el nombre de la estructura
			getEstructura().setNombre(nombre);
	}



	/**
	 * Establece selector.
	 *
	 * @param selector el nuevo selector
	 */
	public void setSelector(Selector selector) {
		this.selector = selector;
	}


	/**
	 * Establece simbolos fin.
	 *
	 * @param simbolosFin el nuevo simbolos fin
	 */
	public void setSimbolosFin(ComponenteLinea simbolosFin) {
		this.simbolosFin = simbolosFin;
	}


	/**
	 * Establece simbolos inicio.
	 *
	 * @param simbolosInicio el nuevo simbolos inicio
	 */
	public void setSimbolosInicio(ComponenteLinea simbolosInicio) {
		this.simbolosInicio = simbolosInicio;
	}


	/**
	 * Establece tipo componente.
	 *
	 * @param tipoComponente el nuevo tipo componente
	 */
	public void setTipoComponente(int tipoComponente) {
		this.tipoComponente = tipoComponente;
	}


	/**
	 * Establece tipo iteracion.
	 *
	 * @param tipoIteracion el nuevo tipo iteracion
	 */
	public void setTipoIteracion(int tipoIteracion) {
		this.tipoIteracion = tipoIteracion;
	}


	/* (non-Javadoc)
	 * @see modelo.Componente#validar(modelo.Presentacion)
	 */
	public String validar(Presentacion presentacion) {
		StringBuffer sb = new StringBuffer();

		for(PropiedadLinea pl: this.getPropiedades()){
			for(Propiedad p: presentacion.getPropiedades()){
				if(pl.getNombre().equals(p.getNombre())){

					// Comprobacin de patrn

					String valor = pl.getValor();
					String patron = p.getTipoDatos().getExpresionRegular();

					boolean resultado = Pattern.matches(patron, valor);
					if(resultado == false){
						sb.append("Celda " + this.getNombre() + "." + p.getAlias() + ": " +
								"Se esperaba un valor " + p.getTipoDatos().getDescripcion()+"\n");
					}

					// Comprobacin de no nulo

					if(valor.trim().equals("")){
						if(p.getTipoDatos().isNulo() == false){
							sb.append("Celda " + this.getNombre() + "." + p.getAlias() + ": " +
									"Esta propiedad es obligatoria. Debe insertar un valor en esta celda.\n");
						}
					}

				}
			}
		}
		for(Componente c: this.getComponentes()){
			sb.append(c.validar(presentacion));
		}

		return sb.toString();
	}


	@Override
	public void setEstructura(Estructura estructura) {
		super.estructura = estructura;
		if(componentes != null){
			for(Componente c: componentes){
				c.setEstructura(estructura);
			}
		}

	}






}
