package modelo;

import java.util.ArrayList;
import java.util.regex.Pattern;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

import utils.ManagerComponentes;

// TODO: Auto-generated Javadoc
/**
 * The Class ComponenteElemental.
 */
@XmlAccessorType(XmlAccessType.NONE)
public class ComponenteElemental extends Componente implements Cloneable{


	/** Lnea que representa a la componente en la tabla del editor. */
	@XmlElement(name = "Linea")
	private ComponenteLinea linea;

	/**
	 * Inicializa un objeto de tipo componente elemental.
	 */
	public ComponenteElemental() {
		super();
	}

	/**
	 * Inicializa un objeto de tipo componente elemental.
	 *
	 * @param nombre el nombre de la componente elemental
	 * @param estructura estructura a la que pertenece la componente
	 * @param parent componente compuesta en la cual est contenida la componente actual
	 */
	public ComponenteElemental(String nombre, Estructura estructura, ComponenteCompuesta parent) {
		super();


		super.setNombre(nombre);
		super.setAnidamiento(0);
		super.setDerivada(false);
		setEstructura(estructura);
		super.setOpcional(false);
		super.setParent(parent);
		

		// Creacin de la lnea || Falta modificar y poner que el tipo es el tipo de la componente compleja que contiene a esta

		linea = new ComponenteLinea(this, getAnidamiento());
		//linea.setAgrIntermedio(true);
		linea.setContenido(getNombre());

	}

	/**
	 * Recupera linea.
	 *
	 * @return el linea
	 */
	public ComponenteLinea getLinea() {
		return linea;
	}

	/**
	 * Establece linea.
	 *
	 * @param linea el nuevo linea
	 */
	public void setLinea(ComponenteLinea linea) {
		this.linea = linea;
	}



	/* (non-Javadoc)
	 * @see logic.Componente#getLineas()
	 */
	@Override
	// Retorna nicamente su propia lnea
	public ArrayList<Linea> getLineas() {
		ArrayList<Linea> lineas = new ArrayList<Linea>();
		lineas.add(linea);
		return lineas;
	}

	/* (non-Javadoc)
	 * @see modelo.Componente#getLineasSinSimbolos()
	 */
	@Override
	public ArrayList<ComponenteLinea> getLineasSinSimbolos() {
		ArrayList<ComponenteLinea> lineas = new ArrayList<ComponenteLinea>();
		lineas.add(linea);
		return lineas;
	}

	/* (non-Javadoc)
	 * @see logic.Componente#setAnidamiento(int)
	 */
	@Override
	public void setAnidamiento(int anidamiento) {
		super.setAnidamiento(anidamiento);
		linea.setAnidamiento(anidamiento);	
	}

	/* (non-Javadoc)
	 * @see logic.Componente#modicarOpcionalidad()
	 */
	@Override
	public void modicarOpcionalidad() {
		if(getOpcional() == true){ 
			setOpcional(false);
			linea.setOpcFinal(false);
			linea.setOpcInicial(false);
		}
		else{
			setOpcional(true);
			linea.setOpcFinal(true);
			linea.setOpcInicial(true);
		}

	}

	/* (non-Javadoc)
	 * @see logic.Componente#setNombre(java.lang.String)
	 */
	@Override
	public void setNombre(String nombre) {
		super.setNombre(nombre);
		linea.setContenido(nombre);
	}

	/* (non-Javadoc)
	 * @see logic.Componente#modificarClase()
	 */
	@Override
	public void modificarClase() {
		// Bsqueda de si mismo en parent
		int posicion = getParent().getPosicionComponente(getId());
		// Borrado
		getParent().getComponentes().remove(posicion);
		// Creacin de componente compleja que sustituir a la componente elemental
		ComponenteCompuesta cc = new ComponenteCompuesta(getNombre(), getEstructura(), getParent());
		cc.setAnidamiento(getAnidamiento());
		// Mantener la opcionalidad
		if(getOpcional()) cc.modicarOpcionalidad();
		// Las propiedades se mantienen
		cc.setPropiedades(getPropiedades());
		// Mantener el id
		cc.setId(getId());
		// Insercin de la componente compleja
		getParent().getComponentes().add(posicion, cc);
		// Actualizar los smbolos de parent
		getParent().actualizarSimbolos();
		
	}

	/* (non-Javadoc)
	 * @see logic.Componente#recuperarReferencias()
	 */
	@Override
	public void recuperarReferencias() {
		//System.out.println("ComponenteElemental.recuperar");
		linea.setComponente(this);
		
		// Recuperar las referencias de las propiedades
		for(int i = 0; i < getPropiedades().size(); i++){
			getPropiedades().get(i).setComponente(this);
		}
	}

	/* (non-Javadoc)
	 * @see modelo.Componente#obtenerSubArbol()
	 */
	@Override
	public DefaultMutableTreeNode obtenerSubArbol() {
		return new DefaultMutableTreeNode(this);
	}

	/* (non-Javadoc)
	 * @see modelo.Componente#clone()
	 */
	@Override
	public Object clone() throws CloneNotSupportedException {
		ComponenteElemental nce = new ComponenteElemental(getNombre(), getEstructura(), getParent());
		nce.setAnidamiento(getAnidamiento());
		nce.setOpcional(getOpcional());
		nce.setIdComponenteOrigen(getIdComponenteOrigen());
		nce.setIdEstructuraOrigen(getIdEstructuraOrigen());
		nce.setId(getId());
		nce.setDerivada(getDerivada());
		
		// Clonacin de lneas
		ComponenteLinea nl = (ComponenteLinea) getLinea().clone();
		nl.setComponente(nce);
		nce.setLinea(nl);
		
		//Actualizacin referncia linea
		nce.getLinea().setComponente(nce);
		
		// Clonacin de las propiedades
		for(int i = 0; i < getPropiedades().size(); i++){
			PropiedadLinea p = (PropiedadLinea) getPropiedades().get(i).clone();
			p.setComponente(nce);
			nce.getPropiedades().add(p);
		}
		return nce;
	}


	/* (non-Javadoc)
	 * @see modelo.Componente#insertarEnDiccionario()
	 */
	@Override
	public void insertarEnDiccionario() {
		ManagerComponentes d = ManagerComponentes.getInstance();
		d.getComponentes().put(getId(), this);
	}

	/* (non-Javadoc)
	 * @see modelo.Componente#validar(modelo.Presentacion)
	 */
	@Override
	public String validar(Presentacion presentacion) {
		StringBuffer sb = new StringBuffer();
		
		for(PropiedadLinea pl: this.getPropiedades()){
			for(Propiedad p: presentacion.getPropiedades()){
				if(pl.getNombre().equals(p.getNombre())){

					// Comprobacin del patrn
					String valor = pl.getValor();
					String patron = p.getTipoDatos().getExpresionRegular();
					boolean resultado = Pattern.matches(patron, valor);
					if(resultado == false){
						sb.append("Celda " + this.getNombre() + "." + p.getAlias() + ": " +
								"Se esperaba un valor " + p.getTipoDatos().getDescripcion()+"\n");
					}
					
					// Comprobacin de no nulo
					
					if(valor.trim().equals("")){
						if(p.getTipoDatos().isNulo() == false){
							sb.append("Celda " + this.getNombre() + "." + p.getAlias() + ": " +
									"Esta propiedad es obligatoria. Debe insertar un valor en esta celda.\n");
						}
					}
					
				}
			}
		}
		
		return sb.toString();
	}

	@Override
	public void setEstructura(Estructura estructura) {
		super.estructura = estructura;
	}

	



}
