package modelo;

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlRootElement;

import datos.EstructuraDAO;


// TODO: Auto-generated Javadoc
/**
 * Una estructura es un conjunto de componentes que el usuario puede editar.
 */
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="Estructura")
public class Estructura implements Cloneable, Comparable{
	
	/** Activacin del modo depuracin. */
	private static final boolean DEBUG = true;
	
	/** El nombre de la estructura. */
	@XmlAttribute(name="Nombre")
	private String nombre;
	
	/** The array ordenado de las lneas que forman la tabla. */
	private ArrayList<Linea> aalc;
	
	/** La raz del rbol de componentes. */
	@XmlElement(name="Raiz")
	private ComponenteCompuesta raiz;
	
	/** Identificador. */
	@XmlID
	private String id = UUID.randomUUID().toString();
	
	@XmlAttribute(name="Descripcion")
	private String descripcion;
	
	/** Fecha de creacin de la presentacin. */
	@XmlAttribute(name="FechaCreacion")
	private Calendar creacion;

	/** Fecha de la ltima modificacin de la presentacin. */
	@XmlAttribute(name="FechaModificacion")
	private Calendar modificacion;
	
	@XmlAttribute(name="UsuarioCreacion")
	private String idUsuarioCreacion;
	
	@XmlAttribute(name="UsuarioModificacion")
	private String idUsuarioModificacion;
	
	@XmlAttribute(name="Area")
	private String idArea;
	
	/**
	 * Inicializa un objeto de tipo estructura.
	 */
	public Estructura() { // Exigido por hibernate
	}

	/**
	 * Inicializa un objeto de tipo estructura.
	 *
	 * @param nombre the nombre
	 */
	public Estructura(String nombre) {
		super();
		
//		if(DEBUG){
//			System.out.println("Estructura.constructor: " + nombre);
//		}
		
		this.nombre = nombre;
		
		// Creacin de la estructura que mantiene indexadas las lneas (AALC = Array asociativo linea-componente)
		this.aalc = new ArrayList<Linea>();
		
		// Creacin de la componente compleja de la raz
		this.raiz = new ComponenteCompuesta(nombre, this, null);
		
		this.descripcion = "";
		
		this.creacion = Calendar.getInstance();
		this.modificacion = Calendar.getInstance();
	}

	
	/**
	 * Abrir estructura.
	 *
	 * @param fileEstructura the file estructura
	 */
//	public void abrirEstructura(File fileEstructura) {
//		// Capa de datos
//		EstructuraDAO dao = EstructuraDAO.getInstance();
//		Estructura estructura = dao.abrirEstructura(fileEstructura);
//		estructura.recuperarReferencias();
//		
//		// Asignacion de los campos a si mismo
//		this.setAalc(estructura.getAalc());
//		this.setNombre(estructura.getNombre());
//		this.setRaiz(estructura.getRaiz());
//		this.setId(estructura.getId());
//		this.setDescripcion(estructura.getDescripcion());
//	}
	
	/* Actualiza la estructura de datos "aalc" recorriendo todo el arbol de componentes, e insertando en la estructura
	 * las lines conforme las va recorriendo
	 */
	/**
	 * Actualizar lineas. Recorre el rbol de componentes para recuperar todas las lneas que forman la tabla
	 */
	public void actualizarLineas(){
		aalc = raiz.getLineas();
	}

	/**
	 * Recupera aalc.
	 *
	 * @return el aalc
	 */
	public ArrayList<Linea> getAalc() {
		return aalc;
	}



	/**
	 * Recupera id.
	 *
	 * @return el id
	 */
	public String getId() {
		return id;
	}

	/**
	 * Recupera nombre.
	 *
	 * @return el nombre
	 */
	public String getNombre() {
		return nombre;
	}

	/**
	 * Recupera raiz.
	 *
	 * @return el raiz
	 */
	public ComponenteCompuesta getRaiz() {
		return raiz;
	}

//	/**
//	 * Recuperar punteros hacia atrs que se haban perdido a la hora de serializar a un fichero xml.
//	 */
//	public void recuperarReferencias() {
//		raiz.setEstructura(this);
//		raiz.recuperarReferencias();
//	}

	/**
	 * Establece aalc.
	 *
	 * @param aalc el nuevo aalc
	 */
	public void setAalc(ArrayList<Linea> aalc) {
		this.aalc = aalc;
	}

	/**
	 * Establece id.
	 *
	 * @param id el nuevo id
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * Establece nombre.
	 *
	 * @param nombre el nuevo nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	/**
	 * Establece raiz.
	 *
	 * @param raiz el nuevo raiz
	 */
	public void setRaiz(ComponenteCompuesta raiz) {
		this.raiz = raiz;
	}

	/**
	 * Validar una estructura. Propaga la validacin a todas las componentes que forman parte de la estructura a travs 
	 * de la estructura en forma de rbol
	 *
	 * @param presentacion Presentacin seleccionada en el editor
	 * @return Informe de errores encontrados a la hora de realizar la validacin
	 */
	public String validar(Presentacion presentacion) {
		return raiz.validar(presentacion);
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	@Override
	public Object clone()  {
		
		Estructura clonada = new Estructura(this.getNombre());
		clonada.setId(this.getId());
		clonada.setDescripcion(this.getDescripcion());
		clonada.setCreacion(creacion);
		clonada.setModificacion(modificacion);
		clonada.setIdArea(idArea);
		clonada.setIdUsuarioCreacion(idUsuarioCreacion);
		clonada.setIdUsuarioModificacion(idUsuarioModificacion);

		try {
			clonada.setRaiz((ComponenteCompuesta) this.getRaiz().clone());
		} catch (CloneNotSupportedException e) {
		}
		clonada.getRaiz().setEstructura(clonada);
		clonada.actualizarLineas();
		return clonada;
	}

	@Override
	public String toString() {
		return this.getNombre();
	}

	public Calendar getCreacion() {
		return creacion;
	}

	public void setCreacion(Calendar creacion) {
		this.creacion = creacion;
	}

	public Calendar getModificacion() {
		return modificacion;
	}

	public void setModificacion(Calendar modificacion) {
		this.modificacion = modificacion;
	}

	public String getIdUsuarioCreacion() {
		return idUsuarioCreacion;
	}

	public void setIdUsuarioCreacion(String idUsuarioCreacion) {
		this.idUsuarioCreacion = idUsuarioCreacion;
	}

	public String getIdUsuarioModificacion() {
		return idUsuarioModificacion;
	}

	public void setIdUsuarioModificacion(String idUsuarioModificacion) {
		this.idUsuarioModificacion = idUsuarioModificacion;
	}

	public String getIdArea() {
		return idArea;
	}

	public void setIdArea(String idArea) {
		this.idArea = idArea;
	}


	@Override
	public int compareTo(Object arg0) {
		Estructura estructura = (Estructura) arg0;
		return this.getNombre().compareTo( estructura.getNombre());
	}

	@Override
	public boolean equals(Object estructura) {
		Estructura e = (Estructura) estructura;
		if(this.getId().equals(e.getId()))
			return true;
		else
			return false;
	}
	
	
}
