package modelo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;

// TODO: Auto-generated Javadoc
/**
 * The Class Selector.
 * Selector de una subestructura derivada
 */
@XmlAccessorType(XmlAccessType.NONE)
public class Selector{
	
	/** Identificador. */
	@XmlID
	private String id = UUID.randomUUID().toString();
	
	@XmlAttribute(name="Anidamiento")
	/** Nivel de anidamiento del selector */
	private int anidamiento;
	
	/** Componentes que integran el selector. */
	private ArrayList<Componente> componentes; // componente que contiene
	
	/** Unicamente el id de las componentes que lo integran. Necesario para la serializacion a XML al no poder incluir el array de componentes. */
	@XmlElement(name="Componentes")
	private ArrayList<String> componentesID;
	
	/** Subestructura derivada a la que pertenece el selector. */
	private ComponenteCompuesta ccSelector;
	
	@XmlElement(name="Lineas")
	/** Lneas que integran el selector */
	private ArrayList<SelectorLinea> lineas;
	
	@XmlElement(name="Expresiones")
	/** Expresiones a las cuales se igual los nombre de las componentes en el selector */
	private ArrayList<String> expresiones;


	/**
	 * Inicializa un objeto de tipo selector.
	 *
	 * @param ccSelector the cc selector
	 */
	public Selector(ComponenteCompuesta ccSelector) {
		super();
		this.ccSelector = ccSelector;
		componentes = new ArrayList<Componente>();
		lineas = new ArrayList<SelectorLinea>();
		expresiones = new ArrayList<String>();
	}

	/**
	 * Inicializa un objeto de tipo selector.
	 */
	public Selector() {
	}


	/**
	 * Recupera componentes.
	 *
	 * @return el componentes
	 */
	public ArrayList<Componente> getComponentes() {
		return componentes;
	}

	/**
	 * Establece componentes.
	 *
	 * @param componentes el nuevo componentes
	 */
	public void setComponentes(ArrayList<Componente> componentes) {
		this.componentes = componentes;
	}

	/**
	 * Recupera cc selector.
	 *
	 * @return el cc selector
	 */
	public ComponenteCompuesta getCcSelector() {
		return ccSelector;
	}

	/**
	 * Establece cc selector.
	 *
	 * @param ccSelector el nuevo cc selector
	 */
	public void setCcSelector(ComponenteCompuesta ccSelector) {
		this.ccSelector = ccSelector;
	}

	/**
	 * Recupera lineas.
	 *
	 * @return el lineas
	 */
	public ArrayList<SelectorLinea> getLineas() {
		return lineas;
	}

	/**
	 * Establece lineas.
	 *
	 * @param lineas el nuevo lineas
	 */
	public void setLineas(ArrayList<SelectorLinea> lineas) {
		this.lineas = lineas;
	}

	/**
	 * Recupera anidamiento.
	 *
	 * @return el anidamiento
	 */
	public int getAnidamiento() {
		return anidamiento;
	}

	/**
	 * Establece anidamiento.
	 *
	 * @param anidamiento el nuevo anidamiento
	 */
	public void setAnidamiento(int anidamiento) {
		this.anidamiento = anidamiento;
		for(int i = 0; i < lineas.size(); i++)
			lineas.get(i).setAnidamiento(anidamiento);
	}

	/**
	 * Recupera id.
	 *
	 * @return el id
	 */
	public String getId() {
		return id;
	}

	/**
	 * Establece id.
	 *
	 * @param id el nuevo id
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * Recupera expresiones.
	 *
	 * @return el expresiones
	 */
	public ArrayList<String> getExpresiones() {
		return expresiones;
	}

	/**
	 * Establece expresiones.
	 *
	 * @param expresiones el nuevo expresiones
	 */
	public void setExpresiones(ArrayList<String> expresiones) {
		this.expresiones = expresiones;
	}

	/**
	 * Recupera componentes id.
	 *
	 * @return el componentes id
	 */
	public ArrayList<String> getComponentesID() {
		return componentesID;
	}

	/**
	 * Establece componentes id.
	 *
	 * @param componentesID el nuevo componentes id
	 */
	public void setComponentesID(ArrayList<String> componentesID) {
		this.componentesID = componentesID;
	}



	
	
}
