package modelo;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;

// TODO: Auto-generated Javadoc
/**
 * The Class SelectorLinea.
 * Linea perteneciente a un selector que se muestra en las distintas tablas del editor
 */
@XmlAccessorType(XmlAccessType.NONE)
public class SelectorLinea extends Linea implements Cloneable{

	/** Linea de inicio de selector. */
	@XmlAttribute(name = "InicioSelector")
	private Boolean inicioSelector;
	
	/** Contiene el simbolo de parentesis inicial */
	@XmlAttribute(name = "ParentesisInicio")
	private Boolean parentesisInicio;
	
	/** Contiene el simbolo de parentesis final */
	@XmlAttribute(name = "ParentesisFin")
	private Boolean parentesisFin;
	
	/** Contiene el simbolo de coma. */
	@XmlAttribute(name = "Coma")
	private Boolean coma;

	/** Selector al que pertenece. */
	private Selector selector;




	/**
	 * Inicializa un objeto de tipo selector linea.
	 */
	public SelectorLinea() {
		super();
	}

	/**
	 * Inicializa un objeto de tipo selector linea.
	 *
	 * @param selector the selector
	 * @param anidamiento the anidamiento
	 */
	public SelectorLinea(Selector selector, int anidamiento) {
		this.selector = selector;
		setAnidamiento(anidamiento);
		setContenido("");
		this.clear();

	}

	/**
	 * Clear.
	 */
	private void clear(){
		inicioSelector = false;
		parentesisInicio = false;
		parentesisFin = false;
		coma = false;
	}
	
	/**
	 * Recupera inicio selector.
	 *
	 * @return el inicio selector
	 */
	public Boolean getInicioSelector() {
		return inicioSelector;
	}

	/**
	 * Establece inicio selector.
	 *
	 * @param inicioSelector el nuevo inicio selector
	 */
	public void setInicioSelector(Boolean inicioSelector) {
		this.inicioSelector = inicioSelector;
	}

	/**
	 * Recupera parentesis inicio.
	 *
	 * @return el parentesis inicio
	 */
	public Boolean getParentesisInicio() {
		return parentesisInicio;
	}

	/**
	 * Establece parentesis inicio.
	 *
	 * @param parentesisInicio el nuevo parentesis inicio
	 */
	public void setParentesisInicio(Boolean parentesisInicio) {
		this.parentesisInicio = parentesisInicio;
	}

	/**
	 * Recupera parentesis fin.
	 *
	 * @return el parentesis fin
	 */
	public Boolean getParentesisFin() {
		return parentesisFin;
	}

	/**
	 * Establece parentesis fin.
	 *
	 * @param parentesisFin el nuevo parentesis fin
	 */
	public void setParentesisFin(Boolean parentesisFin) {
		this.parentesisFin = parentesisFin;
	}

	/**
	 * Recupera coma.
	 *
	 * @return el coma
	 */
	public Boolean getComa() {
		return coma;
	}

	/**
	 * Establece coma.
	 *
	 * @param coma el nuevo coma
	 */
	public void setComa(Boolean coma) {
		this.coma = coma;
	}



	/**
	 * Recupera selector.
	 *
	 * @return el selector
	 */
	public Selector getSelector() {
		return selector;
	}

	/**
	 * Establece selector.
	 *
	 * @param selector el nuevo selector
	 */
	public void setSelector(Selector selector) {
		this.selector = selector;
	}

	/* (non-Javadoc)
	 * @see modelo.Linea#toString()
	 */
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for(int i = 0; i<getAnidamiento(); i++)
			sb.append("     ");

		if(getInicioSelector()) sb.append("$SEL =");
		if(getParentesisInicio()) sb.append(" ( ");
		sb.append(getContenido());
		if(getParentesisFin()) sb.append(" ) ");
		if(getComa()) sb.append(",");
		return sb.toString();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected Object clone() throws CloneNotSupportedException {
		SelectorLinea lineaClonada = new SelectorLinea(getSelector(), getAnidamiento());
		lineaClonada.setId(getId());
		lineaClonada.setComa(getComa());
		lineaClonada.setContenido(getContenido());
		lineaClonada.setInicioSelector(getInicioSelector());
		lineaClonada.setParentesisFin(getParentesisFin());
		lineaClonada.setParentesisInicio(getParentesisInicio());
		
		return lineaClonada;
	}



}
