package modelo.prototypes;

import java.util.ArrayList;

import modelo.Acabado;

// TODO: Auto-generated Javadoc
/**
 * PrototipoListaAcabado contiene la lista de caractersticas de acabado que por defecto tiene la aplicacin
 * Al crear una nueva presentacin, esta lista es clonada.
 */
public class PrototipoListaAcabado implements Cloneable {
    
    /** INSTANCE. */
    private static PrototipoListaAcabado INSTANCE = null;
    
    /** Conjunto de caractersticas de acabado. */
    private ArrayList<Acabado> acabados;
 
    // Private constructor suppresses 
    /**
     * Inicializa un objeto de tipo prototipo lista acabado.
     */
    private PrototipoListaAcabado() {
    	generarAcabados();
    }
 
    /**
     * Generar la lista predefinida de caractersticas de acabado.
     */
    private void generarAcabados() {
		acabados = new ArrayList<Acabado>();
		acabados.add(new Acabado("Agregados", 5));
		acabados.add(new Acabado("Especializados", 5));	
	}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    /**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new PrototipoListaAcabado();
        }
    }
 
    /**
     * Gets the single instance of PrototipoListaAcabado.
     *
     * @return single instance of PrototipoListaAcabado
     */
    public static PrototipoListaAcabado getInstance() {
        createInstance();
        return INSTANCE;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone(){
		ArrayList<Acabado> clonados =  new ArrayList<Acabado>();
		for(Acabado a: acabados){
			clonados.add((Acabado) a.clone());
		}
		return clonados;
	}
    
    
}