package modelo.prototypes;

import java.util.ArrayList;

import modelo.Aspecto;

// TODO: Auto-generated Javadoc
/**
 * PrototipoListaAspecto contiene la lista de caractersticas de aspecto que por defecto tiene la aplicacin
 * Al crear una nueva presentacin, esta lista es clonada. 
 */
public class PrototipoListaAspecto implements Cloneable {
    
    /** INSTANCE. */
    private static PrototipoListaAspecto INSTANCE = null;
    
    /** Conjunto de caractersticas de aspecto. */
    private ArrayList<Aspecto> listaAspecto;
 
    // Private constructor suppresses 
    /**
     * Inicializa un objeto de tipo prototipo lista aspecto.
     */
    private PrototipoListaAspecto() {
    	generarListaAspecto();
    }
 
    /**
     * Generar la lista de caractersticas de aspecto que tienen las presentaciones de la aplicacin por defecto.
     */
    private void generarListaAspecto() {
		listaAspecto = new ArrayList<Aspecto>();
		listaAspecto.add(new Aspecto("Nombre de una estructura de adquisicin"));
		listaAspecto.add(new Aspecto("Nombre de una componente compuesta"));
		listaAspecto.add(new Aspecto("Nombre de una componente elemental"));
		listaAspecto.add(new Aspecto("Nombre de una estructura derivada"));
		listaAspecto.add(new Aspecto("Nombre de los componentes de una estructura derivada"));
		listaAspecto.add(new Aspecto("Cabecera de un selector"));
		listaAspecto.add(new Aspecto("Elementos presentes en un selector"));
		listaAspecto.add(new Aspecto("Smbolos"));
		
	}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    /**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new PrototipoListaAspecto();
        }
    }
 
    /**
     * Gets the single instance of PrototipoListaAspecto.
     *
     * @return single instance of PrototipoListaAspecto
     */
    public static PrototipoListaAspecto getInstance() {
        createInstance();
        return INSTANCE;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone()  {
		ArrayList<Aspecto> clonado = new ArrayList<Aspecto>();
		for(Aspecto a: listaAspecto){
			clonado.add((Aspecto) a.clone());
		}
		return clonado;
	}
    
    
}
