package modelo.prototypes;

import java.util.ArrayList;

import javax.xml.bind.JAXBException;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import datos.PrototipoPropiedadesDAO;

import modelo.Aspecto;
import modelo.Propiedad;
import modelo.tipoDatos.Alfanumerico;
import modelo.tipoDatos.Binario;
import modelo.tipoDatos.Rango;

// TODO: Auto-generated Javadoc
/**
 * PrototipoListaPropiedades lee de disco las propiedades que por defecto tendrn las nuevas presentaciones realizadas con la 
 * aplicacin. Si no encuentra el fichero correspondiente, o si ste est corrupto, vuelve a generar uno nuevo.
 * A la hora de crear una nueva presentacin, esta lista de propiedades es clonada.
 */
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="ListadoPropiedades")
public class PrototipoListaPropiedades implements Cloneable{

	/** Conjunto de propiedades que contendrn las presentaciones creadas. */
	@XmlElement(name="Propiedad")
	private ArrayList<Propiedad> propiedades;
	
	/**
	 * Inicializa un objeto de tipo prototipo lista propiedades.
	 */
	public PrototipoListaPropiedades() {
		super();
	}
	
	/**
	 * Guardar la lista de propiedades, para crear el fichero si ste no existe.
	 */
	public void guardar(){
		PrototipoPropiedadesDAO dao = PrototipoPropiedadesDAO.getInstance();
		dao.guardarPrototipo(this);
	}
	
	/**
	 * Cargar el listado de propiedades de disco, o crearlo en el caso de que no exista.
	 */
	public void abrir(){
		PrototipoPropiedadesDAO dao = PrototipoPropiedadesDAO.getInstance();
		try {
			// Intenta recuperar la lista de propiedades de fichero
			PrototipoListaPropiedades p = dao.abrirPrototipo();
			propiedades = p.getPropiedades();
		} catch (JAXBException e) {
			// En el caso de que no pueda, genera de nuevo la lista y la guarda
			generarPropiedades();
			guardar();
		}
	}
	
	/**
	 * Generar propiedades por defecto
	 */
	public void generarPropiedades(){
		propiedades = new ArrayList<Propiedad>();
		propiedades.add(new Propiedad("Origen", null, true, new Alfanumerico()));
		propiedades.add(new Propiedad("Dominio", null, true, new Alfanumerico()));
		propiedades.add(new Propiedad("Operacin", null, true, new Alfanumerico()));
		propiedades.add(new Propiedad("Editable", null, true, new Binario()));
		propiedades.add(new Propiedad("Descripcin", null, true, new Alfanumerico()));
		propiedades.add(new Propiedad("Columna asociada", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Control asociado", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Ejemplo", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Etiqueta en columnas", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Etiqueta en formularios", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Formato", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Rango", null, false, new Rango()));
		propiedades.add(new Propiedad("Clave primaria", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Obligatorio", null, false, new Binario()));
		propiedades.add(new Propiedad("Rango razonable", null, false, new Rango()));
		propiedades.add(new Propiedad("Sinnimo", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Tabla asociada", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Tipo de unidades", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Prop. Auxiliar 1", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Prop. Auxiliar 2", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Prop. Auxiliar 3", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Prop. Auxiliar 4", null, false, new Alfanumerico()));
		propiedades.add(new Propiedad("Prop. Auxiliar 5", null, false, new Alfanumerico()));
	}

	/**
	 * Recupera propiedades.
	 *
	 * @return el propiedades
	 */
	public ArrayList<Propiedad> getPropiedades() {
		return propiedades;
	}

	/**
	 * Establece propiedades.
	 *
	 * @param propiedades el nuevo propiedades
	 */
	public void setPropiedades(ArrayList<Propiedad> propiedades) {
		this.propiedades = propiedades;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone()  {
		ArrayList<Propiedad> clonado = new ArrayList<Propiedad>();
		for(Propiedad p: propiedades){
			clonado.add((Propiedad) p.clone());
		}
		return clonado;
	}
	
	
	

}

