package utils;


import java.util.ArrayDeque;
import java.util.ArrayList;

import controlador.undo.Command;

/**
 * The Class ColaComandos.
 * Integra las colas para la realizacin de las operaciones de deshacer y rehacer
 */
public class ColaComandos{

	/** Cola de comandos para deshacer. */
	private ArrayDeque<Command> deshacer = new ArrayDeque<Command>();
	
	/** Cola de comandos para rehacer. */
	private ArrayDeque<Command> rehacer = new ArrayDeque<Command>();
	
	/** INSTANCE. */
	private static ColaComandos INSTANCE = null;

	// Private constructor suppresses 
	/**
	 * Inicializa un objeto de tipo cola comandos.
	 */
	private ColaComandos() {
		super();
	}

	/**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new ColaComandos();
		}
	}

	/**
	 * Gets the single instance of ColaComandos.
	 *
	 * @return single instance of ColaComandos
	 */
	public static ColaComandos getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}

	/**
	 * Rehacer.
	 */
	public void rehacer (){
		if(rehacer.isEmpty()) return ;
		rehacer.peekFirst().ejecutar();
		deshacer.addFirst(rehacer.peekFirst());
		rehacer.removeFirst();
	}
	
	/**
	 * Deshacer.
	 */
	public void deshacer(){
		if(deshacer.isEmpty()) return;
		deshacer.peekFirst().deshacer();
		rehacer.addFirst(deshacer.peekFirst());
		deshacer.removeFirst();
	}
	
	/**
	 * Agregar comando ejecutado.
	 *
	 * @param command el command ejecutado
	 */
	public void agregarComando(Command command){
		deshacer.addFirst(command);
		rehacer.clear();
	}

	/**
	 * Vaciar ambas colas.
	 */
	public void clear(){
		deshacer.clear();
		rehacer.clear();
	}
	
	/**
	 * Es posible deshacer.
	 *
	 * @return true, si es posible deshacer
	 */
	public boolean esPosibleDeshacer(){
		return !deshacer.isEmpty();
	}
	
	/**
	 * Es posible rehacer.
	 *
	 * @return true, si es posible rehacer
	 */
	public boolean esPosibleRehacer(){
		return !rehacer.isEmpty();
	}
}
