package utils;

import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.apache.commons.lang3.StringEscapeUtils;

import utils.tablemodels.ModeloEditorEstructuras;

import modelo.Area;
import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Esquema;
import modelo.Visualizacion;
import modelo.Estructura;
import modelo.Linea;
import modelo.Presentacion;
import modelo.Selector;
import modelo.SelectorLinea;

public class HTMLUtils {

	public static void generarEsquemaHTML(Esquema esquema){

		StringBuffer buffer = new StringBuffer();

		//----------------------------- Documento
		String inicio = generarInicio();
		buffer.append(inicio);

		String estilo = generarEstilo();
		buffer.append(estilo);

		//----------------------------- Cabecera
		String cabeceraInicio = generarInicioCabecera();
		buffer.append(cabeceraInicio);

		String cabecera = generarEncabezado(esquema.getNombre(), 1);
		buffer.append(cabecera);

		buffer.append(lineaHorizontal());

		String descripcionEsquema = generarDescripcion(esquema.getDescripcion());
		buffer.append(descripcionEsquema);

		String cabeceraFin = generarFinCabecera();
		buffer.append(cabeceraFin);

		//----------------------------- Cuerpo

		buffer.append(generarInicioCuerpo());

		ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();
		ManagerPresentaciones managerPresentaciones = ManagerPresentaciones.getInstance();
		
		Presentacion presentacion = managerPresentaciones.buscarPorId(esquema.getIdPresentacion());
		if(presentacion == null)
			presentacion = managerPresentaciones.getPresentacionDefault();
		
		// Por cada estructura del esquema
		for(String id: esquema.getIdEstructuras()){

			Estructura estructura = managerEstructuras.getEstructurasID().get(id);

			if(estructura != null){
				buffer.append(generarEncabezado(estructura.getNombre(), 2));
				buffer.append(lineaHorizontal());
				buffer.append(generarDescripcion(estructura.getDescripcion()));
				
				/*
				 * Para aprovechar el codigo existente de antes, se hace esta guarrada.
				 * Sorry si te toca modificarlo, pero a mi me queda poco para acabar el PFC ;)
				 */
				
				estructura.actualizarLineas();
				ModeloEditorEstructuras modelo = new ModeloEditorEstructuras(presentacion, estructura);
				JTable jtable = new JTable(modelo);
				Visualizacion visualizacion = new Visualizacion();
				visualizacion.setEstructura(estructura);
				visualizacion.setPresentacion(presentacion);
				String codigoTabla = generarTabla(jtable, visualizacion);
				buffer.append(codigoTabla);
				
			}
		}

		buffer.append(generarFinCuerpo());

		//----------------------------- Generacin

		generarFicheroEsquema(esquema, buffer);

		lanzarFicheroEsquema(esquema);

	}
	
	public static void generarHTMLEstructura(Visualizacion visualizacion, JTable table){

		StringBuffer buffer = new StringBuffer();

		String inicio = generarInicio();
		buffer.append(inicio);

		String estilo = generarEstilo();
		buffer.append(estilo);

		buffer.append(generarInicioCabecera());
		String cabecera = generarEncabezado(visualizacion.getEstructura().getNombre(), 1);
		buffer.append(cabecera);
		buffer.append(generarFinCabecera());

		buffer.append(generarInicioCuerpo());
		String descripcion = generarDescripcion(visualizacion.getEstructura().getDescripcion());
		buffer.append(descripcion);

		String tabla = generarTabla(table, visualizacion);
		buffer.append(tabla);
		buffer.append(generarFinCuerpo());

		String fin = generarFin();
		buffer.append(fin);

		generarFicheroEstructura(visualizacion.getEstructura().getNombre(), buffer);

		lanzarFicheroEstructura(visualizacion.getEstructura().getNombre());

	}

	private static void lanzarFicheroEstructura(String nombre) {
		String ruta = System.getProperty("user.dir")+"\\EstructuraHTML\\"+nombre+".html";
		File fichero = new File(ruta);
		if(fichero.isFile()){
			Desktop desktop;
			if (Desktop.isDesktopSupported()){// si ste Host soporta esta API 
				desktop = Desktop.getDesktop();//objtengo una instancia del Desktop(Escritorio)de mi host 
				try {
					desktop.open(fichero);//abro el archivo con el programa predeterminado
				}
				catch (IOException ex) {
					ex.printStackTrace();
				}
			}
			else{ JOptionPane.showMessageDialog(null,"No es posible mostrar el manual de usuario en esta mquina.");
			}
		}

	}
	
	private static void lanzarFicheroEsquema(Esquema esquema) {
		ManagerAreas ma = ManagerAreas.getInstance();
		Area area = ma.obtenerAreaPorID(esquema.getIdArea());
		
		String ruta = System.getProperty("user.dir")+"\\Esquemas\\"+area.getNombre()+"\\"+ esquema.getNombre() + ".html";
		File fichero = new File(ruta);
		if(fichero.isFile()){
			Desktop desktop;
			if (Desktop.isDesktopSupported()){// si ste Host soporta esta API 
				desktop = Desktop.getDesktop();//objtengo una instancia del Desktop(Escritorio)de mi host 
				try {
					desktop.open(fichero);//abro el archivo con el programa predeterminado
				}
				catch (IOException ex) {
					ex.printStackTrace();
				}
			}
			else{ JOptionPane.showMessageDialog(null,"No es posible mostrar el manual de usuario en esta mquina.");
			}
		}

	}

	private static void generarFicheroEsquema(Esquema esquema, StringBuffer buffer) {
		ManagerAreas ma = ManagerAreas.getInstance();
		Area area = ma.obtenerAreaPorID(esquema.getIdArea());
		
		String direccion = System.getProperty("user.dir")+"\\Esquemas\\"+area.getNombre()+"\\";
		File ruta = new File(direccion);
		File fichero = null;
		if(!ruta.exists()){
			ruta.mkdirs();
		}
		try{
			// Create file 
			fichero = new File(direccion + esquema.getNombre() + ".html");
			FileWriter fstream = new FileWriter(fichero);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write(buffer.toString());
			//Close the output stream
			out.close();
		}catch (Exception e){//Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}

	}
	
	private static void generarFicheroEstructura(String nombre, StringBuffer buffer) {
		String direccion = System.getProperty("user.dir")+"\\EstructuraHTML\\";
		File ruta = new File(direccion);
		File fichero = null;
		if(!ruta.exists()){
			ruta.mkdirs();
		}
		try{
			// Create file 
			fichero = new File(direccion + nombre + ".html");
			FileWriter fstream = new FileWriter(fichero);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write(buffer.toString());
			//Close the output stream
			out.close();
		}catch (Exception e){//Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}

	}

	private static String generarTabla(JTable table, Visualizacion visualizacion) {

		/*
		 * Hay que vigilar el comportamiento de la celda actualmente seleccionada en la tabla,
		 * ya que el color de sta es blanca, cuando debera ser del color de su aspecto. Por lo tanto,
		 * la componente seleccionada hay que colorearla de otro color.
		 */
		Color colorFilaSeleccionada = Color.BLACK;
		if(table.getSelectedRow() > -1)
			colorFilaSeleccionada = buscarColorFilaSeleccionada(visualizacion, table.getSelectedRow());


		StringBuffer sb = new StringBuffer();
		sb.append("<p><b>" + StringEscapeUtils.escapeHtml4("Estructura:") + "</b></p>");
		sb.append("<table  class=\"Estructura\">\n");
		/****************************************/

		// cabeceras
		sb.append("<tr class=\"Estructura\">\n");
		for(int column = 0; column < table.getModel().getColumnCount(); column++){
			sb.append("<th class=\"Estructura\">"+table.getModel().getColumnName(column)+"</th>\n");
		}
		sb.append("</tr>\n");
		// cuerpo tabla
		for(int row = 0;row< table.getModel().getRowCount() ;row++) {
			sb.append("<tr class=\"Estructura\">\n");
			for(int column =0; column<table.getModel().getColumnCount();column++){
				String contenido = table.getModel().getValueAt(row,column).toString();

				// Contenido
				contenido = StringEscapeUtils.escapeHtml4(contenido);
				contenido = mantenerEspacios(contenido);

				// Componente
				Component c = table.getCellRenderer(row, column).getTableCellRendererComponent(table, contenido, false, false, row, column);

				// Color
				int red = 0;
				int green = 0;
				int blue = 0;

				if(row == table.getSelectedRow()){
					red = colorFilaSeleccionada.getRed();
					green = colorFilaSeleccionada.getGreen();
					blue = colorFilaSeleccionada.getBlue();
				}
				else{
					red = c.getForeground().getRed();
					green = c.getForeground().getGreen();
					blue = c.getForeground().getBlue();
				}

				// Tamao
				int tamanyo = c.getFont().getSize();

				// Fuente
				String fuente = c.getFont().getFontName();

				// Estilo

				String estilo = "style=\"font-size: " + tamanyo +"px; font-family: " + fuente +
						"; color: rgb(" + red + ", " + green + ", " + blue + ");\"";

				sb.append("<td class=\"Estructura\" " + estilo + ">" +  contenido + "</td>\n");
			}
			sb.append("</tr>\n");
		}

		/****************************************/
		sb.append("</table>\n");
		return sb.toString();
	}

	static Color buscarColorFilaSeleccionada(Visualizacion visualizacion, int selectedRow) {

		Estructura estructura = visualizacion.getEstructura();
		Presentacion presentacion = visualizacion.getPresentacion();

		Linea lineaSeleccionada = estructura.getAalc().get(selectedRow);

		if(lineaSeleccionada.getClass() == SelectorLinea.class){

			SelectorLinea sl = (SelectorLinea) lineaSeleccionada;

			Selector selector = sl.getSelector();

			if(selector.getLineas().get(0).getId().equals(sl.getId())){
				return new Color(presentacion.getAspectos().get(5).getColor());
			}
			else{
				return new Color(presentacion.getAspectos().get(6).getColor());
			}

		}
		else if (lineaSeleccionada.getClass() == ComponenteLinea.class){

			// Bsqueda de la componente
			ComponenteLinea cl = (ComponenteLinea) lineaSeleccionada;

			Componente componente = cl.getComponente();

			// Comprobacin del tipo de componente

			if(componente.getClass() == ComponenteElemental.class){

				ComponenteElemental ce = (ComponenteElemental) componente;

				// Se colorea de una forma u otra dependiendo de si es derivada o no
				if(ce.getDerivada() == true){
					return new Color(presentacion.getAspectos().get(4).getColor());
				}
				else{
					return new Color(presentacion.getAspectos().get(2).getColor());
				}

			}
			else if(componente.getClass() == ComponenteCompuesta.class){

				ComponenteCompuesta cc = (ComponenteCompuesta) componente;

				// Comprobar de qu lnea se trata

				if(cl.getId().equals(cc.getCabecera().getId())){

					if(cc.getParent() == null){
						return new Color(presentacion.getAspectos().get(0).getColor());
					}
					else{ // parent != null
						if(cc.getDerivada() == false){
							return new Color(presentacion.getAspectos().get(1).getColor());
						}
						else{
							if(cc.getParent().getDerivada() == false){
								return new Color(presentacion.getAspectos().get(3).getColor());
							}
							else{
								return new Color(presentacion.getAspectos().get(4).getColor());
							}
						}
					}

				}
				else if((cl.getId().equals(cc.getSimbolosFin().getId()))||
						(cl.getId().equals(cc.getSimbolosInicio().getId()))){
					return new Color(presentacion.getAspectos().get(7).getColor());
				}

			}
		}
		return Color.BLACK;
	}

	private static String generarDescripcion(String descripcion) {
		if(descripcion == null) descripcion = "";

		descripcion = StringEscapeUtils.escapeHtml4(descripcion);
		//descripcion = mantenerEspacios(descripcion);

		StringBuffer sb = new StringBuffer();
		sb.append("<p><b>" + StringEscapeUtils.escapeHtml4("Descripcin:") + "</b></p>");
		//		sb.append("<p class=\"descripcion\">" + descripcion + "</p>\n");
		String [] partesDescripcion = descripcion.split("\n");

		// Crear tabla transparente
		sb.append("<table  border=\"0\">\n");

		// Celdas
		for(int i = 0; i < partesDescripcion.length; i++){
			sb.append("<tr><td>"+partesDescripcion[i]+"</td></tr>\n");
		}


		// Finalizar tabla
		sb.append("</table>");

		// Linea final
		sb.append("<hr>\n");
		return sb.toString();
	}

	private static String generarEstilo() {
		StringBuffer sb = new StringBuffer();
		sb.append("<style type=\"text/css\">\n");

		String direccion = System.getProperty("user.dir")+"\\configuracion\\";
		String filename = "estiloTabla.css";

		File archivo = null;
		FileReader fr = null;
		BufferedReader br = null;

		try {
			// Apertura del fichero y creacion de BufferedReader para poder
			// hacer una lectura comoda (disponer del metodo readLine()).
			archivo = new File (direccion+filename);
			fr = new FileReader (archivo);
			br = new BufferedReader(fr);

			// Lectura del fichero
			String linea;
			while((linea=br.readLine())!=null)
				sb.append(linea+"\n");
		}
		catch(Exception e){
			e.printStackTrace();
		}finally{
			// En el finally cerramos el fichero, para asegurarnos
			// que se cierra tanto si todo va bien como si salta 
			// una excepcion.
			try{                    
				if( null != fr ){   
					fr.close();     
				}                  
			}catch (Exception e2){ 
				e2.printStackTrace();
			}
		}

		sb.append("</style>\n");
		return sb.toString();
	}

	private static String generarEncabezado(String nombre, int tipo) {
		StringBuffer sb = new StringBuffer();
		sb.append("<p><h" + tipo +">"+ StringEscapeUtils.escapeHtml4(nombre) + "</h" + tipo +"></p>\n");
		return sb.toString();
	}

	private static String generarFin() {
		StringBuffer sb = new StringBuffer();
		sb.append("</html>\n");
		return sb.toString();
	}

	private static String generarInicio() {

		StringBuffer sb = new StringBuffer();
		sb.append("<!DOCTYPE html>\n");
		sb.append("<html>\n");

		return sb.toString();
	}

	private static String mantenerEspacios(String origen){
		String str =  origen.replace(" ", "&nbsp;");
		str = str.replaceAll("(\r\n|\n)", "<br />");
		return str;
	}

	private static String generarInicioCabecera(){
		return "<head>\n";
	}

	private static String generarFinCabecera(){
		return "</head>\n";
	}

	private static String generarInicioCuerpo(){
		return "<body>\n";
	}

	private static String generarFinCuerpo(){
		return "</body>\n";
	}

	public static String lineaHorizontal(){
		return "<hr>";
	}

}
