package utils;

import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.apache.commons.lang3.StringEscapeUtils;

import controlador.EditorEsquemasController;
import controlador.EditorEstructurasController;

import utils.tablemodels.ModeloEditorEstructuras;
import word.*;
import word.api.interfaces.IDocument;
import word.utils.TestUtils;
import word.utils.Utils;
import word.w2004.Document2004;
import word.w2004.Document2004.Encoding;
import word.w2004.elements.BreakLine;
import word.w2004.elements.Heading1;
import word.w2004.elements.Heading2;
import word.w2004.elements.Heading3;
import word.w2004.elements.HyperLink;
import word.w2004.elements.Image;
import word.w2004.elements.ImageLocation;
import word.w2004.elements.PageBreak;
import word.w2004.elements.Paragraph;
import word.w2004.elements.ParagraphPiece;
import word.w2004.elements.Table;
import word.w2004.elements.tableElements.TableEle;
import word.w2004.style.Font;
import word.w2004.style.HeadingStyle.Align;

import modelo.Area;
import modelo.ComponenteCompuesta;
import modelo.Esquema;
import modelo.Presentacion;
import modelo.Visualizacion;
import modelo.Estructura;

public class MSWordUtils {

	private Estructura estructura;
	private Visualizacion visualizacion;
	private JTable table;
	private IDocument documento;
	private Esquema esquema;



	public MSWordUtils(Visualizacion visualizacion, JTable table) {
		super();
		this.estructura = visualizacion.getEstructura();
		this.visualizacion = visualizacion;
		this.table = table;
	}

	public MSWordUtils(Esquema esquema){
		super();
		this.esquema = esquema;
	}

	public void generarMSWordEsquema(){


		generarInicio();

		generarCabecera(esquema.getNombre(), 1);

		generarCabecera("Descripcin", 2);
		generarParrafo(esquema.getDescripcion());

		/* Aqui viene la guarrada */
		ManagerEstructuras managerEstructuras = ManagerEstructuras.getInstance();
		ManagerPresentaciones managerPresentaciones = ManagerPresentaciones.getInstance();

		Presentacion presentacion = managerPresentaciones.buscarPorId(esquema.getIdPresentacion());
		if(presentacion == null)
			presentacion = managerPresentaciones.getPresentacionDefault();

		// Por cada estructura del rea
		for(String id: esquema.getIdEstructuras()){

			Estructura estructura = managerEstructuras.getEstructurasID().get(id);

			if(estructura != null){
				generarCabecera(estructura.getNombre(), 2);

				generarCabecera("Descripcin", 3);
				generarParrafo(estructura.getDescripcion());

				generarCabecera("Estructura", 3);
				estructura.actualizarLineas();
				ModeloEditorEstructuras modelo = new ModeloEditorEstructuras(presentacion, estructura);
				table = new JTable(modelo);
				visualizacion = new Visualizacion();
				visualizacion.setEstructura(estructura);
				visualizacion.setPresentacion(presentacion);

				generarTabla();

			}
		}

		if(generarFicheroEsquema())
			lanzarFicheroEsquema();

	}

	public void generarMSWordEstructura(){


		generarInicio();

		generarCabecera(estructura.getNombre(), 1);

		generarCabecera("Descripcin", 2);
		generarParrafo(estructura.getDescripcion());

		generarCabecera("Estructura", 2);
		generarTabla();

		if(generarFicheroEstructura(estructura.getNombre()))
			lanzarFicheroEstructura(estructura.getNombre());

	}

	private void lanzarFicheroEstructura(String nombre) {
		String ruta = System.getProperty("user.dir")+"\\EstructuraMSWord\\"+nombre+".doc";
		File fichero = new File(ruta);
		if(fichero.isFile()){
			Desktop desktop;
			if (Desktop.isDesktopSupported()){// si ste Host soporta esta API 
				desktop = Desktop.getDesktop();//objtengo una instancia del Desktop(Escritorio)de mi host 
				try {
					desktop.open(fichero);//abro el archivo con el programa predeterminado
				}
				catch (IOException ex) {
					ex.printStackTrace();
				}
			}
			else{ JOptionPane.showMessageDialog(null,"No es posible mostrar el manual de usuario en esta mquina.");
			}
		}

	}

	private void lanzarFicheroEsquema() {
		ManagerAreas ma = ManagerAreas.getInstance();
		Area area = ma.obtenerAreaPorID(esquema.getIdArea());
		String ruta = System.getProperty("user.dir")+"\\Esquemas\\"+area.getNombre()+"\\"+ esquema.getNombre() + ".doc";
		File fichero = new File(ruta);
		if(fichero.isFile()){
			Desktop desktop;
			if (Desktop.isDesktopSupported()){// si ste Host soporta esta API 
				desktop = Desktop.getDesktop();//objtengo una instancia del Desktop(Escritorio)de mi host 
				try {
					desktop.open(fichero);//abro el archivo con el programa predeterminado
				}
				catch (IOException ex) {
					ex.printStackTrace();
				}
			}
			else{ JOptionPane.showMessageDialog(null,"No es posible mostrar el manual de usuario en esta mquina.");
			}
		}

	}

	private boolean generarFicheroEstructura(String nombre) {
		String direccion = System.getProperty("user.dir")+"\\EstructuraMSWord\\";
		File ruta = new File(direccion);
		if(!ruta.exists()){
			ruta.mkdirs();
		}
		File fileObj = new File(direccion+nombre+".doc");

		PrintWriter writer = null;
		try {
			writer = new PrintWriter(fileObj);
			String myWord = documento.getContent();

			writer.println(myWord);
			writer.close();
			return true;
		} catch (FileNotFoundException e) {
			EditorEstructurasController controller = EditorEstructurasController.getInstance();
			JOptionPane.showMessageDialog(controller.getView().getFrmEditorDeEstructuras(),
					"No se pudo completar la generacin del fichero.\n" +
							"Asegrese de no tener el fichero abierto con ningn otro programa.",
							"Error de generacin",
							JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}

	private boolean generarFicheroEsquema() {
		ManagerAreas ma = ManagerAreas.getInstance();
		Area area = ma.obtenerAreaPorID(esquema.getIdArea());

		String direccion = System.getProperty("user.dir")+"\\Esquemas\\"+area.getNombre()+"\\";
		File ruta = new File(direccion);
		if(!ruta.exists()){
			ruta.mkdirs();
		}
		File fileObj = new File(direccion+esquema.getNombre()+".doc");

		PrintWriter writer = null;
		try {
			writer = new PrintWriter(fileObj);
			String myWord = documento.getContent();

			writer.println(myWord);
			writer.close();
			return true;
		} catch (FileNotFoundException e) {
			EditorEsquemasController controller = EditorEsquemasController.getInstance();
			JOptionPane.showMessageDialog(controller.getView(),
					"No se pudo completar la generacin del fichero.\n" +
							"Asegrese de no tener el fichero abierto con ningn otro programa.",
							"Error de generacin",
							JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}

	private void generarTabla() {

		/*
		 * Hay que vigilar el comportamiento de la celda actualmente seleccionada en la tabla,
		 * ya que el color de sta es blanca, cuando debera ser del color de su aspecto. Por lo tanto,
		 * la componente seleccionada hay que colorearla de otro color.
		 */
		Color colorFilaSeleccionada = Color.BLACK;
		if(table.getSelectedRow() > -1)
			colorFilaSeleccionada = HTMLUtils.buscarColorFilaSeleccionada(visualizacion, table.getSelectedRow());

		// Obtener el nombre de las columnas
		ModeloEditorEstructuras modelo = (ModeloEditorEstructuras) table.getModel();
		ArrayList<String> columnas = modelo.getNombresColumnas();
		String []columnasString = new String[columnas.size()];
		columnas.toArray(columnasString);

		// Cabecera de la tabla
		Table tbl = new Table();
		tbl.addTableEle(TableEle.TH, columnasString);
		tbl.setRepeatTableHeaderOnEveryPage();
		/****************************************/

		// Cuerpo de la tabla
		for(int row = 0;row< table.getModel().getRowCount() ;row++) {

			//String []contenido = new String[modelo.getColumnCount()];
			Paragraph [] contenido = new Paragraph[modelo.getColumnCount()];

			// Recuperar todas las columnas de la fila
			for(int column =0; column<table.getModel().getColumnCount();column++){

				// Contenido
				String aux = table.getModel().getValueAt(row,column).toString();

				// Color
				Component c = table.getCellRenderer(row, column).getTableCellRendererComponent(table, contenido, false, false, row, column);
				String rgbColor = "000000";
				if(row == table.getSelectedRow()){
					rgbColor = Integer.toHexString(colorFilaSeleccionada.getRGB());
				}
				else{
					rgbColor = Integer.toHexString(c.getForeground().getRGB());
				}

				// Tamao
				int tamanyo = c.getFont().getSize();

				// Fuente
				String fuente = c.getFont().getFontName();


				ParagraphPiece piece = ParagraphPiece
						.with(texto(aux))
						.withStyle().textColor(rgbColor.substring(2)).fontSize(String.valueOf(tamanyo)).create();

				// Aadir al array
				contenido[column] = Paragraph.withPieces(piece).create();

				//contenido[column] = texto(aux);

			}

			// Aadir a la tabla
			tbl.addTableEle(TableEle.TD, contenido);
		}

		documento.addEle(tbl);

		documento.addEle(BreakLine.times(1).create());
	}

	private void generarParrafo(String descripcion) {
		if(descripcion == null) 
			descripcion = "";
		documento.addEle(BreakLine.times(1).create());
		//documento.addEle(Paragraph.with(texto(descripcion)).create());
		String []parrafos = descripcion.split("\n");
		for(int i = 0; i < parrafos.length; i++){
			documento.addEle(Paragraph.with(texto(parrafos[i])).create());
			//documento.addEle(BreakLine.times(1).create());
		}

		documento.addEle(BreakLine.times(1).create());
	}


	private void generarCabecera(String nombre, int tipo) {
		if(nombre != null){
			switch(tipo){
			case 1:
				documento.addEle(Heading1.with(nombre).create());
				break;
			case 2:
				documento.addEle(Heading2.with(nombre).create());
				break;
			case 3:
				documento.addEle(Heading3.with(nombre).create());
				break;
			}
			documento.addEle(BreakLine.times(1).create());
		}
	}

	private void generarInicio() {

		documento = new Document2004();
		documento.encoding(Encoding.ISO8859_1);
		documento.author("Adam Kepa");
		documento.company("DSIC-UPV");
		documento.description("Generacin automtica de un visualizacion");
	}
	// EJEMPLO

	//	public static void testJava2wordAllInOne() {
	//
	//		IDocument myDoc = new Document2004();
	//		myDoc.addEle(Heading1.with("Heading01").create());
	//		myDoc.addEle(Paragraph.with("This document is an example of paragraph").create());
	//		
	//		File fileObj = new File("C:\\Users\\Adam\\Desktop\\test.doc");
	//
	//		PrintWriter writer = null;
	//		try {
	//		    writer = new PrintWriter(fileObj);
	//		} catch (FileNotFoundException e) {
	//		    e.printStackTrace();
	//		}
	//		String myWord = myDoc.getContent();
	//
	//		writer.println(myWord);
	//		writer.close();    
	//
	//		myDoc.addEle(new BreakLine(2)); //two break lines
	//        
	//    }

	private String texto(String t){
		String aux = Utils.replaceSpecialCharacters(t);
		aux = aux.replace("<", "&lt;");
		aux = aux.replace(">", "&gt;");
		aux = aux.replace("\"", "&quot;");
		return aux;
	}
}
