package utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.SortedSet;
import java.util.TreeSet;

import datos.AreasDAO;
import datos.GruposDAO;

import modelo.Area;
import modelo.Estructura;
import modelo.Grupo;
import modelo.Usuario;

public class ManagerAreas {

	private static ManagerAreas INSTANCE = null;
	private HashMap<String, Area> nombresAreas;
	private HashMap<String, Area> idsAreas;

	// Private constructor suppresses 
	private ManagerAreas() {
		inicializar();
	}

	private void inicializar() {
		nombresAreas = new HashMap<String, Area>();
		idsAreas = new HashMap<String, Area>();

		// cargar grupos
		cargarAreas();

	}

	private void cargarAreas() {
		AreasDAO dao = AreasDAO.getInstance();
		ArrayList<Area> areas = new ArrayList<Area>();

		ManagerEstructuras me = ManagerEstructuras.getInstance();
		try {
			areas = dao.cargarAreas();
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		// Por cada area leda
		for(Area a: areas){
			a.setEstructuras(new ArrayList<Estructura>());
			// Recuperar estructuras propias

			for(int i = a.getIdEstructuras().size()-1; i >= 0; i--){

				String id = a.getIdEstructuras().get(i);
				
				Estructura e = me.getEstructurasID().get(id);

				// La estructura no existe o ha sido borrada
				if(e == null){
					a.getIdEstructuras().remove(id);
				}
				else{
					a.getEstructuras().add(e);
				}

			}
			nombresAreas.put(a.getNombre(), a);
			idsAreas.put(a.getId(), a);
		}

	}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new ManagerAreas();
		}
	}

	public static ManagerAreas getInstance() {
		createInstance();
		return INSTANCE;
	}

	public void eliminarAreaPorNombre(String nombre) {
		if(nombresAreas.containsKey(nombre)){
			Area area = nombresAreas.get(nombre);
			nombresAreas.remove(area.getNombre());
			idsAreas.remove(area.getId());
		}

	}


	public void eliminarAreaPorID(String id) {
		if(idsAreas.containsKey(id)){
			Area area = idsAreas.get(id);
			idsAreas.remove(area.getId());
			nombresAreas.remove(area.getNombre());
		}

	}


	public Area obtenerAreaPorNombre(String nombre) {
		if(nombresAreas.containsKey(nombre)){
			return nombresAreas.get(nombre);
		}
		return null;
	}


	public Area obtenerAreaPorID(String id) {
		if(idsAreas.containsKey(id)){
			return idsAreas.get(id);
		}
		return null;	
	}

	public boolean existeNombreArea(String nombre) {
		return nombresAreas.containsKey(nombre);
	}


	public boolean existeIDArea(String id) {
		return idsAreas.containsKey(id);
	}

	public Area nuevaArea(String nombre){
		Area area = new Area(nombre);
		idsAreas.put(area.getId(), area);
		nombresAreas.put(area.getNombre(), area);
		return area;
	}

	public void insertarArea(Area area){
		idsAreas.put(area.getId(), area);
		nombresAreas.put(area.getNombre(), area);
	}

	public SortedSet<String> listarNombresAreas() {
		SortedSet<String> ss = new TreeSet();
		ss.addAll(nombresAreas.keySet());

		return ss;
	}

	public SortedSet<Area> obtenerAreas() {
		SortedSet<Area> ss = new TreeSet();
		ss.addAll(nombresAreas.values());

		return ss;
	}

	public SortedSet<String> filtarNombresAreasPorUsuario(String idUsuario) {

		ManagerUsuarios mu = ManagerUsuarios.getInstance();
		Usuario u = mu.obtenerUsuarioPorID(idUsuario);

		// Grupos a los que pertenece el usuario

		ArrayList<Grupo> gruposUsuario = u.obtenerGrupos();

		if(u != null){

			SortedSet<Area> ss = new TreeSet();
			ss.addAll(nombresAreas.values());

			SortedSet<String> nombresAreas = new TreeSet();


			// Por cada area

			for(Area a: ss){

				// Seleccionar el mayor permiso
				int permiso = Area.SIN_ACCESO;

				// Busca el permiso para los grupos de usuario hasta que encuentra uno con acceso a lectura o escritura
				for(int i = 0; (i < gruposUsuario.size())&&(permiso == Area.SIN_ACCESO);i++ ){
					permiso = a.getPermisos().get(gruposUsuario.get(i).getID());	    			
				}

				if(permiso != Area.SIN_ACCESO){
					nombresAreas.add(a.getNombre());
				}


			}

			return nombresAreas;

		}

		return null;
	}


	public SortedSet<Area> filtarAreasPorUsuario(String idUsuario) {

		ManagerUsuarios mu = ManagerUsuarios.getInstance();
		Usuario u = mu.obtenerUsuarioPorID(idUsuario);

		// Grupos a los que pertenece el usuario

		ArrayList<Grupo> gruposUsuario = u.obtenerGrupos();

		if(u != null){

			SortedSet<Area> ss = new TreeSet();
			ss.addAll(nombresAreas.values());

			SortedSet<Area> areas = new TreeSet();


			// Por cada area

			for(Area a: ss){

				// Seleccionar el mayor permiso
				int permiso = Area.SIN_ACCESO;

				// Busca el permiso para los grupos de usuario hasta que encuentra uno con acceso a lectura o escritura
				for(int i = 0; (i < gruposUsuario.size())&&(permiso == Area.SIN_ACCESO);i++ ){
					permiso = a.getPermisos().get(gruposUsuario.get(i).getID());	    			
				}

				if(permiso != Area.SIN_ACCESO){
					areas.add(a);
				}


			}

			return areas;

		}

		return null;
	}


	public void guardarAreas(){
		AreasDAO dao = AreasDAO.getInstance();

		ArrayList<Area> areas = new ArrayList<Area>();
		areas.addAll(nombresAreas.values());

		try {
			dao.guardarAreas(areas);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}