package utils;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.StringTokenizer;

import modelo.Componente;
import modelo.Estructura;

// TODO: Auto-generated Javadoc
/**
 * The Class Diccionario.
 * Diccionario escanea todo el workspace en busca de estructuras y componentes.
 */
public class ManagerComponentes {

	/** INSTANCE. */
	private static ManagerComponentes INSTANCE = null;


	/** Componentes existentes en el workspace. */
	HashMap<String, Componente> componentes= new HashMap<String, Componente>();

	// Private constructor suppresses 
	/**
	 * Inicializa un objeto de tipo diccionario.
	 */
	private ManagerComponentes() {

	}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
	// otra prueba para evitar instanciacin mltiple 
	/**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
		if (INSTANCE == null) { 
			INSTANCE = new ManagerComponentes();
		}
	}

	/**
	 * Gets the single instance of Diccionario.
	 *
	 * @return single instance of Diccionario
	 */
	public static ManagerComponentes getInstance() {
		if (INSTANCE == null) createInstance();
		return INSTANCE;
	}


	/**
	 * Recupera componentes.
	 *
	 * @return el componentes
	 */
	public HashMap<String, Componente> getComponentes() {
		return componentes;
	}

	/**
	 * Establece componentes.
	 *
	 * @param componentes the componentes
	 */
	public void setComponentes(HashMap<String, Componente> componentes) {
		this.componentes = componentes;
	}

	/**
	 * Realiza un barrido por todo el espacio de trabajo e indexa todas las componentes y estructuras.
	 * Habilita la busqueda de una estructura o una componente por su id
	 */
	public void actualizar(){
		componentes.clear();
		ManagerEstructuras manager = ManagerEstructuras.getInstance();

		SortedSet<String> estructuras = manager.obtenerNombresEstructuras();

		Iterator it = estructuras.iterator();
		while(it.hasNext()){
			String nombre = (String) it.next();

			Estructura estructura = manager.getEstructurasNombre().get(nombre);

			estructura.getRaiz().insertarEnDiccionario();

		}

	}

}
