package utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import datos.EstructuraDAO;

import modelo.Estructura;

public class ManagerEstructuras {
	
    private static ManagerEstructuras INSTANCE = null;
    private Map<String, Estructura> estructurasNombre;
    private Map<String, Estructura> estructurasID;
    private Estructura copia;
    private int tipo;
    private EstructuraDAO dao;
    
    public static final int NUEVA = 0;
    
    public static final int MODIFICAR = 1;
    
    public static final int NINGUNO = 2;
    
    // Private constructor suppresses 
    private ManagerEstructuras() {
    	inicializar();
    }
 
    private void inicializar() {
		dao = EstructuraDAO.getInstance();
		ArrayList<Estructura> estructuras = dao.abrirEstructuras();
		
		estructurasNombre = new HashMap<String, Estructura>();
		estructurasID = new HashMap<String, Estructura>();
		
		for(Estructura e: estructuras){
			estructurasNombre.put(e.getNombre(), e);
			estructurasID.put(e.getId(), e);
		}
		
		copia = null;
		tipo = this.NINGUNO;
		
	}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new ManagerEstructuras();
        }
    }
 
    public static ManagerEstructuras getInstance() {
        createInstance();
        return INSTANCE;
    }
    
    public Estructura nuevaEstructura(){
    	
    	// Restaurar
    	restaurar();
    	
    	// Crear nombre por si ya existe
    	int i = 2;
    	String nombre = "NuevaEstructura";
    	while(estructurasNombre.containsKey(nombre)){
    		nombre = "NuevaEstructura"+i;
    		i++;
    	}
    	Estructura nueva = new Estructura(nombre);
    	nueva.getAalc();
    	
    	// Adicin
    	estructurasID.put(nueva.getId(), nueva);
    	estructurasNombre.put(nueva.getNombre(), nueva);
    	
    	tipo = this.NUEVA;
    	copia = (Estructura) nueva.clone();
    	
    	return nueva;
    }
    
    public Estructura abrirEstructura(String nombre){
    	
    	restaurar();
    	Estructura e = null;
    	e = estructurasNombre.get(nombre);
    	copia = (Estructura) e.clone();
    	tipo = this.MODIFICAR;
    	return e;
    }
    
    private void restaurar(){
    	if(copia != null){
    		
    		estructurasNombre.remove(copia.getNombre());
    		estructurasID.remove(copia.getId());
    		
    		if(tipo == this.MODIFICAR){
    			
    			estructurasNombre.put(copia.getNombre(), copia);
    			estructurasID.put(copia.getId(), copia);
    			
    		}
    	}
    }
    
    public void guardar(Estructura e){
    	
    	dao.guardarEstructura(e);
    	
    	// Previamente no estaba en los hashmaps
    	if(tipo == this.NUEVA){
    		estructurasNombre.put(e.getNombre(), e);
    		estructurasID.put(e.getId(), e);
    	}
    	if(tipo == this.MODIFICAR){
    		
    		if(!copia.getNombre().equals(e.getNombre())){
    			dao.eliminarEstructura(copia);
    			// Borrado con nombre antiguo
    			estructurasID.remove(copia.getId());
    			estructurasNombre.remove(copia.getNombre());
    			
    			// Insercin con nombre nuevo
    			estructurasNombre.put(e.getNombre(), e);
    			estructurasID.put(e.getId(), e);
    		}
    		copia = (Estructura) e.clone();
        	tipo = this.MODIFICAR;
    	}
    	
    	copia = (Estructura) e.clone();
    	tipo = this.MODIFICAR;
    }
    
    public void eliminar(Estructura e){
    	estructurasNombre.remove(e.getNombre());
    	estructurasID.remove(e.getId());
    	dao.eliminarEstructura(e);
    	copia = null;
    	tipo = this.NINGUNO;
    }
    
    public void eliminar(String id){
    	Estructura estructura = estructurasID.get(id);
    	if(estructura !=null){
    		estructurasNombre.remove(estructura.getNombre());
    		estructurasID.remove(estructura.getId());
    		dao.eliminarEstructura(estructura);
    	}
    }
    
    public boolean existeNombreEstructura(String nombre){
    	if(nombre.trim().toLowerCase().equals(copia.getNombre().trim().toLowerCase()))
    		return false;
    	else
    		return estructurasNombre.containsKey(nombre);
    }
    
    public boolean existeIDEstructura(String id){
    	if(id.trim().toLowerCase().equals(copia.getId().trim().toLowerCase()))
    		return false;
    	else
    		return estructurasNombre.containsKey(id);    	
    }
    
    public SortedSet<String> obtenerNombresEstructuras(){
    	
    	SortedSet<String> ss = new TreeSet();
    	ss.addAll(estructurasNombre.keySet());
    	
    	return ss;
    }

	public Map<String, Estructura> getEstructurasNombre() {
		return estructurasNombre;
	}

	public void setEstructurasNombre(Map<String, Estructura> estructurasNombre) {
		this.estructurasNombre = estructurasNombre;
	}

	public Map<String, Estructura> getEstructurasID() {
		return estructurasID;
	}

	public void setEstructurasID(Map<String, Estructura> estructurasID) {
		this.estructurasID = estructurasID;
	}

	public int getTipo() {
		return tipo;
	}

	public Estructura getCopia() {
		return copia;
	}
    
    
}
