package utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.SortedSet;
import java.util.TreeSet;

import datos.GruposDAO;

import modelo.Grupo;
import modelo.Grupo;
import modelo.Usuario;

public class ManagerGrupos{
	
    private static ManagerGrupos INSTANCE = null;
    // creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new ManagerGrupos();
        }
    }
    public static ManagerGrupos getInstance() {
        createInstance();
        return INSTANCE;
    }
    
    private HashMap<String, Grupo> nombresGrupo;
 
    private HashMap<String, Grupo> idsGrupo;
 
    // Private constructor suppresses 
    private ManagerGrupos() {
    	inicializar();
    }


	public void cargarGrupos() {
		GruposDAO dao = GruposDAO.getInstance();
		ManagerUsuarios mu = ManagerUsuarios.getInstance();
		
		ArrayList<Grupo> grupos = new ArrayList<Grupo>();
		try {
			grupos = dao.cargarGrupos();
			
			// Por cada grupo ledo
			for(Grupo g: grupos){
				
				// Se recupera el listado de ids de los usuarios que pertenecen al grupo
				ArrayList<String> idsUsuario = g.getIdsUsuarios();
				
				// Se crea el array list de usuarios
				g.setUsuarios(new ArrayList<Usuario>());
				
				System.out.println(idsUsuario.size());
				// Por cada uno de los ids
				for(String id: idsUsuario){
					
					// Se intenta recuperar el usuario desde el manager
					Usuario usuario = mu.obtenerUsuarioPorID(id);
					System.out.println(usuario.getNombreUsuario());
					// Si el usuario no ha sido borrado, se aade
					if(usuario != null){
						g.getUsuarios().add(usuario);
					}
				}
				
				// Insertar en HashMaps
				nombresGrupo.put(g.getNombre(), g);
				idsGrupo.put(g.getID(), g);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		

	}


	public void eliminarGrupoPorID(String id) {
		if(idsGrupo.containsKey(id)){
			Grupo grupo = idsGrupo.get(id);
			idsGrupo.remove(grupo.getID());
			nombresGrupo.remove(grupo.getNombre());
		}
		
	}


	public void eliminarGrupoPorNombre(String nombre) {
		if(nombresGrupo.containsKey(nombre)){
			Grupo grupo = nombresGrupo.get(nombre);
			nombresGrupo.remove(grupo.getNombre());
			idsGrupo.remove(grupo.getID());
		}
		
	}


	public boolean existeIDGrupo(String id) {
		return idsGrupo.containsKey(id);
	}


	public boolean existeNombreGrupo(String nombre) {
		return nombresGrupo.containsKey(nombre);
	}


	public void guardarGrupos() {
		GruposDAO dao = GruposDAO.getInstance();
		
		// Preparacin
		ArrayList<Grupo> grupos = new ArrayList<Grupo>();
		ArrayList<ArrayList<Usuario>> auxUsuarios = new ArrayList<ArrayList<Usuario>>();
		grupos.addAll(nombresGrupo.values());

		for(Grupo g: grupos){
			auxUsuarios.add(g.getUsuarios());
			g.setUsuarios(null);
		}
		
		try {
			dao.guardarGrupos(grupos);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		// Recuperacin
		
		for(int i = 0 ; i < grupos.size(); i++){
			grupos.get(i).setUsuarios(auxUsuarios.get(i));
		}
	}


	private void inicializar(){
		nombresGrupo = new HashMap<String, Grupo>();
		idsGrupo = new HashMap<String, Grupo>();
		
		// cargar grupos
		cargarGrupos();
		
	}


	public void insertarGrupo(Grupo grupo) {
		nombresGrupo.put(grupo.getNombre(), grupo);
		idsGrupo.put(grupo.getID(), grupo);
	}


	public Grupo nuevoGrupo() {
		// Crear
		Grupo user = new Grupo();
		
		
		return user;
		
	}


	public Grupo obtenerGrupoPorID(String id) {
		if(idsGrupo.containsKey(id)){
			return idsGrupo.get(id);
		}
		return null;	
	}
	
	public Grupo obtenerGrupoPorNombre(String nombre) {
		if(nombresGrupo.containsKey(nombre)){
			return nombresGrupo.get(nombre);
		}
		return null;
	}


	public SortedSet<String> listarGrupos() {
    	SortedSet<String> ss = new TreeSet();
    	ss.addAll(nombresGrupo.keySet());
    	
    	return ss;
	}
	
	public SortedSet<Grupo> obtenerGrupos() {
    	SortedSet<Grupo> ss = new TreeSet();
    	ss.addAll(nombresGrupo.values());
    	
    	return ss;
	}
    
}
