package utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.SortedSet;
import java.util.TreeSet;

import datos.UsuariosDAO;

import modelo.Usuario;

public class ManagerUsuarios implements IManagerUsuarios{
	
    private static ManagerUsuarios INSTANCE = null;
    private HashMap<String, Usuario> nombresUsuario;
    private HashMap<String, Usuario> idsUsuario;
    
    // Private constructor suppresses 
    private ManagerUsuarios() {
    	inicializar();
    }
 
    // creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new ManagerUsuarios();
        }
    }
 
    public static ManagerUsuarios getInstance() {
        createInstance();
        return INSTANCE;
    }

	@Override
	public boolean existeNombreUsuario(String nombre) {
		return nombresUsuario.containsKey(nombre);
	}

	@Override
	public boolean existeIDUsuario(String id) {
		return idsUsuario.containsKey(id);
	}

	@Override
	public Usuario nuevoUsuario() {
		// Crear
		Usuario user = new Usuario();
		
		// Aadir
		nombresUsuario.put(user.getNombreUsuario(), user);
		idsUsuario.put(user.getId(), user);
		
		return user;
		
	}

	@Override
	public void eliminarUsuarioPorNombre(String nombre) {
		if(nombresUsuario.containsKey(nombre)){
			Usuario usuario = nombresUsuario.get(nombre);
			nombresUsuario.remove(usuario.getNombreUsuario());
			idsUsuario.remove(usuario.getId());
		}
		
	}

	@Override
	public void eliminarUsuarioPorID(String id) {
		if(idsUsuario.containsKey(id)){
			Usuario usuario = idsUsuario.get(id);
			idsUsuario.remove(usuario.getId());
			nombresUsuario.remove(usuario.getNombreUsuario());
		}
		
	}

	@Override
	public Usuario obtenerUsuarioPorNombre(String nombre) {
		if(nombresUsuario.containsKey(nombre)){
			return nombresUsuario.get(nombre);
		}
		return null;
	}

	@Override
	public Usuario obtenerUsuarioPorID(String id) {
		if(idsUsuario.containsKey(id)){
			return idsUsuario.get(id);
		}
		return null;	
	}

	@Override
	public SortedSet<String> obtenerListaNombresUsuarios() {
    	SortedSet<String> ss = new TreeSet();
    	ss.addAll(nombresUsuario.keySet());
    	ss.remove("#SU#"); // No se muestra el superusuario
    	
    	return ss;
	}

	@Override
	public void guardarUsuarios() {
		UsuariosDAO dao = UsuariosDAO.getInstance();
		
		ArrayList<Usuario> usuarios = new ArrayList<Usuario>();
		usuarios.addAll(nombresUsuario.values());
		usuarios.remove(nombresUsuario.get("#SU#")); // el SU no se serializa
		
		try {
			dao.guardarUsuarios(usuarios);
		} catch (Exception e) {
		}
	}


	public void cargarUsuarios() {
		UsuariosDAO dao = UsuariosDAO.getInstance();
		
		ArrayList<Usuario> usuarios = new ArrayList<Usuario>();
		try {
			usuarios = dao.cargarUsuarios();
			for(Usuario u: usuarios){
				nombresUsuario.put(u.getNombreUsuario(), u);
				idsUsuario.put(u.getId(), u);
			}
		} catch (Exception e) {
		}
		

	}
	
	private void inicializar(){
		nombresUsuario = new HashMap<String, Usuario>();
		idsUsuario = new HashMap<String, Usuario>();
		
		// cargar usuarios
		cargarUsuarios();
		
		// Crear superUsuario
		Usuario superUsuario = new Usuario();
		superUsuario.setTipo(Usuario.TIPO_ADMINISTRADOR);
		superUsuario.setId("#SU#");
		superUsuario.setNombreUsuario("#SU#");
		superUsuario.setPassword("Dsic.2012");
		
		nombresUsuario.put(superUsuario.getNombreUsuario(), superUsuario);
		idsUsuario.put(superUsuario.getId(), superUsuario);
		
	}

	@Override
	public void insertarUsuario(Usuario usuario) {
		nombresUsuario.put(usuario.getNombreUsuario(), usuario);
		idsUsuario.put(usuario.getId(), usuario);
	}
    
}
