package utils.renders;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Visualizacion;
import modelo.Estructura;
import modelo.Linea;
import modelo.Presentacion;
import modelo.Selector;
import modelo.SelectorLinea;

public class EditorEstructurasCellRender extends DefaultTableCellRenderer{
	
	private Estructura estructura;
	private Presentacion presentacion;

	public Component getTableCellRendererComponent(JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
		
		Color color = Color.BLACK;
		Font fuente = new Font("Arial", 0, 10);
		
		int filaSeleccionada = table.getSelectedRow();
		int columnaSeleccionada = table.getSelectedColumn();
		
		// Componente mostrada en la tabla
		
		Component c = 
				super.getTableCellRendererComponent(table, value,
						isSelected, hasFocus,
						row, column);

		// Busqueda de la linea seleccionada
		
		Linea lineaSeleccionada = estructura.getAalc().get(row);
		
		// Comprobacin del tipo de lnea
		
		if(lineaSeleccionada.getClass() == SelectorLinea.class){
			
			SelectorLinea sl = (SelectorLinea) lineaSeleccionada;
			
			Selector selector = sl.getSelector();
			
			if(selector.getLineas().get(0).getId().equals(sl.getId())){
				color = new Color(presentacion.getAspectos().get(5).getColor());
				String tipo = presentacion.getAspectos().get(5).getFuente();
				int tam = presentacion.getAspectos().get(5).getTamanyo();
				fuente = new Font(tipo, 0, tam);
			}
			else{
				color = new Color(presentacion.getAspectos().get(6).getColor());
				String tipo = presentacion.getAspectos().get(6).getFuente();
				int tam = presentacion.getAspectos().get(6).getTamanyo();
				fuente = new Font(tipo, 0, tam);
			}
			
		}
		else if (lineaSeleccionada.getClass() == ComponenteLinea.class){
			
			// Bsqueda de la componente
			ComponenteLinea cl = (ComponenteLinea) lineaSeleccionada;
			
			Componente componente = cl.getComponente();
			
			
			// Comprobacin del tipo de componente
			
			if(componente.getClass() == ComponenteElemental.class){
				
				ComponenteElemental ce = (ComponenteElemental) componente;
				
				// Se colorea de una forma u otra dependiendo de si es derivada o no
				if(ce.getDerivada() == true){
					color = new Color(presentacion.getAspectos().get(4).getColor());
					String tipo = presentacion.getAspectos().get(4).getFuente();
					int tam = presentacion.getAspectos().get(4).getTamanyo();
					fuente = new Font(tipo, 0, tam);
				}
				else{
					color = new Color(presentacion.getAspectos().get(2).getColor());
					String tipo = presentacion.getAspectos().get(2).getFuente();
					int tam = presentacion.getAspectos().get(2).getTamanyo();
					fuente = new Font(tipo, 0, tam);
				}
				
			}
			else if(componente.getClass() == ComponenteCompuesta.class){
				
				ComponenteCompuesta cc = (ComponenteCompuesta) componente;
				
				// Comprobar de qu lnea se trata
				
				if(cl.getId().equals(cc.getCabecera().getId())){
					
					if(cc.getParent() == null){
						color = new Color(presentacion.getAspectos().get(0).getColor());
						String tipo = presentacion.getAspectos().get(0).getFuente();
						int tam = presentacion.getAspectos().get(0).getTamanyo();
						fuente = new Font(tipo, 0, tam);
					}
					else{ // parent != null
						if(cc.getDerivada() == false){
							color = new Color(presentacion.getAspectos().get(1).getColor());
							String tipo = presentacion.getAspectos().get(1).getFuente();
							int tam = presentacion.getAspectos().get(1).getTamanyo();
							fuente = new Font(tipo, 0, tam);
						}
						else{
							if(cc.getParent().getDerivada() == false){
								color = new Color(presentacion.getAspectos().get(3).getColor());
								String tipo = presentacion.getAspectos().get(3).getFuente();
								int tam = presentacion.getAspectos().get(3).getTamanyo();
								fuente = new Font(tipo, 0, tam);
							}
							else{
								color = new Color(presentacion.getAspectos().get(4).getColor());
								String tipo = presentacion.getAspectos().get(4).getFuente();
								int tam = presentacion.getAspectos().get(4).getTamanyo();
								fuente = new Font(tipo, 0, tam);
							}
						}
					}
					
				}
				else if((cl.getId().equals(cc.getSimbolosFin().getId()))||
						(cl.getId().equals(cc.getSimbolosInicio().getId()))){
					color = new Color(presentacion.getAspectos().get(7).getColor());
					String tipo = presentacion.getAspectos().get(7).getFuente();
					int tam = presentacion.getAspectos().get(7).getTamanyo();
					fuente = new Font(tipo, 0, tam);
				}
				
				
			}
			
		}
		
		if(filaSeleccionada == row && columnaSeleccionada == column)
			c.setForeground(Color.WHITE);
		else
			c.setForeground(color);
		c.setFont(fuente);
//
		return c;
	}

	public EditorEstructurasCellRender(Visualizacion visualizacion) {
		super();
		this.presentacion = visualizacion.getPresentacion();
		this.estructura = visualizacion.getEstructura();
//		System.out.println("Render:" + estructura.getNombre());
//		System.out.println("Render:" + estructura.getRaiz().getNombre());
	}
	
	

}
