package utils.tablemodels;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableModel;

import utils.ManagerComponentes;
import utils.ManagerEstructuras;

import controlador.EditorEstructuraDerivadaController;
import controlador.EditorEstructurasController;

import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.SelectorLinea;

// TODO: Auto-generated Javadoc
/**
 * ModeloEditorEstructuraDerivada utilizado por la tabla en la cual se muestra la estructura derivada.
 */
public class ModeloEditorEstructuraDerivada extends AbstractTableModel{


	/** Nombre de las columnas. */
	private ArrayList<String> columnNames = null;
	
	/** La estructura derivada. */
	private ComponenteCompuesta estructuraDerivada = null;
	
	/** Conjunto de lneas de la estructura derivada. */
	private ArrayList<Linea> lineas = null;
	
	/** Diccionario de estructuras y componentes. */
	private ManagerComponentes d;

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		return columnNames.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		return lineas.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		Linea lineaSeleccionada = lineas.get(rowIndex);
		
		//Diccionario d = Diccionario.getInstance();
		if(columnIndex == 0){
			return lineaSeleccionada.toString();
		}
		else{
			
			// Si se trata de un selector, las columnas 1 y dos son vacas
			ComponenteLinea cl;
			if(lineaSeleccionada.getClass() == SelectorLinea.class){
				return "";
			}
			else{
				cl = (ComponenteLinea) lineaSeleccionada;
			}
			
			
			// Si es una componente compleja, las lineas de simbolos no contienen estructura ni componente origen
			
			Componente c = cl.getComponente();
			if(c.getClass() == ComponenteCompuesta.class){
				ComponenteCompuesta cc = (ComponenteCompuesta) c;
				// Comprobacin de si la linea es la lnea de simbolos finales
//				System.out.println("Contenido Linea:" + lineaSeleccionada.toString());
//				System.out.println("ID:" + lineaSeleccionada.getId());
//				System.out.println("Inicio:" + cc.getSimbolosInicio().getId());
//				System.out.println("Fin:" + cc.getSimbolosFin().getId());
				if(cc.getSimbolosFin().getId().equals(lineaSeleccionada.getId()))
					return "";
				// Comprobacin de si la lnea es la lnea de smbolos iniciales
				if(cc.getSimbolosInicio().getId().equals(lineaSeleccionada.getId()))
					return "";
			}
			
			// En caso contrario, se devuelve el valor correspondiente
			
			if (columnIndex == 1 && rowIndex != 0){ // Columna de estructra origen
				// Obtener el id de la estructura origen
				String id =  cl.getComponente().getIdEstructuraOrigen();
//				System.out.println("Busco estructura: " + id);
				// Obtener la estructura origen del diccionario
				ManagerEstructuras manager = ManagerEstructuras.getInstance();
				Estructura e = manager.getEstructurasID().get(id);

				
				// La componente compleja base no tiene estructura origen ni componente origen
				if(e != null)
					return e.getNombre();
				else
					return "[Estructura eliminada]";
			}
			else if (columnIndex == 2 && rowIndex != 0){ // Columna de componente origen
				// Obtener el id de la componente origen
				String id =  cl.getComponente().getIdComponenteOrigen();
//				System.out.println("Busco componente: " + id);
				// Obtener la componente origen del diccionario
				c = d.getComponentes().get(id);
				// La componente compleja base no tiene estructura origen ni componente origen
				if(c != null)
					return c.getNombre();
				else
					return "[Componente eliminada]";
			}
			else
				return "";
			
		}
		
	}


	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int col) {
		return columnNames.get(col);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override
	public Class getColumnClass(int c) {
		return String.class;
	}



	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		// Solo es editable la primera columna, siempre y cuando la linea no sea de simbolos
//		if(columnIndex == 0){
//			
//			ComponenteLinea lineaSeleccionada = lineas.get(rowIndex);
//			Componente componenteSeleccionada = lineaSeleccionada.getComponente();
//			
//			if(componenteSeleccionada.getClass() == ComponenteElemental.class){
//				return true;
//			}
//			else{
//				ComponenteCompuesta ccSeleccionada = (ComponenteCompuesta) componenteSeleccionada;
//				if(ccSeleccionada.getSimbolosInicio().hashCode() == lineaSeleccionada.hashCode())
//					return false;
//				else if (ccSeleccionada.getSimbolosFin().hashCode() == lineaSeleccionada.hashCode())
//					return false;
//				else
//					return true;
//			}
//		}
//		else return false;
		return false;
	}

	/*
	 * Don't need to implement this method unless your table's
	 * data can change.
	 */
	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object value, int row, int col) {

		//EditorEstructuraDerivadaController controller = EditorEstructuraDerivadaController.getInstance();
		//controller.modificarNombreComponente(String.valueOf(value), row);
		//fireTableCellUpdated(row, col);
	}


	/**
	 * Inicializa un objeto de tipo modelo editor estructura derivada.
	 *
	 * @param estructuraDerivada the estructura derivada
	 */
	public ModeloEditorEstructuraDerivada(ComponenteCompuesta estructuraDerivada) {
		super();
		d = ManagerComponentes.getInstance();
		d.actualizar();
		columnNames = new ArrayList<String>();
		this.estructuraDerivada = estructuraDerivada;
		this.lineas = estructuraDerivada.getLineas();

		// Nombres de las columnas

		columnNames.add("Componente");
		columnNames.add("Estructura Origen");
		columnNames.add("Componente Origen");

	}
}
