package utils.tablemodels;

import java.nio.channels.Selector;
import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import controlador.EditorEstructurasController;

import modelo.Componente;
import modelo.ComponenteCompuesta;
import modelo.ComponenteElemental;
import modelo.ComponenteLinea;
import modelo.Estructura;
import modelo.Linea;
import modelo.Presentacion;
import modelo.Propiedad;
import modelo.SelectorLinea;

// TODO: Auto-generated Javadoc
/**
 * The Class ModeloEditorEstructuras.
 * Modelo de la tabla de edicin de estructuras. Recibe la presentacin y la estructura que se van a mostrar en la tabla.
 */
public class ModeloEditorEstructuras extends AbstractTableModel{


	/** Presentacin, cuyas propiedades visibles son las columnas de la tabla. */
	private Presentacion presentacion = null;
	
	/** Estructura, cuyas componentes son las filas de la tabla. */
	private Estructura estructura = null;
	
	/** Nmero de columnas. */
	private int numCols; 
	
	/** Nombres que figuran en la cabecera de las columnas. */
	private ArrayList<String> nombresColumnas;
	
	/** Conjunto de propiedades visibles que se muestran en la tabla. */
	private ArrayList<Propiedad> propiedadesVisibles;

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {	
		return numCols; // uno ms por la primera columna que contiene la componente
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		if(estructura != null)
			return estructura.getAalc().size();
		else return 0;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {



		if(columnIndex == 0) // La columna 0 no es una propiedad, sino la componente en s.
			return estructura.getAalc().get(rowIndex).toString();
		else{ // El resto de las columnas hacen referencia a alguna de las propiedades de la componente.

			Linea l = estructura.getAalc().get(rowIndex);
			ComponenteLinea cl;
			
			// Las lineas pertenecientes a un selector no tienen propiedades, y por tanto, son siempre vacas las casillas asociadas.
			if(l.getClass() == SelectorLinea.class){
				return "";
			}
			else{
				cl = (ComponenteLinea) l;
			}
			
			// Dicha componente se obtiene a partir de la fila de la tabla
			Componente c = cl.getComponente();


			// El valor de las celdas de propiedades de las lneas de smbolos permanecen vacas!!
			if(c.getClass() == ComponenteCompuesta.class){
				ComponenteCompuesta cc = (ComponenteCompuesta) c;
				if(cl.hashCode() == cc.getSimbolosInicio().hashCode())
					return "";
				if(cl.hashCode() == cc.getSimbolosFin().hashCode())
					return "";
			}

			
			Propiedad propiedad = propiedadesVisibles.get(columnIndex-1);
			for(int i = 0; i < c.getPropiedades().size(); i++){
				if(c.getPropiedades().get(i).getNombre().equals(propiedad.getNombre()))
					return c.getPropiedades().get(i).getValor();
			}

		}
		return "";
	}


	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int col) {
		return nombresColumnas.get(col);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override
	public Class getColumnClass(int c) {
		return getValueAt(0, c).getClass();
	}



	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {


		// Lnea seleccionada
		Linea l = estructura.getAalc().get(rowIndex);
		ComponenteLinea cl;
		
		// Si la linea pertenece a un selector, entonces no es editable
		if(l.getClass() == SelectorLinea.class)
			return false;
		else
			cl = (ComponenteLinea) l;
		

		/* Si es una componente elemental, es editable */
		if(cl.getComponente().getClass() == ComponenteElemental.class){
			return true;
		}
		else{
			/* Si es una componente compuesta, depende. La lnea de cabecera es editable,
			 * las que contienen nicamente smbolos no son editables.
			 */
			ComponenteCompuesta cc = (ComponenteCompuesta) cl.getComponente();
			if(cl.hashCode() == cc.getCabecera().hashCode()) return true;
			if(cl.hashCode() == cc.getSimbolosInicio().hashCode()) return false;
			if(cl.hashCode() == cc.getSimbolosFin().hashCode()) return false;
		}

		return true;

	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object value, int row, int col) {
		EditorEstructurasController controller = EditorEstructurasController.getInstance();
		controller.actualizarLinea(String.valueOf(value), row, col); // row = componente, getColumnName(col) = propiedad
		// data[row][col] = value;
		// fireTableCellUpdated(row, col);
	}

	/**
	 * Establece presentacion.
	 *
	 * @param presentacion el nuevo presentacion
	 */
	public void setPresentacion(Presentacion presentacion){
		this.presentacion = presentacion;
	}

	/**
	 * Establece estructura.
	 *
	 * @param estructura el nuevo estructura
	 */
	public void setEstructura(Estructura estructura){
		this.estructura = estructura;
	}

	/**
	 * Inicializa un objeto de tipo modelo editor estructuras.
	 *
	 * @param presentacion the presentacion
	 * @param estructura the estructura
	 */
	public ModeloEditorEstructuras(Presentacion presentacion,
			Estructura estructura) {
		super();
		this.presentacion = presentacion;
		this.estructura = estructura;
		this.propiedadesVisibles = new ArrayList<Propiedad>();
		
		// Generacin de las cabeceras y del nmero de columnas, para no tener que repetir el proceso constantemente
		nombresColumnas = new ArrayList<String>();
		nombresColumnas.add("Componente"); // Al menos contiene el nombre de la componente

		
		numCols = 1; // Al menos tiene el nombre de la componente, que no se considera una propiedad
		for(int i = 0; i < presentacion.getPropiedades().size(); i++){
			if(presentacion.getPropiedades().get(i).getVisible() == true){
				numCols++;
				nombresColumnas.add(presentacion.getPropiedades().get(i).getAlias());
				propiedadesVisibles.add(presentacion.getPropiedades().get(i));
			}
		}
	}

	/**
	 * Inicializa un objeto de tipo modelo editor estructuras.
	 */
	public ModeloEditorEstructuras() {
		super();
		numCols = 0;
	}

	public ArrayList<String> getNombresColumnas() {
		return nombresColumnas;
	}

	public void setNombresColumnas(ArrayList<String> nombresColumnas) {
		this.nombresColumnas = nombresColumnas;
	}



}
