package utils.tablemodels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.table.AbstractTableModel;

import controlador.EditorPresentacionesController;
import modelo.Propiedad;
import modelo.TipoDatos;

/**
 * ModeloEditorPresentacionesPropiedades es el modelo utilizado en la tabla en la que se permite
 * realizar modificaciones sobre propiedades.
 */
public class ModeloEditorPresentacionesPropiedades extends AbstractTableModel{

	/** Nombres de las columnas. */
	String columnas [] = {"Visible", "Nombre", "Seudnimo", "Editar"};
	
	/** Constante COLUMN_TYPES.
	 * Es utilizada para poder incrustar JButtons en la tabla */
	private static final Class<?>[] COLUMN_TYPES = new Class<?>[] {Boolean.class, String.class, String.class,  JButton.class};
	
	/** Conjunto de propiedades mostradas. */
	ArrayList<Propiedad> propiedades;

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		return columnas.length;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		return propiedades.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(final int rowIndex, int columnIndex) {
		Propiedad p = propiedades.get(rowIndex);
		
		switch(columnIndex){
		case 0:
			return p.getVisible();
		case 1:
			return p.getNombre();
		case 2:
			return p.getAlias();
		case 3: final JButton button = new JButton(columnas[columnIndex]);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				EditorPresentacionesController controller = EditorPresentacionesController.getInstance();
				controller.abrirEditorTipoDatos();
			}
		});
		return button;
			default:
				return "Error";
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int col) {
		return columnas[col];
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override public Class<?> getColumnClass(int columnIndex) {
		return COLUMN_TYPES[columnIndex];
	}



	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		if((columnIndex == 1)||(columnIndex == 3))
			return false;
		else return true;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object value, int row, int col) {
		EditorPresentacionesController controller = EditorPresentacionesController.getInstance();
		controller.propiedadesActualizarModelo(value, row, col);
		
	}
	
	/**
	 * Inicializa un objeto de tipo modelo editor presentaciones propiedades.
	 *
	 * @param propiedades the propiedades
	 */
	public ModeloEditorPresentacionesPropiedades(
			ArrayList<Propiedad> propiedades) {
		super();
		this.propiedades = propiedades;
	}

	

}
